#####################################################
## Bombable
## Brent Hugh, brent@brenthugh.com
bombableVersion = "4.3";
## 
## Copyright (C) 2009 - 2010  Brent Hugh  (brent@brenthugh.com)
## This file is licensed under the GPL license version 2 or later.
# 
# The Bombable module implements several different but interrelated functions
# that can be used by, for example, AI objects and scenery objects.  The 
# functions allow shooting and bombing of AI and multiplayer objects, explosions 
# and disabling of objects that are hit, and even multiplayer communications to 
# allow dogfighting:
#
# 1. BOMBABLE: Makes objects bombable.  They will detect hits, change livery according to damage, and finally start on fire and smoke when sufficiently damaged. There is also a function to change the livery colors according to damage level.
# 
# Bombable also works for multiplayer objects and allows multiplayer dogfighting.
# 
# In addition, it creates an explosion whenever the main aircraft hits the ground and crashes.
#
# 2. GROUND: Makes objects stay at ground level, adjusting pitch to match any slope they are on.  So, for instance, cars, trucks, or tanks can move and always stay on the ground, drive up slopes somewhat realistically, etc. Ships can be placed in any lake and will automatically find their correct altitude, etc.
#
# 3. LOCATE: Usually AI objects return to their initial start positions when FG re-inits (ie, file/reset). This function saves and maintains their previous position prior to the reset
# 
# 4. ATTACK: Makes AI Aircraft (and conceivable, other AI objects as well) swarm and attack the main aircraft
#
#TYPICAL USAGE--ADDING BOMBABILITY TO AN AI OR SCENERY MODEL
#
# Required: 
#  1. The Fire-Particles subdirectory (included in this package) 
#     must be installed in the FG/data/AI/Aircraft/Fire-Particles subdirectory
#  2. This file, bombable.nas, must be installed in the FG/data/Nasal 
#     subdirectory
# 
# To make any particular object "bombable", simply include code similar to that 
# included in the AI aircraft XML files in this distribution.
# 
# This approach generally should work with any AI objects or scenery objects.
# 
# You then typically create an AI scenario that includes these "bombable 
# objects" and then, to see and bomb the objects, load the scenario using
# the command line or fgrun when you start FlightGear. 
# 
# Or two (or more) players can choose aircraft set up for dogfighting (see readme in accompanying documentation) and dogfight via multiplayer.  Damage, smoke, fire, and explosions are all transmitted via multiplayer channels.
# 
# Notes:
#  - The object's node name can be found using cmdarg().getPath()
#  - You can make slight damage & high damage livery quite easily by modifying
#    any existing livery a model may have.  Note, however, that many objects
#    do not use liveries, but simply include color in the model itself. You
#    won't be able to change liveries on such models unless you alter the 
#    model (.ac file) to use external textures. 
#  
# 
# See file bombable-modding-aircraft-for-dogfighting.txt included in this 
# package for more details about adding bombable to aircraft or other objects.
# 
# See m1-abrams/m1.xml and other AI object XML files in this package for 
# sample implementations.
#
#
#AUTHORS
# 	Base code for M1 Abrams tank by Emmanuel BARANGER - David BASTIEN
#   Modded heavily by Brent HUGH to add re-location and ground altitude 
#   functionality, crashes for ships and aircraft, evasive manuevers when
#   under attack, multiplayer functionality, other functionality, 
#   and to abstract the code to a unit that can be included in most 
#   any AI or scenery object.
# 
#   Many snippets of code and examples of implemention were borrowed from other
#   FlightGear projects--thanks to all those contributors!
# 


#################################
# prints values to console only if 'debug' flag is set in props
var debprint = func {

   if (getprop(""~bomb_menu_pp~"debug")) {
    outputs="";
    foreach (var elem;arg) {
       if (elem != nil) outputs=outputs~elem;
    }; 
    print (outputs);
  }
}

###############################
# returns whole number + first dec. place; for formatted numbers 
var decplaces = func (a ){
  whole=int(a);
  dec=a-whole;
  frac=int(dec*10);
  return whole~"."~frac;


}

var mpprocesssendqueue = func {
 	
  #we do the settimer first so any runtime errors, etc., below, don't stop the
  # future instances of the timer from being started  
  settimer (func {mpprocesssendqueue()}, mpTimeDelaySend);
  
  if (!getprop(MP_share_pp)) return "";
  if (!getprop (MP_broadcast_exists_pp)) return "";

  if (size(mpsendqueue) > 0) {
      setprop (MP_message_pp, mpsendqueue[0]);
      mpsendqueue=subvec(mpsendqueue,1); 
  }

}

var mpsend = func (msg) {
   #adding systime to the end of the message ensures that each message is unique--otherwise messages that happen to be the same twice in a row will 
   # be ignored.  The system() at the end is ignored by the parser.
   #     
   if (!getprop(MP_share_pp)) return "";
   if (!getprop (MP_broadcast_exists_pp)) return "";
    
   append(mpsendqueue, msg~systime());
}

var mpreceive = func (mpMessageNode) {
  
  if (!getprop(MP_share_pp)) return "";
  if (!getprop (MP_broadcast_exists_pp)) return "";
  
  msg=mpMessageNode.getValue();    
  mpMessageNodeName=mpMessageNode.getPath();
  mpNodeName=string.replace (mpMessageNodeName, MP_message_pp, "");
  if (msg!=nil and msg !="") {
      debprint("Bombable: Message received from ", mpNodeName,": ", msg);
      parse_msg (mpNodeName, msg);
  }
      
  

}

################################################################################
#put_ballistic_model places a new model that is starts atanother AI model's
# position but moves independently, like a bullet, bomb, etc
# this is still not working/experimental

var put_ballistic_model = func(myNodeName="/ai/models/aircraft", path="AI/Aircraft/Fire-Particles/fast-particles.xml") {

  #debprint ("Bombable: setprop 149");
  # "environment" means the main aircraft
	if (myNodeName=="/environment" or myNodeName=="environment") myNodeName="";

  fgcommand("add-model", ballisticNode=props.Node.new({ 
          "path": path, 
          "latitude-deg-prop": myNodeName ~ "/position/latitude-deg", 
          "longitude-deg-prop":myNodeName ~ "/position/longitude-deg", 
          "elevation-ft-prop": myNodeName ~ "/position/altitude-ft", 
          "heading-deg-prop": myNodeName ~ "/orientation/true-heading-deg", 
          "pitch-deg-prop": myNodeName ~ "/orientation/pitch-deg", 
          "roll-deg-prop": myNodeName ~ "/orientation/roll-deg", 
  })); 
  
  
  print (ballisticNode.getName());
  print (ballisticNode.getName().getNode("property").getName());
  print (props.globals.getNode(ballisticNode.getNode("property").getValue()));
  print (ballisticNode.getNode("property").getValue());
  return props.globals.getNode(ballisticNode.getNode("property").getValue());

}


################################################################################
#put_tied_model places a new model that is tied to another AI model 
# (given by myNodeName) and will move with it in lon, lat, & alt

var put_tied_model = func(myNodeName="", path="AI/Aircraft/Fire-Particles/Fire-Particles.xml ") {

  #debprint ("Bombable: setprop 174"); 
  # "environment" means the main aircraft
	if (myNodeName=="/environment" or myNodeName=="environment") myNodeName="";

  fgcommand("add-model", fireNode=props.Node.new({ 
          "path": path, 
          "latitude-deg-prop": myNodeName ~ "/position/latitude-deg", 
          "longitude-deg-prop":myNodeName ~ "/position/longitude-deg", 
          "elevation-ft-prop": myNodeName ~ "/position/altitude-ft", 
          "heading-deg-prop": myNodeName ~ "/orientation/true-heading-deg", 
          "pitch-deg-prop": myNodeName ~ "/orientation/pitch-deg", 
          "roll-deg-prop": myNodeName ~ "/orientation/roll-deg", 
  })); 
  
  return props.globals.getNode(fireNode.getNode("property").getValue());

}


####################################################
#Delete a fire object (model) created earlier, turn off the fire triggers
#and unlink the fire from the parent object.
#This sets the object up so it can actually start on fire again if 
#wanted (or hit again by ballistics . . . though damage is already to max if 
#it has been on fire for a while, and damage is not re-set)
var deleteFire = func (myNodeName="/environment",fireNode="") {

    if (myNodeName=="") myNodeName="/environment";
    if (fireNode=="") {
        fireNodeName=getprop(""~myNodeName~"/bombable/fire-particles/fire-particles-model");
        fireNode = props.globals.getNode(fireNodeName);
    }
        
    #remove the fire node/model altogether
    if (fireNode!= nil) fireNode.remove();
    
    #turn off the object's fire trigger & unlink it from its fire model
    setprop(""~myNodeName~"/bombable/fire-particles/fire-burning", 0);
    setprop(""~myNodeName~"/bombable/fire-particles/fire-particles-model", "");

}

####################################################
#Check current speed & add damage due to excessive speed
#
var speedDamage = func {

    var damage_enabled=getprop (""~GF_damage_menu_pp~"damage_enabled");
    var warning_enabled=getprop (""~GF_damage_menu_pp~"warning_enabled");
    
    if (!  damage_enabled and ! warning_enabled ) return;
    
      var currSpeed_kt=getprop("/velocities/airspeed-kt");
      if (currSpeed_kt==0 or currSpeed_kt == nil) return; 
      
      var speedDamageThreshold_kt = getprop(""~vulnerabilities_pp~"airspeed_damage/damage_threshold_kt/");
      var speedWarningThreshold_kt = getprop(""~vulnerabilities_pp~"airspeed_damage/warning_threshold_kt/");
      
      if (speedDamageThreshold_kt==0 or speedDamageThreshold_kt==nil) speedDamageThreshold_kt=7000;
      if (speedWarningThreshold_kt==0 or speedWarningThreshold_kt==nil) speedWarningThreshold_kt=7000;
       
      var speedDamageMultiplier_PercentPerSecond = getprop(""~vulnerabilities_pp~"airspeed_damage/damage_multiplier_percentpersecond/");
      
      if (speedDamageMultiplier_PercentPerSecond==nil) speedDamageMultiplier_PercentPerSecond=1;
      
      #debprint ("Bombable: Speed checking ", currSpeed_kt, " ", speedDamageThreshold_kt, " ", speedWarningThreshold_kt," ", speedDamageMultiplier_PercentPerSecond);

      if (warning_enabled and currSpeed_kt > speedWarningThreshold_kt ) {
          var msg="Overspeed warning: "~ int( currSpeed_kt ) ~" kts";
          debprint(msg);
          #only put the message on the screen if damage is less than 100%
          # after that there is little point AND it will overwrite
          # any "you're out of commission" message                              
          if ( getprop("environment/bombable/attributes/damage") <1) 
            selfStatusPopupTip (msg, 5 );
      }

      if (damage_enabled and currSpeed_kt > speedDamageThreshold_kt ) {
          mp_add_damage( speedDamageMultiplier_PercentPerSecond * (currSpeed_kt -speedDamageThreshold_kt)/100,
          0, "speed", "" ~ int( currSpeed_kt ) ~ " kts (overspeed) damaged airframe!" );
        
      }
    

}


####################################################
#Check current accelerations & add damage due to excessive acceleration
#
var accelerationDamage = func {

    var damage_enabled=getprop (""~GF_damage_menu_pp~"damage_enabled");
    var warning_enabled=getprop (""~GF_damage_menu_pp~"warning_enabled");
    
    if (! damage_enabled and ! warning_enabled ) return;
    
    #debprint ("Bombable: Checking acceleration");
    #The acceleration nodes are updated once per second
    
    
      var currAccel_g=getprop("/accelerations/pilot-gdamped");
      if (currAccel_g==0 or currAccel_g == nil) return;
      
      if (currAccel_g>0 ) a="positive";
      else a="negative";
       
      currAccel_fg=math.abs(currAccel_g);
      
      
      var accelDamageThreshold_g = getprop(""~GF_damage_pp~"damage_threshold_g/"~a);
      var accelWarningThreshold_g = getprop(""~GF_damage_pp~"warning_threshold_g/"~a);
      
      if (accelDamageThreshold_g==0 or accelDamageThreshold_g==nil) accelDamageThreshold_g=50;
      if (accelWarningThreshold_g==0 or accelWarningThreshold_g==nil) accelWarningThreshold_g=10;
       
      var accelDamageMultiplier_PercentPerSecond = getprop(""~GF_damage_pp~"damage_multiplier_percentpersecond/"~a);
      
      if (accelDamageMultiplier_PercentPerSecond==nil) accelDamageMultiplier_PercentPerSecond=8;
      
      #debprint ("Bombable: Accel checking ", a, " ", currAccel_g, " ", accelDamageThreshold_g, " ", accelWarningThreshold_g," ", accelDamageMultiplier_PercentPerSecond);

      if (warning_enabled and currAccel_g > accelWarningThreshold_g ) {
          var msg="G-force warning: "~ int( currAccel_g ) ~"g";
          debprint(msg);
          #only put the message on the screen if damage is less than 100%
          # after that there is little point AND it will overwrite
          # any "you're out of commission" message                              
          if ( getprop("environment/bombable/attributes/damage") <1) 
            selfStatusPopupTip (msg, 5 );
      }

      if (damage_enabled and currAccel_g > accelDamageThreshold_g ) {
          mp_add_damage( accelDamageMultiplier_PercentPerSecond * (currAccel_g -accelDamageThreshold_g)/100,
          0, "gforce", "" ~ decplaces( currAccel_g*10 ) ~ "g force damaged airframe!" );
        
      }
}

#########################################################################
# timer for accel  & speed damage checks
# 
var damageCheck = func () {
  settimer (func {damageCheck (); }, damageCheckTime);
  #debprint ("Bombable: Checking damage.");
  accelerationDamage();
  speedDamage();

}

    #Notes on g-force:
  	# Max G tolerated by a person for periods of c. 1 sec or more 
  	# is about 30-50g even in fairly ideal circumstances.  So even if the aircraft
  	# survives your 30+g maneuver--you won't!  
  	# F22 has max g of 9.5 and Su-47 has max g of 9, so those numbers
  	# are probably pretty typical for modern fighter jets.    	
  	# See http://www.thespacereview.com/article/410/1  	
  	# http://www.airforce-technology.com/projects/s37/
  	# http://www.airforce-technology.com/projects/s37/  	
  	# Max G tolerated routinely by fighter or acrobatic
  	#   pilots etc seems to be about 9-12g
  	# 12-17g is tolerated long term, depending on the direction of the 
  	# acceleration  See http://en.wikipedia.org/wiki/G-force
  	# #max g for WWI era aircraft was 4-5g (best guess).  Repeat high gs do
    # weaken the structure.
    # In modern aircraft, F-16 has a maximum G of 9, F-18: 9.6, Mirage M.III/V: 7, A-4:6.
    # http://www.ww2aircraft.net/forum/technical/strongest-aircraft-7494-3.html :  
   # WW2 aircraft sometimes has higher max G, but it is interesting because pilot did not have G-suit, and trained pilots could not resist 5g for more than some seconds without G-suit.       
   # the strongest aircraft of WWII were the Italian monoplane fighters. They were built to withstand 8g normal load with 12g failure load. The same spec for German aircraft was 6g - 8.33g. For the late war P51s it was 5.33g
   # Spitfire VIII can pull about 9 and dive to about 570 mph before ripping apart while the F4U will only dive to about 560 mph and pull a similar load. 
   # at normal weight the designed limit load was 7.5 g positive and 3.5 g negative for the Corsair.
   #  FIAT G.50 had an ultimate factor of 14 g. According to Dottore Eng. Gianni Cattaneos Profile booklet on the Macchi C.202, it had an ultimate factor of no less than 15.8 g! That would make it virtually indestructible. Also the Hawker Tempest was strong with its 14+ G strength.
   # http://www.aviastar.org/air/japan/mitsubishi_a6m.php : 
   # Most Japanese fighters were designed to withstand a force of 7g. From 1932 all Japanese warplanes were required to meet a safety load factor of 1.8 so the limit for the A6M had to be 12.6g (1.8x7g).      
  

########################################################
#Set vulnerabilities for main aircraft
#  You can set vulnerabilities for any aircraft by 
#  simply creating a file 'vulnerabilities.nas',  
#  defining vulsObject as below, and including the line
#      bombable.setVulnerabilities (vulsObject);
#  
var setVulnerabilities = func (vulsObject=nil) {

  if (vulsObject==nil) { 
      vulsObject = {
        engineDamageVulnerability_percent : 3,
        fireVulnerability_percent:15,
        damageVulnerability:5,
        fireExtinguishSuccess_percentage:10,
        fireExtinguishMaxTime_seconds:100,
        fireDamageRate_percentpersecond : .4,
        explosiveMass_kg : 20000,
        airspeed_damage:{
          damage_threshold_kt: 6500,
          warning_threshold_kt: 6500,
          #1kt over the threshold for 1 second will add 1% damage:
          damage_multiplier_percentpersecond: 0.07
        },   
        gforce_damage: {
          damage_enabled: 0,
          warning_enabled: 1,
          damage_threshold_g: {positive:30, negative:20},
          warning_threshold_g: {positive:12, negative:7},
          #1g over the threshold for 1 second will add 8% damage:
          damage_multiplier_percentpersecond: {positive:8, negative:8 }
        },
        redout: {
          enabled: 1,
          parameters: {
            blackout_onset_g: 4,
            blackout_complete_g: 5,
            redout_onset_g: -2,
            redout_complete_g: -3
          }        
        } 
      }; 
      
              	
      
      #predefined values for a few aircraft we have set up for
      # dogfighting       
      var aircraftname=getprop("sim/aircraft");
      if (string.match(aircraftname,"A6M2" )){
      
        vulsObject = {                                  
          engineDamageVulnerability_percent : 6,
          fireVulnerability_percent:34,
          fireDamageRate_percentpersecond : .4,
          damageVulnerability:2.5,
          fireExtinguishSuccess_percentage:50,
          fireExtinguishMaxTime_seconds:50,
          explosiveMass_kg : 27772, 
          airspeed_damage:{
            damage_threshold_kt: 356, #http://en.wikipedia.org/wiki/A6M_Zero
            warning_threshold_kt: 325,
            #1 kt over the threshold for 1 second will add 1% damage:
            damage_multiplier_percentpersecond: 0.07
          },
          gforce_damage: {
              damage_enabled: 1,
              warning_enabled: 1,
              damage_threshold_g: {positive:12.6, negative:9},
              warning_threshold_g: {positive:7, negative:6},
              damage_multiplier_percentpersecond: {positive:12, negative:12 }
          },
          redout: {
            enabled: 1,
            parameters: {
              blackout_onset_g: 4, #no g-suit in WWI so really 6gs is pushing it
              blackout_complete_g: 6,
              redout_onset_g: -2,
              redout_complete_g: -3
            }        
          }  
        }
       
      } elsif ( string.match(aircraftname,"sopwithCamel*" ) ) {
      
        vulsObject = {
          engineDamageVulnerability_percent : 7,
          fireVulnerability_percent:15,
          fireDamageRate_percentpersecond : .5,
          damageVulnerability:3.5,
          fireExtinguishSuccess_percentage:23,
          fireExtinguishMaxTime_seconds:30,
          explosiveMass_kg : 735,
          airspeed_damage:{
            damage_threshold_kt: 185, #max speed, level flight is 100 kt, so this is a guess
            warning_threshold_kt: 175,
            #1 kt over the threshold for 1 second will add 1% damage:
            damage_multiplier_percentpersecond: 0.5
          },
          gforce_damage: {
              damage_enabled: 1,
              warning_enabled: 1,
              damage_threshold_g: {positive:4, negative:3},
              warning_threshold_g: {positive:3, negative:2.5},
              damage_multiplier_percentpersecond: {positive:12, negative:12 }

              },          
          redout: {
            enabled: 1,
            parameters: {
              blackout_onset_g: 3,
              blackout_complete_g: 4,
              redout_onset_g: -2,
              redout_complete_g: -3
            }        
          }  

          }   
      } elsif ( string.match(aircraftname, "spadvii" )  ) {
      
        vulsObject = {
          engineDamageVulnerability_percent : 3,
          fireVulnerability_percent:20,
          fireDamageRate_percentpersecond : .2,
          damageVulnerability:4,
          fireExtinguishSuccess_percentage:10,
          fireExtinguishMaxTime_seconds:100,
          explosiveMass_kg : 735,
          airspeed_damage:{
            damage_threshold_kt: 195, #max speed, level flight is 103 kt, so this is a guess based on that plus Spad's rep as able to hold together in "swift dives" better than most
            warning_threshold_kt: 185,
            #1 kt over the threshold for 1 second will add 1% damage:
            damage_multiplier_percentpersecond: 0.4
          },
          gforce_damage: {
              damage_enabled: 1,
              warning_enabled: 1,
              #"swift dive" capability must mean it is a bit more structurally 
              #   sound than camel/DR1
              damage_threshold_g: {positive:4.5, negative:3},
              warning_threshold_g: {positive:3, negative:2.5},
              damage_multiplier_percentpersecond: {positive:9, negative:9 }

              },          
          redout: {
            enabled: 1,
            parameters: {
              blackout_onset_g: 3,
              blackout_complete_g: 4,
              redout_onset_g: -2,
              redout_complete_g:-3
            }        
          }  
        }   
      } elsif ( string.match(aircraftname,"fkdr*" ) ) {
      
        vulsObject = {
          engineDamageVulnerability_percent : 3,
          fireVulnerability_percent:20,
          fireDamageRate_percentpersecond : .2,
          damageVulnerability:4,
          fireExtinguishSuccess_percentage:10,
          fireExtinguishMaxTime_seconds:100,
          explosiveMass_kg : 735,
          airspeed_damage:{
            damage_threshold_kt: 170, #max speed, level flight is 100 kt, so this is a guess based on that plus the DR1's reputation for wing damage at high speeds
            warning_threshold_kt: 155,
            #1 kt over the threshold for 1 second will add 1% damage:
            damage_multiplier_percentpersecond: 0.8
          },

          gforce_damage: {
              damage_enabled: 1,
              warning_enabled: 1,
              #wing breakage problems indicate weaker construction 
              #    than SPAD VII, Sopwith Camel              
              damage_threshold_g: {positive:3.8, negative:2.8},
              warning_threshold_g: {positive:3, negative:2.2},
              damage_multiplier_percentpersecond: {positive:14, negative:14 }
          },          
          redout: {
            enabled: 1,
            parameters: {
              blackout_onset_g: 4,
              blackout_complete_g: 5,
              redout_onset_g: -2,
              redout_complete_g:-3
            }        
          }  
        }
        
     }   
  }     

  props.globals.getNode(""~vulnerabilities_pp, 1).setValues(vulsObject);
  #put the redout properties in place, too; wait a couple of 
  # seconds so we aren't overwritten by the redout.nas subroutine:
  settimer ( func { 
  
     props.globals.getNode("/sim/rendering/redout/enabled", 1).setValue(vulsObject.redout.enabled);
     props.globals.getNode("/sim/rendering/redout/parameters/blackout-onset-g", 1).setValue(vulsObject.redout.parameters.blackout_onset_g);
     props.globals.getNode("/sim/rendering/redout/parameters/blackout-complete-g", 1).setValue(vulsObject.redout.parameters.blackout_complete_g);
     props.globals.getNode("/sim/rendering/redout/parameters/redout-onset-g", 1).setValue(vulsObject.redout.parameters.redout_onset_g);
     props.globals.getNode("/sim/rendering/redout/parameters/redout-complete-g", 1).setValue(vulsObject.redout.parameters.redout_complete_g);
     
     }, 3);


  #reset the vulnerabilities for the main object whenever FG
  # reinits.  
  # Important especially for setting redout/blackout, which otherwise
  # reverts to FG's defaults on reset.  
  # We need to do it here so that if some outside aircraft
  # calls setVulnerabilities with its own vulsObject
  # we will be able to use that here & reinit with that vulsObject
  #           
  vulsSet=getprop (""~vulnerabilities_pp~"vulnerabilities_set");
  if (vulsSet==nil) vulsSet=0; 
  if (vulsSet==0) { setlistener("/sim/signals/reinit", func {
    setVulnerabilities(vulsObject)} );
    
    #also set the default gforce/speed damage/warning enabled/disabled
    # but only on initial startup, not on reset    
    # lock means we don't write these changes to the menusettings save file  
  setprop (bomb_menu_save_lock, 1);
    if (getprop (GF_damage_menu_pp ~"/damage_enabled")==nil)     
      props.globals.getNode(GF_damage_menu_pp ~"/damage_enabled", 1).setValue(vulsObject.gforce_damage.damage_enabled);
    if (getprop (GF_damage_menu_pp ~"/warning_enabled")==nil)
      props.globals.getNode(GF_damage_menu_pp ~"/warning_enabled", 1).setValue(vulsObject.gforce_damage.warning_enabled);
    # lock means we don't write these changes to the menusettings save file  
  setprop (bomb_menu_save_lock, 0);
    
    
  }
  
  
  props.globals.getNode(""~vulnerabilities_pp~"vulnerabilities_set", 1).setValue(1);
  
}
  
  
####################################################
#start a fire in a given location & associated with a given object
#
#
#A fire is different than the smoke, contrails, and flares below because
#when the fire is burning it adds damage to the object and eventually
#destroys it.
#
#object is given by "myNodeName" and directory path to the model in "model"
#Also sets the fire trigger on the object itself so it knows it is on fire
#and saves the name of the fire (model) node so the object can find
#the fire (model) it is associated with to update it etc.
#Returns name of the node with the newly started fire object (model)
var startFire = func (myNodeName="", model="")
  {
  if (myNodeName=="") myNodeName="/environment";
  #if there is already a fire going/associated with this object
  # then we don't want to start another  
  var currFire= getprop(""~myNodeName~"/bombable/fire-particles/fire-particles-model");
  if ((currFire != nil) and (currFire != "")) {
    setprop(""~myNodeName~"/bombable/fire-particles/fire-burning", 1); 
    return currFire;
  }  
  
      
  if (model==nil or model=="") model="AI/Aircraft/Fire-Particles/fire-particles.xml";
  var fireNode=put_tied_model(myNodeName, model);
  
  # if (myNodeName!="") type=props.globals.getNode(myNodeName).getName();
  #else type="";
  #if (type=="multiplayer") mp_send_damage(myNodeName, 0);
  
  
  
  #var fire_node=geo.put_model("Models/Effects/Wildfire/wildfire.xml", lat, lon, alt*.3048);
  #print ("started fire! ", myNodeName);
  
  #turn off the fire after user-set amount of time (default 1800 seconds)
  var burnTime=getprop ("/environment/bombable/fire-particles/fire-burn-time");
  if (burnTime==0 or burnTime==nil) burnTime=1800;
  settimer (func {deleteFire(myNodeName,fireNode)}, burnTime);

  #name of this prop is "/models" + getname() + [ getindex() ]
  fireNodeName="/models/" ~ fireNode.getName() ~ "[" ~ fireNode.getIndex() ~ "]";
     

   setprop(""~myNodeName~"/bombable/fire-particles/fire-burning", 1);
   setprop(""~myNodeName~"/bombable/fire-particles/fire-particles-model", fireNodeName);
   
   return fireNodeName; #we usually start with the name & then use props.globals.getNode(nodeName) to get the node object if necessary.  
   #you can also use cmdarg().getPath() to get the full path from the node
   
}




####################################################
#Delete any of the various smoke, contrail, flare, etc. objects
#and unlink the fire from the smoke object.
#

var deleteSmoke = func (smokeType, myNodeName="/environment",fireNode="") {
    
    if (myNodeName=="") myNodeName="/environment";
    
    if (fireNode=="") {

        fireNodeName=getprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-particles-model");
        fireNode = props.globals.getNode(fireNodeName);
    }
    #remove the fire node/model altogether
    if (fireNode != nil) fireNode.remove();
    
    #turn off the object's fire trigger & unlink it from its fire model
    setprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-burning", 0);
    setprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-particles-model", "");

   #if (myNodeName!="") type=props.globals.getNode(myNodeName).getName();
   #else type="";
   #if (type=="multiplayer") mp_send_damage(myNodeName, 0);
   
  
  

}


####################################################
# Smoke is like a fire, but doesn't cause damage & can use one of 
# several different models to create different effects.
#
# smokeTypes are flare, smoketrail, pistonexhaust, contrail, damagedengine
#
# This func starts a flare in a given location & associated with a given object
#object is given by "myNodeName" and directory path to the model in "model"
#Also sets the fire burning flag on the object itself so it knows it is on fire
#and saves the name of the fire (model) node so the object can find
#the fire (model) it is associated with to update it etc.
#Returns name of the node with the newly started fire object (model)
var startSmoke = func (smokeType, myNodeName="/environment", model="")
  {
  if (myNodeName=="") myNodeName="/environment"; 
  #if there is already smoke of this type going/associated with this object
  # then we don't want to start another  
  var currFire= getprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-particles-model");
  if ((currFire != nil) and (currFire != "")) return currFire;
  
      
  if (model==nil or model=="") model="AI/Aircraft/Fire-Particles/"~smokeType~"-particles.xml";
  var fireNode=put_tied_model(myNodeName, model);
  
  
  #var fire_node=geo.put_model("Models/bombable/Wildfire/wildfire.xml", lat, lon, alt*.3048);                                             
  #debprint ("started fire! "~ myNodeName);
  
  #turn off the flare after user-set amount of time (default 1800 seconds)
  var burnTime=getprop (burntime1_pp~smokeType~burntime2_pp);
  if (burnTime==0 or burnTime==nil) burnTime=1800;
  #burnTime=-1 means leave it on indefinitely
  if (burnTime >= 0) settimer (func {deleteSmoke(smokeType, myNodeName,fireNode)}, burnTime);

  #name of this prop is "/models" + getname() + [ getindex() ]
  fireNodeName="/models/" ~ fireNode.getName() ~ "[" ~ fireNode.getIndex() ~ "]";
  
   # if (myNodeName!="") type=props.globals.getNode(myNodeName).getName();
   #else type="";
   #if (type=="multiplayer") mp_send_damage(myNodeName, 0);
   
  

   setprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-burning", 1);
   setprop(""~myNodeName~"/bombable/fire-particles/"~smokeType~"-particles-model", fireNodeName);
   
   return fireNodeName; #we usually pass around the name & then use props.globals.getNode(nodeName) to get the node object if necessary.  
}

############################################################

####################################################
#Add a new menu item to turn smoke on/off
#todo: need to integrate this into the menus rather than just 
#arbitrarily adding it to menu[97]
#
#This function adds the dialog object to an actual GUI menubar item

var init_fire_particles_dialog = func () {
                
        #todo: figure out how to position it in the center of the screen or somewhere better
        dialog.init(0,0);
        #make the GUI menu
        props.globals.getNode ("/sim/menubar/default/menu[97]/enabled", 1).setBoolValue(1);
        props.globals.getNode ("/sim/menubar/default/menu[97]/label", 1).setValue("Bombable");
        props.globals.getNode ("/sim/menubar/default/menu[97]/item/enabled", 1).setBoolValue(1);
        #Note: the label must be distinct from all other labels in the menubar
        #or you will get duplicate functionality with the other menu item
        #sharing the same label
        props.globals.getNode ("/sim/menubar/default/menu[97]/item/label", 1).setValue("Bombable Options"); #must be unique name from all others in the menubar or they both pop up together
        props.globals.getNode ("/sim/menubar/default/menu[97]/item/binding/command", 1).setValue("nasal");
        props.globals.getNode ("/sim/menubar/default/menu[97]/item/binding/script", 1).setValue("bombable.dialog.create()");
        
       #reinit makes the property changes to both the GUI & input become active
       #the delay is to avoid a segfault under dev version of FlightGear, 2010/09/07
       #This just a workaround, a real fix would like:  
       #  overwritting preferences.xml with a new one including a line like <menubar include="Dialogs/bombable.xml"/>'
       #Thx Citronnier for tracking this down  
       #settimer (func {fgcommand("reinit")}, 15);
       #As of FG 2.4.0, a straight "reinit" leads to FG crash or the dreaded NAN issue
       #at least with some aircraft.  Reinit/gui (as below) gets around this problem.
       fgcommand("reinit", props.Node.new({subsystem : "gui"}));
       
}
	
var targetStatusPopupTip = func (label, delay = 5, override = nil) {	

    var tmpl = props.Node.new({
            name : "PopTipTarget", modal : 0, layout : "hbox",
            y: 70,
            text : { label : label, padding : 6 }
    });
    if (override != nil) tmpl.setValues(override);
   
    popdown(tipArgTarget);
    fgcommand("dialog-new", tmpl);
    fgcommand("dialog-show", tipArgTarget);

    currTimerTarget += 1;
    var thisTimerTarget = currTimerTarget;

    # Final argument is a flag to use "real" time, not simulated time
    settimer(func { if(currTimerTarget == thisTimerTarget) { popdown(tipArgTarget) } }, delay, 1);
}

var selfStatusPopupTip = func (label, delay = 10, override = nil) {	

    var tmpl = props.Node.new({
            name : "PopTipSelf", modal : 0, layout : "hbox",
            y: 140,
            text : { label : label, padding : 6 }
    });
    if (override != nil) tmpl.setValues(override);
   
    popdown(tipArgSelf);
    fgcommand("dialog-new", tmpl);
    fgcommand("dialog-show", tipArgSelf);

    currTimerSelf += 1;
    var thisTimerSelf = currTimerSelf;

    # Final argument is a flag to use "real" time, not simulated time
    settimer(func { if(currTimerSelf == thisTimerSelf) { popdown(tipArgSelf) } }, delay, 1);
}

var popdown = func ( tipArg ) { fgcommand("dialog-close", tipArg); }
    


###############################################################################
## Set up Bombable Menu to turn on/off contrails etc.
## Based on the WildFire configuration dialog,
## which is partly based on Till Bush's multiplayer dialog
## to start, do dialog.init(30,30); dialog.create();

var CONFIG_DLG = 0;

var dialog = {
#################################################################
    init : func (x = nil, y = nil) {
        me.x = x;
        me.y = y;
        me.bg = [0, 0, 0, 0.3];    # background color
        me.fg = [[1.0, 1.0, 1.0, 1.0]]; 
        #
        # "private"
        me.title = "Bombable";
        me.basenode = props.globals.getNode("/environment/bombable/fire-particles");
        me.dialog = nil;
        me.namenode = props.Node.new({"dialog-name" : me.title });
        me.listeners = [];
    },
#################################################################
    create : func {
        if (me.dialog != nil)
            me.close();

        me.dialog = gui.Widget.new();
        me.dialog.set("name", me.title);
        if (me.x != nil)
            me.dialog.set("x", me.x);
        if (me.y != nil)
            me.dialog.set("y", me.y);

        me.dialog.set("layout", "vbox");
        me.dialog.set("default-padding", 0);
        var titlebar = me.dialog.addChild("group");
        titlebar.set("layout", "hbox");
        titlebar.addChild("empty").set("stretch", 1);
        titlebar.addChild("text").set("label", "Bombable Objects Settings");
        var w = titlebar.addChild("button");
        w.set("pref-width", 16);
        w.set("pref-height", 16);
        w.set("legend", "");
        w.set("default", 0);
        w.set("key", "esc");
        w.setBinding("nasal", "bombable.dialog.destroy(); ");
        w.setBinding("dialog-close");
        me.dialog.addChild("hrule");

        var content = me.dialog.addChild("group");
        content.set("layout", "vbox");
        content.set("halign", "center");
        content.set("default-padding", 5);

        #triggers (-trigger) are the overall on/off flag for that type of fire/smoke globally in Bombable
        # burning (-burning) is the local flag telling whether the type of 
        # fire/smoke is burning on that particle node/aircraft
        #                                                                        
        foreach (var b; [["Multiplayer enabled", MP_share_pp, "checkbox"],
                         ["Excessive acceleration/speed warnings", GF_damage_menu_pp~"warning_enabled", "checkbox"],   
                         ["Excessive acceleration/speed damages aircraft", GF_damage_menu_pp~"damauuuuuge_enabled", "checkbox"],
                         ["AI fighter aircraft maneuver and attack", bomb_menu_pp~"fighter-aircraft-attack", "checkbox"],
                         ["AI aircraft can shoot at you", bomb_menu_pp~"ai-aircraft-weapons-enabled", "checkbox"],        
                         ["Fires/Explosions enabled", trigger1_pp~"fire"~trigger2_pp, "checkbox"],
                         ["Jet Contrails enabled", trigger1_pp~"jetcontrail"~trigger2_pp, "checkbox"],
                         ["Smoke Trails enabled", trigger1_pp~"smoketrail"~trigger2_pp, "checkbox"],
                         ["Piston engine exhaust enabled", trigger1_pp~"pistonexhaust"~trigger2_pp, "checkbox"],
                         ["Damaged engine smoke enabled", trigger1_pp~"damagedengine"~trigger2_pp, "checkbox"],
                         ["Flares enabled", trigger1_pp~"flare"~trigger2_pp, "checkbox"],              
                         ["Realistic Mode (hits must be closer to the target to count, and do less damage; for the hard-core)", bomb_menu_pp~"realistic-impact-mode", "checkbox"],
                         ["Easy mode enabled (twice as easy to hit targets; AI aircraft do easier manuevers; may combine w/Super Easy)", bomb_menu_pp~"easy-mode", "checkbox"],
                         ["Super Easy Mode (3X as easy to hit targets; damaged tripled; AI aircraft do yet easier manuevers)", bomb_menu_pp~"super-easy-mode", "checkbox"],
                         #["Your Weapon Power", bomb_menu_pp~"main-weapon-power", "combo", 200, ["Super Realistic", "Normal", "Easy", "Super Easy", "Super-Duper Easy"]],
                         #["AI Weapon Effectiveness", bomb_menu_pp~"ai-weapon-power", "slider", 200, 0, 100 ],
                         
                         ["Print debug messages to console", bomb_menu_pp~"debug", "checkbox"]
                         ]
                         ) {
            var w = content.addChild(b[2]);
            w.node.setValues({"label"    : b[0],
                              "halign"   : "left",
                              "property" : b[1],
                              # "width"    : "200",
                              
                              });
                              
            if (b[2]=="select" or b[2]=="combo" or b[2]=="list" ){
               
              w.node.setValues({"pref-width"    : b[3],
                              });
              foreach (var r; b[4]) {
               var newentry = w.addChild("value");
               newentry.node.setValue(r);
              }
            }  
            
            if (b[2]=="slider"){
               
              w.node.setValues({"pref-width"    : b[3],
                                 "min" : b[4],
                                 "max" : b[5],
                              });
              
              
            }
            
            #lentry = combo.addChild("value");
            #lentry.prop().setValue(lname);              
            
            #}                              
            if (b[2]=="checkbox") w.setBinding("nasal",
                         "setprop(\"" ~ b[1] ~ "\"," ~
                         "!getprop(\"" ~ b[1] ~ "\"))");
        }
        me.dialog.addChild("hrule");
        
        var buttonBar = me.dialog.addChild("group");
        buttonBar.set("layout", "hbox");
        buttonBar.set("default-padding", 10);
     
        lsave = buttonBar.addChild("button");
        lsave.set("legend", "Save");
        lsave.set("default", 1);
        lsave.set("equal", 1);
        lsave.prop().getNode("binding[0]/command", 1).setValue("dialog-apply");
        lsave.prop().getNode("binding[1]/command", 1).setValue("dialog-close");

        # Load button.
        #var load = me.dialog.addChild("button");
        #load.node.setValues({"legend"    : "Load Wildfire log",
        #                      "halign"   : "center"});
        #load.setBinding("nasal",
        #                "wildfire.dialog.select_and_load()");

        fgcommand("dialog-new", me.dialog.prop());
        fgcommand("dialog-show", me.namenode);
    },
#################################################################
    close : func {
        fgcommand("dialog-close", me.namenode);
    },
#################################################################
    destroy : func {
        CONFIG_DLG = 0;
        me.close();
        foreach(var l; me.listeners)
            removelistener(l);
        delete(gui.dialog, "\"" ~ me.title ~ "\"");
    },
#################################################################
    show : func {
        if (!CONFIG_DLG) {
            CONFIG_DLG = 1;
            me.init();
            me.create();
        }
    },
#################################################################
    select_and_load : func {
        var selector = gui.FileSelector.new
            (func (n) { CAFire.load_event_log(n.getValue()); },
             "Load Wildfire log",                    # dialog title
             "Load",                                 # button text
             ["*.xml"],                              # pattern for files
             SAVEDIR,                                # start dir
             "fire_log.xml");                        # default file name
        selector.open();
    }


};   #oh yeah, that final ; is REALLy needed    
###############################################################################
  

####################################################
#return altitude (in feet) of given lat/lon

var elev = func (lat, lon) {

  var info = geodinfo(lat, lon);
  
  if (info != nil) {
      var alt=info[0];
      if (alt==nil) alt=0; 
      return alt/0.3048;
  } else  return 0;
  
}
			
###############################################################################
# MP messages
# directly based on similar functions in wildfire.nas
# 

var damage_msg = func (callsign, damageAdd, damageTotal, smoke=0, fire=0, messageType=1) {
  if (!getprop(MP_share_pp)) return;
  if (!getprop (MP_broadcast_exists_pp)) return;

  n=0;
  
  #bits.switch(n,1, checkRange(smoke,0,1,0 ));  # !! makes sure it's a boolean value
  #bits.switch(n,2, checkRange(fire,0,1,0 )); #can send up to 7 bits in a byte this way

  
  msg=sprintf ("%6s", callsign) ~ 
        Binary.encodeByte(messageType) ~ 
        Binary.encodeDouble(damageAdd) ~ 
        Binary.encodeDouble(damageTotal) ~ 
        Binary.encodeByte(smoke) ~
        Binary.encodeByte(fire);
  
  #too many messages overwhelm the system.  So we set a lock & only send messages
  # every 5 seconds or so (lockWaitTime); at the end we send the final message 
  # (which has the final damage percentage)  
  # of any that were skipped in the meanwhile    
  lockName=""~callsign~messageType;
  
  lock=props.globals.getNode("/environment/bombable/locks/"~lockName~"/lock", 1).getValue();
  if (lock==nil or lock=="") lock=0;
  
  masterLock=props.globals.getNode("/environment/bombable/locks/masterLock", 1).getValue();
  if (masterLock==nil or masterLock=="") masterLock=0;
  
  currTime=systime();
  
  #We can send 1 message per callsign & per message type, per lockWaitTime 
  # seconds.  It sets a lock to prevent messages being sent in the meanwhile.
  # It sets a timer to send a cumulative damage message at the end of the 
  # lock time to give a single update for damage in the meanwhile.
  # As a failsafe it also saves system time in the lock & any new
  # damage messages coming through after that lockWaitTime seconds are 
  # allowed to go forward.   
  # For vitally important messages (like master reset) we can set a masterLock
  # and no other messages can go out during that time.
  # This is abit of a kludge.   For real we should queue up messages &
  # send them out at a rate no faster than say 1/2 as fast as the rate
  # mpreceive checks for new messages.                        
  if ((currTime - masterLock > masterLockWaitTime) and (lock==nil or lock=="" or lock==0 or currTime - lock > lockWaitTime)) {
    
    lockNum=lockNum+1;
    props.globals.getNode("/environment/bombable/locks/"~lockName~"/lock", 1).setDoubleValue(currTime);
    settimer (func {
        lock=getprop ("/environment/bombable/locks/"~lockName~"/lock");
        msg2=getprop ("/environment/bombable/locks/"~lockName~"/msg");
        setprop ("/environment/bombable/locks/"~lockName~"/lock", 0);
        setprop ("/environment/bombable/locks/masterLock", 0);
        setprop ("/environment/bombable/locks/"~lockName~"/msg", "");
        if (msg2!=nil and msg2 != ""){
          mpsend(msg2);
          debprint ("Bombable: Sending delayed message "~msg);
        }
    }, lockWaitTime);
        
    return msg
    
        
  } else {
      setprop ("/environment/bombable/locks/"~lockName~"/msg", msg);
      return nil;
  }

}

###############################################################################
# reset_msg - part of MP messages
# 

var reset_msg = func () {
  if (!getprop(MP_share_pp)) return "";
  if (!getprop (MP_broadcast_exists_pp)) return "";

  n=0;
  
  #bits.switch(n,1, checkRange(smoke,0,1,0 ));  # !! makes sure it's a boolean value
  #bits.switch(n,2, checkRange(fire,0,1,0 )); #can send up to 7 bits in a byte this way
  
  callsign=getprop ("/sim/multiplay/callsign");
  props.globals.getNode("/environment/bombable/locks/masterLock", 1).setDoubleValue(systime());  
  #messageType=2 is the reset message
  return sprintf ("%6s", callsign) ~ 
      Binary.encodeByte(2);
}

var parse_msg = func (source, msg) {
  if (!getprop(MP_share_pp)) return;
  if (!getprop (MP_broadcast_exists_pp)) return;  
  debprint("Bombable: typeof source: ", typeof(source));    
  debprint ("Bombable: source: ", source, " msg: ",msg);   
  var ourcallsign=getprop ("/sim/multiplay/callsign"); 
  var p = 0;
  var msgcallsign = substr(msg, 0, 6);
  p = 6;
  
  var type = Binary.decodeByte(substr(msg, p));
  p += Binary.sizeOf["byte"];
  #debprint ("msgcallsign:"~ msgcallsign," type:"~ type);
 
  #not our callsign and type !=2, we ignore it & return (type=2 broadcasts to 
  #*everyone* that their callsign is re-setting, so we always listen to that)
  if ((sprintf ("%6s", msgcallsign) != sprintf ("%6s", ourcallsign)) and 
     type != 2 and type != 3 ) return;
 
 
  
  #damage message
  if (type == 1) {
    var damageAdd = Binary.decodeDouble(substr(msg, p));
    p += Binary.sizeOf["double"];
    var damageTotal = Binary.decodeDouble(substr(msg, p));
    p += Binary.sizeOf["double"];
    var smokeStart = Binary.decodeByte(substr(msg, p));
    p += Binary.sizeOf["byte"];
    var fireStart = Binary.decodeByte(substr(msg, p));
    p += Binary.sizeOf["byte"];
      
    debprint ("damageAdd:",damageAdd," damageTotal:",damageTotal," smoke:",smokeStart," fire:", fireStart);  
      
    mp_add_damage (damageAdd, damageTotal, "weapons", "Hit by weapons!" );
    
  }
  
  #reset message for callsign
  elsif (type == 2) {
  
   #ai_loc="/ai/models";
   #var mp_aircraft = props.globals.getNode(ai_loc).getChildren("multiplayer");
   #foreach (mp;mp_aircraft) { #mp is the node of a multiplayer AI aircraft
   
   #    mp_callsign=mp.getNode("callsign").getValue();
   #    mp_childname=mp.getName();
   #    mp_index=mp.getIndex();
   #    mp_name=ai_loc~"/"~mp_childname~"["~mp_index~"]";
   #    mp_path=cmdarg().getPath(mp);
   #    debprint ("Bombable: mp_path=" ~mp_path);
   
         mp_name=source;   
         debprint ("Bombable: Resetting fire/damage for - name: ", source, " callsign: "~string.trim(msgcallsign) );
       
   #    if (sprintf ("%6s", mp_callsign) == sprintf ("%6s", msgcallsign)) { 
       
         #blow away the locks for MP communication--shouldn't really
         # be needed--just a little belt & suspendors things here
         # to make sure that no old damage (prior to the reset) is sent
         # to the aircraft again after the reset, and that none of the 
         # locks are stuck.                            
         props.globals.getNode("/environment/bombable").removeChild("locks",0);
         deleteFire(source);
         deleteSmoke("damagedengine", source);
         setprop(""~source~"/bombable/attributes/damage", 0);
         msg= string.trim(msgcallsign)~" is resetting; damage reset to 0% for "~string.trim(msgcallsign);
         debprint ("Bombable: "~msg);
         targetStatusPopupTip (msg, 30);
         
         
    #  }
       
   #}
   


  }
  #update of callsign's current damage, smoke, fire situation
  elsif (type == 3) {

   #  ai_loc="/ai/models";
   #var mp_aircraft = props.globals.getNode(ai_loc).getChildren("multiplayer");
   #foreach (mp;mp_aircraft) { #mp is the node of a multiplayer AI aircraft
   
   #    mp_callsign=mp.getNode("callsign").getValue();
   #    mp_childname=mp.getName();
   #   mp_index=mp.getIndex();
   #    mp_name=ai_loc~"/"~mp_childname~"["~mp_index~"]";
   #    mp_path=cmdarg().getPath(mp);
       
       
   #    if (sprintf ("%6s", mp_callsign) == sprintf ("%6s", msgcallsign)) { 
          debprint ("Bombable: Updating fire/damage from - name: ", source ," callsign: "~string.trim(msgcallsign) );
          var damageAdd = Binary.decodeDouble(substr(msg, p));
          p += Binary.sizeOf["double"];
          var damageTotal = Binary.decodeDouble(substr(msg, p));
          p += Binary.sizeOf["double"];
          var smokeStart = Binary.decodeByte(substr(msg, p));
          p += Binary.sizeOf["byte"];
          var fireStart = Binary.decodeByte(substr(msg, p));
          p += Binary.sizeOf["byte"];
          
          mp_update_damage (source, damageAdd, damageTotal, smokeStart, fireStart, msgcallsign );      
          
      #}
       
   # }



  }  
  elsif (type == 4) {
    var pos    = Binary.decodeCoord(substr(msg, 6));
    var radius = Binary.decodeDouble(substr(msg, 36));
    resolve_foam_drop(pos, radius, 0, 0);
  }  
}

####################################################
#timer function, every 1.5 to 2.5 seconds, adds damage if on fire
var fire_loop = func(id, myNodeName="") {
  if (myNodeName=="") myNodeName="/environment";
  var loopid = getprop(""~myNodeName~"/bombable/fire-loopid"); 
	id == loopid or return;
  
  #Set the timer function here at the top 
  #   so if there is some runtime error in the code
  #   below the timer function still continues to run   
  var fireLoopUpdateTime_sec=2;
  # add rand() so that all objects dont do this function simultaneously 
	settimer(func { fire_loop(id, myNodeName); }, fireLoopUpdateTime_sec - 0.5 + rand());
  	
	node= props.globals.getNode(myNodeName);
  type=node.getName();
  
  if(getprop(""~myNodeName~"/bombable/fire-particles/fire-burning")) {
	  var myFireNodeName = getprop(""~myNodeName~"/bombable/fire-particles/fire-particles-model");
	

  	#we have one single property to control the startsize & endsize
  	#of ALL fire-particles active at one time. This is a bit fakey but saves on processor time.
  	#  The idea here is to change
  	# the values of the start/endsize randomly and fairly quickly so the
  	# various smoke columns don't all look like clones of each other
  	# each smoke column only puts out particles 2X per second so 
  	# if the sizes are changed more often than that they can affect only
  	# some of the smoke columns independently.            	
    var smokeEndsize = rand()*100+50;
  	setprop ("/environment/bombable/fire-particles/smoke-endsize", smokeEndsize);
  	
  	var smokeEndsize = rand()*125+60;
  	setprop ("/environment/bombable/fire-particles/smoke-endsize-large", smokeEndsize);
  	
  	var smokeEndsize = rand()*75+33;
  	setprop ("/environment/bombable/fire-particles/smoke-endsize-small", smokeEndsize);
  	
  	
    var smokeStartsize=rand()*10 + 5;
  	
    #occasionally make a really BIG explosion
  	if (rand()<.02/fireLoopUpdateTime_sec)  {
  	
        settimer (func {setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize); }, 0.1);#turn the big explosion off quickly so it only affects a few of the fires for a moment--they put out smoke particles 4X/second
        smokeStartsize = smokeStartsize * rand() * 15 + 100; #make the occasional really big explosion
    }
         
  	setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize);
  	setprop ("/environment/bombable/fire-particles/smoke-startsize-small", smokeStartsize * (rand()/2 + 0.5));
  	setprop ("/environment/bombable/fire-particles/smoke-startsize-large", smokeStartsize* (rand()*4 + 1));
  	
  	damageRate_percentpersecond = getprop (""~myNodeName~"/bombable/attributes/vulnerabilities/fireDamageRate_percentpersecond");
  	
  	if (damageRate_percentpersecond==nil) damageRate_percentpersecond=0;
  	if (damageRate_percentpersecond==0) damageRate_percentpersecond=0.1;
  	
  	# The object is burning, so we regularly add damage.	
  	# Have to do it differently if it is the main aircraft ("/environment")
  	if (myNodeName=="/environment") {
  	mp_add_damage( damageRate_percentpersecond/100 * fireLoopUpdateTime_sec,0, "fire", "Fire damage!" );
    } 
    #we don't add damage to multiplayer--we let the remote object do it & send 
    #  it back to us
    else {        	
  	 if (type != "multiplayer") add_damage( damageRate_percentpersecond/100 * fireLoopUpdateTime_sec , myNodeName,"nonweapon" );
    } 
	}
                                                                                      

}

##########################################################
#Puts myNodeName right at ground level, explodes, sets up 
#for full damage & on-ground trigger to make it stop real fast now
#
var hitground_stop_explode = func (myNodeName, alt) {
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var vuls = b.getNode("vulnerabilities").getValues();	
  

   startFire( myNodeName ); #if it wasn't on fire before it is now
   #debprint ("Bombable: setprop 4256");
   setprop (""~myNodeName~"/position/altitude-ft",  alt  );
   setprop (""~myNodeName~"/bombable/on-ground",  1 ); #this affects the slow-down system which is handled by add-damage, and will stop any forward movement very quickly
   add_damage(1, myNodeName, "nonweapon");  #and once we have buried ourselves in the ground we are surely dead; this also will stop any & all forward movement
         
   #check if this object has exploded already 
   exploded= getprop (""~myNodeName~"/bombable/exploded" );
   
   #if not, explode for ~3 seconds
   if ( exploded==nil or !exploded ){      
     #and we cover our tracks by making a really big explosion momentarily
     #if it hit the ground that hard it's justified, right?
     if (vuls.explosiveMass_kg<0) vuls.explosiveMass_kg=1;
     lnexpl= math.ln (vuls.explosiveMass_kg/10);      
     var smokeStartsize = rand()*lnexpl*20 + 30;
     setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize);
     setprop ("/environment/bombable/fire-particles/smoke-startsize-small", smokeStartsize * (rand()/2 + 0.5));
     setprop ("/environment/bombable/fire-particles/smoke-startsize-large", smokeStartsize * (rand()*4 + 1));
     
     #explode for, say, 3 seconds but then we're done for this object
     settimer (   func {setprop(""~myNodeName~"/bombable/exploded" , 1 ); }, 3 + rand() );
   }

}          

var addAltitude_ft = func  (myNodeName, altAdd_ft=40 , time=1 ) {
   
   var loopTime=0.033;
   
   elapsed = getprop(""~myNodeName~"/position/addAltitude_elapsed");
   if (elapsed==nil) elapsed=0;
   elapsed+= loopTime;
   #debprint ("Bombable: setprop 4257");
   setprop(""~myNodeName~"/position/addAltitude_elapsed", elapsed);
   
   
   
   currAlt_ft = getprop (""~myNodeName~"/position/altitude-ft");
   
   #if (elapsed==0) setprop (""~myNodeName~"/position/addAltitude_starting_alt_ft", currAlt_ft ) 
   #else var startAlt_ft=getprop (""~myNodeName~"/position/addAltitude_starting_alt_ft");  
   
   #debprint ("Bombable: setprop 1284");
   setprop (""~myNodeName~"/position/altitude-ft", currAlt_ft+altAdd_ft*loopTime/time);
   
   #debprint ("Bombable: setprop 1287");
   if (elapsed < time) settimer (func { addAltitude_ft (myNodeName,altAdd_ft,time)}, loopTime);
   
   else setprop(""~myNodeName~"/position/addAltitude_elapsed", 0 ); 

}
            

###################################################
#ground_loop
#timer function, every (0.5 to 1.5 * updateTime_s) seconds, to keep object at 
# ground level 
# or other specified altitude above/below ground level, and at a 
# reasonable-looking pitch. length_m & width_m are distances (in meters) 
# needed to clear the object and find open earth on either side and front/back.
# damagealtadd is the total amount to subtract from the normal the altitude above ground level (in meters) as 
# the object becomes damaged--say a sinking ship or tires flattening on a 
# vehicle.
# damageAltMaxRate is the max rate to allow the object to rise or sink
# as it becomes disabled
var ground_loop = func( id, myNodeName ) {
  var loopid = getprop(""~myNodeName~"/bombable/ground-loopid"); 
	id == loopid or return;
	
	
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var alts = b.getNode("altitudes").getValues();	
  var dims= b.getNode("dimensions").getValues();
  var vels= b.getNode("velocities").getValues(); 
  var updateTime_s=b.getNode("updateTime_s").getValue();

  #reset the timer loop first so we don't lose it entirely in case of a runtime
	# error or such	
	#    add rand() so that all objects don't do this function simultaneously
	settimer(func { ground_loop(id, myNodeName)}, (0.5 + rand())*updateTime_s );
  
  
  node= props.globals.getNode(myNodeName);
  type=node.getName();
  
  
  
  #bhugh, update altitude to keep moving objects at ground level the ground	
  var currAlt_ft= getprop(""~myNodeName~"/position/altitude-ft"); #where the object is, in feet
	var lat = getprop(""~myNodeName~"/position/latitude-deg");
  var lon = getprop(""~myNodeName~"/position/longitude-deg");
  var heading = getprop(""~myNodeName~"/orientation/true-heading-deg");
  var speed_kt = getprop(""~myNodeName~"/velocities/true-airspeed-kt");
  var damageValue = getprop(""~myNodeName~"/bombable/attributes/damage");
  var damageAltAddPrev_ft = getprop(""~myNodeName~"/bombable/attributes/damageAltAddCurrent_ft");
  if (damageAltAddPrev_ft == nil) damageAltAddPrev_ft=0;
  var damageAltAddCumulative_ft = getprop(""~myNodeName~"/bombable/attributes/damageAltAddCumulative_ft");
  if (damageAltAddCumulative_ft == nil) damageAltAddCumulative_ft=0;
  
  if (lat==nil) {
        lat=0; 
        debprint ("Bombable: Lat=NIL, ground_loop ", myNodeName);
  }
  if (lon==nil) {
        lon=0; 
        debprint ("Bombable: Lon=NIL, ground_loop ", myNodeName);
  }

  
  
  
     #calculate the altitude behind & ahead of the object, this determines the pitch angle and helps determine the overall ground level at this spot
     var GeoCoord = geo.Coord.new();
     GeoCoord.set_latlon(lat, lon);
     #debprint ("Bombable: GeoCoord.apply_course_distance(heading, dims.length_m/2); ",heading, " ", dims.length_m/2 );
     GeoCoord.apply_course_distance(heading, dims.length_m/2);    #frontreardist in meters
     toFrontAlt_ft=elev (GeoCoord.lat(), GeoCoord.lon()  ); #in feet
     GeoCoord.apply_course_distance(heading+180, dims.length_m);
     toRearAlt_ft=elev (GeoCoord.lat(), GeoCoord.lon()  ); #in feet    

    
     
     #debprint ("oFront:", toFrontAlt_ft);
     if (type=="aircraft") { 
           #poor man's look-ahead radar

           GeoCoord.apply_course_distance(heading, dims.length_m + speed_kt * 0.5144444 * 10 );
             
             var radarAheadAlt_ft=elev (GeoCoord.lat(), GeoCoord.lon()  ); #in feet
             
           
           #debprint ("result: "~ radarAheadAlt_ft);
             # our target altitude (for aircraft purposes) is the greater of the
             # altitude immediately in front and the altitude from our
             # poor man's lookahead radar. (ie, up to 2 min out at current 
             # speed).  If the terrain is rising we add 300 to our taret
             # alt just to be on the safe side.                 
             # But if we're crashing, we don't care about
             # what is ahead.                                                 
           lookingAheadAlt_ft=toFrontAlt_ft;
           #debprint ("tofrontalt ft: ", toFrontAlt_ft, " radaraheadalt ", radarAheadAlt_ft);
           # Use the radar lookahead altitude if
           #  1. higher than elevation fo current location
           #  2. not damaged
           #  3. we'll end up below our minimumAGL if we continue at 
           #  4. current altitude                                                       
           if ( radarAheadAlt_ft > toFrontAlt_ft and  (damageValue < 0.8 ) 
               and (radarAheadAlt_ft + alts.minimumAGL_ft > currAlt_ft )  )        
               lookingAheadAlt_ft = radarAheadAlt_ft;
               
           #if we're low to the ground we add this extra 300 ft just to be safe           
           if (currAlt_ft-radarAheadAlt_ft < 500) 
                  lookingAheadAlt_ft +=300;
     } else {
           lookingAheadAlt_ft =toFrontAlt_ft;
     }
     
     
     pitchangle1_deg = rad2degrees * math.atan2(toFrontAlt_ft - toRearAlt_ft, dims.length_ft ); #must convert this from radians to degrees, thus the 180/pi
     
     pitchangle_deg=pitchangle1_deg;

    #figure altitude of ground to left & right of object to determine roll &
    #to help in determining altitude  
     
     var GeoCoord2 = geo.Coord.new();
     GeoCoord2.set_latlon(lat, lon);
     GeoCoord2.apply_course_distance(heading+90, dims.width_m/2);  #sidedist in meters
     toRightAlt_ft=elev (GeoCoord2.lat(), GeoCoord2.lon()  ); #in feet
     GeoCoord2.apply_course_distance(heading-90, dims.width_m);
     toLeftAlt_ft=elev (GeoCoord2.lat(), GeoCoord2.lon()  ); #in feet
     rollangle_deg = 90 - rad2degrees * math.atan2(dims.width_ft, toLeftAlt_ft - toRightAlt_ft ); #must convert this from radians to degrees, thus the 180/pi
       
     #in CVS, taking the alt of an object's position actually finds the top
     #of that particular object.  So to find the alt of the actual landscape
     # we do ahead, behind, to left, to right of object & take the average.
     #luckily this also helps us calculate the pitch of the slope,
     #which we need to set pitch & roll,  so little is 
     #lost 
     alt_ft = (toFrontAlt_ft + toRearAlt_ft + toLeftAlt_ft + toRightAlt_ft) / 4; #in feet
  
  #The first time this is called just initialize all the altitudes and exit
  if ( alts.initialized != 1 ) {
     var initial_altitude_ft= getprop (""~myNodeName~"/position/altitude-ft");
     if (initial_altitude_ft<alt_ft + alts.wheelsOnGroundAGL_ft +  alts.minimumAGL_ft) {
          initial_altitude_ft = alt_ft + alts.wheelsOnGroundAGL_ft +  alts.minimumAGL_ft;
     }
     if (initial_altitude_ft>alt_ft + alts.wheelsOnGroundAGL_ft +  alts.maximumAGL_ft) { 
          initial_altitude_ft = alt_ft + alts.wheelsOnGroundAGL_ft +  alts.maximumAGL_ft;
     }
      
     target_alt_ft=initial_altitude_ft - alt_ft - alts.wheelsOnGroundAGL_ft;
     
     debprint ("Bombable: Initial Altitude: "~ initial_altitude_ft~ " target altitude: "~target_alt_ft~ " object="~ myNodeName);
     
     #debprint ("Bombable: setprop 1430");
     setprop (""~myNodeName~"/position/altitude-ft", initial_altitude_ft );
     setprop (""~myNodeName~"/controls/flight/target-alt",  initial_altitude_ft);
     #debprint ("1349 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
     #set target AGL here. This way the aircraft file can simply set altitude
     # limits for the craft while the scenario files sets the specific altitude
     # target for a specific plane in a specific scenario          
     b.getNode("altitudes/targetAGL_ft", 1).setDoubleValue(target_alt_ft);
     b.getNode("altitudes/targetAGL_m", 1).setDoubleValue(target_alt_ft*.3048);
     b.getNode("altitudes/initialized", 1).setBoolValue(1); 
     
     #set the timer & exit
     #settimer(func { ground_loop(id, myNodeName)}, (0.5 + rand())*updateTime_s );
     return;
  
  
  }
   
  var objectsLowestAllowedAlt_ft =alt_ft + alts.wheelsOnGroundAGL_ft + alts.crashedAGL_ft;
  #debprint (" objectsLowestAllowedAlt_ft=", objectsLowestAllowedAlt_ft);
  
  onGround= getprop (""~myNodeName~"/bombable/on-ground");
  if (onGround==nil) onGround=0;
  
  if (onGround){
     #go to object's resting altitude
     
     #debprint ("Bombable: setprop 1457");
     setprop (""~myNodeName~"/position/altitude-ft", objectsLowestAllowedAlt_ft );
     setprop (""~myNodeName~"/controls/flight/target-alt",  objectsLowestAllowedAlt_ft);
     #debprint ("1373 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
     
      #all to a complete stop
      setprop(""~myNodeName~"/controls/tgt-speed-kt", 0);
      setprop(""~myNodeName~"/controls/flight/target-spd", 0);    
      setprop(""~myNodeName~"/velocities/true-airspeed-kt", 0); 

      #we don't even really need the timer any more, since this object
      #is now exploded to heck & stopped also.  But just in case . . .

      #and that's it
      return;
  
  }
  #our target altitude for normal/undamaged forward movement
  #this isn't based on our current altitude but the results of our 
  # "lookahead radar" to provide the base altitude
  # However as the craft is more damaged it loses its ability to do this 
  # (see above: lookingAheadAlt just becomes the same as toFrontAlt)       
  targetAlt_ft = lookingAheadAlt_ft + alts.targetAGL_ft + alts.wheelsOnGroundAGL_ft;
  #debprint ("laa ", lookingAheadAlt_ft, " tagl ", alts.targetAGL_ft, " awog ", alts.wheelsOnGroundAGL_ft); 
  
  
  fullDamageAltAdd_ft =   (alt_ft+alts.crashedAGL_ft +alts.wheelsOnGroundAGL_ft) - currAlt_ft; #amount we should add to our current altitude when fully crashed.  This is to get the object to "full crashed position", ie, on the ground for an aircraft, fully sunk for a ship, etc.
  
          
  #now calculate how far to force the thing down if it is crashing/damaged
  if ( damageValue > 0.8  )  {
      damageAltAddMax_ft= (damageValue) * fullDamageAltAdd_ft; #max altitude amount to add to altitude to this object based on its current damage.
      #
      #Like fullDamageAltAdd & damageAltAddPrev this should always be zero
      #or negative as everything on earth falls or sinks when it loses
      #power. And assuming that simplifies calculations immensely.
      
      #The altitude the object should be at, based on damagealtAddMax & the 
      #ground level:
      shouldBeAlt=currAlt_ft + damageAltAddMax_ft;
      #debprint ("shouldBeAlt ", shouldBeAlt);
  
      #debprint ( "alt=", alt_ft, " currAlt_ft=",currAlt_ft, " fulldamagealtadd", fullDamageAltAdd_ft," damagealtaddmax", damageAltAddMax_ft, " damagevalue", damageValue," ", myNodeName );
      
      #debprint ("shouldBeAlt=oldalt+ alts.wheelsOnGroundAGL_ft + damageAltAddMax; ",
      #   shouldBeAlt, " ", oldalt, " ", alts.wheelsOnGroundAGL_ft, " ", damageAltAddMax  ); 
      
      #limit amount of sinkage to damageAltMaxRate in one hit/loop--otherwise it just goes down too fast, not realistic.  This is basically like the terminal
      # velocity for this type of object.    
      damageAltMaxPerCycle_ft = -abs(vels.damagedAltitudeChangeMaxRate_meterspersecond*updateTime_s/.3048);
      
      
      
      #move 10% more than previous or if no previous, start at 1% the max rate
      #making sure to move in the right direction! (using sgn of damageAltAdd)  
      if (damageAltAddPrev_ft != 0) damageAltAddCurrent_ft = -abs((1 + 0.1*updateTime_s) * damageAltAddPrev_ft);
      else damageAltAddCurrent_ft=- abs(0.01*damageAltMaxPerCycle_ft);  
      
      # make sure this is not bigger than the max rate, if so only change
      #it by the max amount allowed per cycle
      if ( abs( damageAltAddCurrent_ft ) > abs(damageAltMaxPerCycle_ft ) ) damageAltAddCurrent_ft =  damageAltMaxPerCycle_ft;  
      
      #Make sure we're not above the max allowed altitude change for this damage level; if so, cut it off
      if ( abs(damageAltAddCurrent_ft) > abs(damageAltAddMax_ft) ) {  
             damageAltAddCurrent_ft =  damageAltAddMax_ft;
      }         
    
                        
       #debprint ( " damageAltAddMax=", damageAltAddMax, " damageAltMaxRate=",  
       #debprint ("damageAltAddCurrent_ft ", damageAltAddCurrent_ft);
       
    
      
      
  } else {
      damageAltAddCurrent_ft=0;
  }



    
    #if the thing is basically as low as allowed by crashedAGL_m
    # we consider it "on the ground" (for an airplane) or 
    # completely sunk (for a ship) etc. 
    # If it is going there at any speed we consider it crashed
    # into the ground. When this 
    # property is set to true then the speed will slow quite dramatically.
    # This allows for example airplanes to continue forward movement
    # in the air but skid to a sudden halt when hitting the ground.
    #
    # alts.wheelsOnGroundAGL_ft + damageAltAdd = the altitude (AGL) the object should be at when 
    # finished crashing, sinking, etc.        
    # It's not that easy to determine if an object crashes--if an airplane
    # hits the ground it crashes but tanks etc are always on the ground        
    noPitch=0;
    if ( (damageValue > 0.8 and 
            ( currAlt_ft <= objectsLowestAllowedAlt_ft and speed_kt > 20 ) or 
            ( currAlt_ft <= objectsLowestAllowedAlt_ft-5)) 
          or (damageValue == 1 and currAlt_ft <= objectsLowestAllowedAlt_ft) )
    
         
         hitground_stop_explode(myNodeName, alt_ft);
    
    

   #if we are dropping faster than the current slope (typically because
   # we are an aircraft diving to the ground because of damage) we 
   # make the pitch match that angle, even if it more acute than the
   # regular slope of the underlying ground          
   if ( (damageValue > 0.8 ) ) {
       
       #this goes off every updateTime_s seconds approximately so the horizontal motion in one second is: (1.68780986 converts knots to ft per second)
       horizontalDistance_ft=speed_kt * 1.68780986 * updateTime_s;
       pitchangle2_deg = rad2degrees * math.atan2(damageAltAddCurrent_ft, horizontalDistance_ft );
       if (damageAltAddCurrent_ft ==0 and horizontalDistance_ft >0) pitchangle2_deg=0; #forward
       if (horizontalDistance_ft == 0 and damageAltAddCurrent_ft<0 ) pitchangle2_deg=-90; #straight down
       #Straight up won't happen here because we are (on purpose) forcing
       #the object down as we crash.  So we ignore the case.
       #if (horizontalDistance==0 and deltaAlt>0 ) pitchangle2=90; straight up
       
       #if no movement at all then we leave the pitch alone
       #if movement is less than 0.4 feet for pitch purposes we consider it
       #no movement at all--just a bit of wiggling
       noPitch=  ( (abs(damageAltAddCurrent_ft)< 0.5) and ( abs(horizontalDistance_ft) < 0.5));
       if (noPitch) pitchangle2_deg=0;   
       
       if (abs(pitchangle2_deg) > abs(pitchangle1_deg)) pitchangle_deg=pitchangle2_deg; 


       #vert-speed prob   
       setprop (""~myNodeName~"/velocities/vertical-speed-fps",damageAltAddCurrent_ft * updateTime_s ); #since we do this updateTime_ss per second the vertical speed in FPS (ideally) exactly equals damageAltAddCurrent*updateTime_s

       #debprint ("speed-based pitchangle=", pitchangle2_deg, " hor=", horizontalDistance_ft, " dalt=", deltaAlt_ft);
     }



  #don't set pitch/roll for aircraft
  #debprint ("Bombable: setprop 4261");
  if (type != "aircraft") {
    debprint ("Bombable: Setting roll-deg for ", myNodeName , " to ", rollangle_deg, " 1610");
    setprop (""~myNodeName~"/orientation/roll-deg", rollangle_deg );
    setprop (""~myNodeName~"/controls/flight/target-roll", rollangle_deg);
    if (!noPitch) {
      setprop (""~myNodeName~"/orientation/pitch-deg", pitchangle_deg );
      setprop (""~myNodeName~"/controls/flight/target-pitch", pitchangle_deg);
    }
  
  }
  
  #if (crashing) debprint ("Crashing! damageAltAdd_deg and damageAltAddCurrent_deg, 
  #damageAltAddPrev & damageAltAddMax, damageAltMaxRate, damageAltMaxPerCycle ",
  #damageAltAdd_ft, " ",damageAltAddCurrent," ", damageAltAddPrev_ft, " ", 
  #damageAltAddMax_ft, " ", myNodeName, " ", damageAltMaxRate, " ", 
  #damageAltMaxPerCycle_ft, " ", updateTime_s );
                                                                      
  
  
  #setprop (""~myNodeName~"/velocities/vertical-speed-fps", verticalspeed);
   
  #set the target alt.  This mainly works for aircraft.         
  #when crashing, only do this if the new target alt is less then the current
  #target al
  #newTgtAlt_ft = targetAlt_ft + damageAltAddCurrent_ft;
  newTgtAlt_ft = targetAlt_ft;  
  currTgtAlt_ft = getprop (""~myNodeName~"/controls/flight/target-alt");#in ft
  if (currTgtAlt_ft==nil) currTgtAlt_ft=0;
  
  if ( (damageValue <= 0.8 ) or newTgtAlt_ft  < currTgtAlt_ft ) {
      #debprint ("Bombable: setprop 1625"); 
      setprop (""~myNodeName~"/controls/flight/target-alt", (newTgtAlt_ft ));   #target altitude--this is 10 feet or so in front of us for a ship or up to 1 minute in front for an aircraft
      #debprint ("1536 ", newTgtAlt_ft);
      #debprint ("1536 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
  }
  
  #if going uphill base the altitude on the front of the vehicle (targetAlt).
  #This keeps the vehicle from sinking into the   
  #hillside when climbing.  This is a bit of a kludge that is simple/fast
  #because we have already calculated targetAlt in calculating the pitch.
  #To make this precise, calculate the correct position forward
  #based on the current speed of the current object and updateTime_s
  #and find the altitude of that spot.
  #For aircraft the targetAlt is the altitude 1 minute out IF that is higher
  #than the ground level.
  if (lookingAheadAlt_ft > alt_ft ) useAlt_ft = lookingAheadAlt_ft; else useAlt_ft=alt_ft;
  calcAlt_ft = (useAlt_ft + alts.wheelsOnGroundAGL_ft + alts.targetAGL_ft +  damageAltAddCumulative_ft + damageAltAddCurrent_ft);
  
   
   
  # calcAlt_ft=where the object should be, in feet                        
  #if it is an aircraft we try to control strictly via setting the target
  # altitude etc. (above).  If a ship etc. we just have to force it to that altitude (below).  However if an aircraft gets too close to the ground
  #the AI aircraft controls just won't react quickly enough so we "rescue"
  #it by simply moving it up a bit (see below).  
  #debprint ("type=", type);
  if (type != "aircraft") {
    #debprint ("Bombable: setprop 1652");
    setprop (""~myNodeName~"/position/altitude-ft", (calcAlt_ft) ); # feet
   }
   # for an aircraft, if it is within feet of the ground (and not forced 
   # there because of damage etc.) then we "rescue" it be putting it 25 feet 
   # above ground again.
   elsif (  currAlt_ft < toFrontAlt_ft + 75 and !(damageValue > 0.8 ) )   { 
         #debprint ("correcting!", myNodeName, " ", toFrontAlt_ft, " ", currAlt_ft, " ", currAlt_ft-toFrontAlt_ft, " ", toFrontAlt_ft+40, " ", currAlt_ft+20 );
         #set the pitch to try to make it look like we're climbing real
         #fast here, not just making an emergency correction . . .
         #for some reason the pitch is always aiming down when we
         #need to make a correction up, using pitchangle1. 
         #Kluge, we just always put pitch @30 degrees
         
         #vert-speed prob 
         setprop (""~myNodeName~"/orientation/pitch-deg", 30 );
         #debprint ("Bombable: setprop 1668");
         setprop (""~myNodeName~"/controls/flight/target-pitch", 30);
            
         if (currAlt_ft < toFrontAlt_ft + 25 ) { #dramatic correction
            debprint ("Bombable: Avoiding ground collision, "~ myNodeName);
            
            #addAltitude_ft is experimental/not quite working yet
            #addAltitude_ft (myNodeName, toFrontAlt_ft + 40-currAlt_ft, updateTime_s  );
            #debprint ("Bombable: setprop 1676");
            setprop (""~myNodeName~"/position/altitude-ft", toFrontAlt_ft + 40 );
            setprop (""~myNodeName~"/controls/flight/target-alt",  toFrontAlt_ft + 40);
            
            #vert-speed prob
            # 250 fps is achieved by a Zero in a normal barrel roll, so 300 fps is 
            # a pretty extreme/edge of reality maneuver for most aircraft
            setprop (""~myNodeName~"/velocities/vertical-speed-fps",  300);
        
            
           #debprint ("1579 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
             
          } else {   #more minor correction
            #debprint ("Correcting course to avoid ground, ", myNodeName);
            #setprop (""~myNodeName~"/position/altitude-ft", currAlt_ft + 20 );
            
            #addAltitude_ft is experimental/not quite working yet
            #addAltitude_ft (myNodeName, toFrontAlt_ft + 20 - currAlt_ft, updateTime_s  );
            #debprint ("Bombable: setprop 1691");            
            setprop (""~myNodeName~"/controls/flight/target-alt",  currAlt_ft + 20);
            
            #vert-speed prob
            # 250 fps is achieved by a Zero in a normal barrel roll, so 70 fps is
            # a very hard pull back on the stick in most aircraft, but not utterly 
            # impossible-looking.
            setprop (""~myNodeName~"/velocities/vertical-speed-fps",  70);
            #debprint ("1585 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
             
          } 

   } 
   
   if ( type == "aircraft" and  (damageValue > 0.8 )) {
     #debprint ("Crashing! damageAltAdd & damageAltAddCurrent, damageAltAddPrev & damageAltAddMax, damageAltMaxRate, damageAltMaxPerCycle ",damageAltAdd, " ",damageAltAddCurrent," ", damageAltAddPrev, " ", damageAltAddMax, " ", myNodeName, " ", damageAltMaxRate, " ", damageAltMaxPerCycle, " ", updateTime_s );
     #if crashing we just force it to the right altitude, even if an aircraft
     #but we move it a maximum of damageAlMaxRate
     #if it's an airplane & it's crashing, we take it down as far as 
     #needed OR by the maximum allowed rate.
     
     #when it hits this altitude it is (or most very soon become)
     #completely kaput
     #For many objects, depending on how the model is set up, this 
     #may be somewhat higher or lower than actual ground level
     
     
     if ( damageAltMaxPerCycle_ft <  damageAltAddCurrent_ft )  {
         #setprop (""~myNodeName~"/position/altitude-ft", (objectsLowestAllowedAlt_ft + alts.wheelsOnGroundAGL_ft + damageAltAddCurrent) ); # feet
         #setprop (""~myNodeName~"/position/altitude-ft", (currAlt_ft + damageAltAddCurrent  - updateTime_s) ); # feet   
         #nice
         #debprint ("damageAltAddCurrent=", damageAltAddCurrent);
          #debprint ("Bombable: setprop 1720");
          setprop (""~myNodeName~"/controls/flight/target-alt",  currAlt_ft -500);
          #debprint ("1610 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
          setprop (""~myNodeName~"/controls/flight/target-pitch", -45);
          
          #vert-speed prob
          var orientPitch=getprop (""~myNodeName~"/orientation/pitch-deg");
          if ( orientPitch > -10) setprop (""~myNodeName~"/orientation/pitch-deg", orientPitch-1);
          
          
     } elsif (currAlt_ft + damageAltMaxPerCycle_ft  > objectsLowestAllowedAlt_ft ) { #put it down by the max allowed rate 
     
        #setprop (""~myNodeName~"/position/altitude-ft", (currAlt_ft + damageAltMaxPerCycle_ft ) );
          
        #setprop (""~myNodeName~"/position/altitude-ft", (currAlt_ft + damageAltAddCurrent_ft - updateTime_s*2 ) );
         #debprint ("damageAltAddCurrent=", damageAltAddCurrent);
         #not that nice
         #debprint ("Bombable: setprop 1737");
         setprop (""~myNodeName~"/controls/flight/target-alt",  currAlt_ft -10000);
         #debprint ("1625 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
         setprop (""~myNodeName~"/controls/flight/target-pitch", -70);
         
         var orientPitch_deg=getprop (""~myNodeName~"/orientation/pitch-deg");
        
        #vert-speed prob
        if (orientPitch_deg > -20) setprop (""~myNodeName~"/orientation/pitch-deg", orientPitch_deg - 1 );
          
         dodge (myNodeName); #it will roll/dodge as though under fire
        
         
     } else { #closer to the ground than MaxPerCycle so, just put it right on the ground.  Oh yeah, also explode etc.
     
        hitground_stop_explode(myNodeName, objectsLowestAllowedAlt_ft);
        
     
     }   
     #somehow the aircraft are getting below ground sometimes
     #sometimes it's just because they hit into a mountain or something
     #else in the way.
     #kludgy fix, just check for it & put them back on the surface 
     #if necessary.  And expldode & stuff.
     aircraftAlt_ft = getprop (""~myNodeName~"/position/altitude-ft" );
     if ( aircraftAlt_ft < alt_ft ) { 

         hitground_stop_explode(myNodeName, objectsLowestAllowedAlt_ft );
      
     }
     
   } 
   
   #whatever else, we don't let objects go below their lowest allowed altitude
   #Maybe they are skidding along on teh ground, but they are not allowed
   # to skid along UNDER the ground . . .       
   if (currAlt_ft < objectsLowestAllowedAlt_ft)
      {
      #debprint ("Bombable: setprop 1775");
      setprop(""~myNodeName~"/position/altitude-ft", objectsLowestAllowedAlt_ft); #where the object is, in feet 
      }
   setprop(""~myNodeName~"/bombable/attributes/damageAltAddCurrent_ft", damageAltAddCurrent_ft);
      setprop(""~myNodeName~"/bombable/attributes/damageAltAddCumulative_ft", damageAltAddCumulative_ft + damageAltAddCurrent_ft);
   
  #debprint ("alt = ", alt, " currAlt_ft = ", currAlt_ft, " deltaAlt= ", deltaAlt, " altAdjust= ", alts.wheelsOnGroundAGL_ft, " calcAlt_ft=", calcAlt_ft, "damageAltAddCurrent=", damageAltAddCurrent, " ", myNodeName);
  
  
  
}



#######################################################
#location-check loop, a timer function, every 15-16 seconds to check if the object has been relocated (this will happen if the object is set up as an AI ship or aircraft and FG is reset).  If so it restores the object to its position before the reset.
#This solves an annoying problem in FG, where using file/reset (which
#you might do if you crash the aircraft, but also if you run out of ammo
#and need to re-load or for other reasons) will also reset the objects to 
#their original positions.
#With moving objects (set up as AI ships or aircraft with velocities, 
#rudders, and/or flight plans) the objects abre often just getting to 
#interesting/difficult positions, so we want to preserve those positions 
# rather than letting them reset back to where they started.
#TODO: Some of this could be done better using a listener on /sim/signals/reinit
var location_loop = func(id, myNodeName) {
  var loopid = getprop(""~myNodeName~"/bombable/location-loopid"); 
	id == loopid or return;
	
	var node = props.globals.getNode(myNodeName);
	
	
	var started = getprop (""~myNodeName~"/position/previous/initialized");
	
  var lat = getprop(""~myNodeName~"/position/latitude-deg");
  var lon = getprop(""~myNodeName~"/position/longitude-deg");
  var alt_ft = getprop(""~myNodeName~"/position/altitude-ft");
  
  if (lat==nil) {
        lat=0; 
        debprint ("Bombable: Lat=NIL, location_loop", myNodeName);
  }
  if (lon==nil) {
        lon=0; 
        debprint ("Bombable: Lon=NIL, location_loop", myNodeName);
  }


  
  #getting the global_x,y,z seems to stop strange behavior from the smoke
  #when we do a relocate of the objects
  var global_x = getprop(""~myNodeName~"/position/global-x");
  var global_y = getprop(""~myNodeName~"/position/global-y");
  var global_z = getprop(""~myNodeName~"/position/global-z");
        
  
  prev_distance=0;
  directDistance=200; # this will be set as previous/distance if we are initializing
 
  # if we have previously recorded the position we check if it has moved too far
  # if it has moved too far it is because FG has reset and we 
  # then restore the object's position to where it was before the reset
	if (started ) {
     
     var prevlat = getprop(""~myNodeName~"/position/previous/latitude-deg");
     var prevlon = getprop(""~myNodeName~"/position/previous/longitude-deg");
     var prevalt_ft = getprop(""~myNodeName~"/position/previous/altitude-ft");
     var prev_global_x = getprop(""~myNodeName~"/position/previous/global-x");
     var prev_global_y = getprop(""~myNodeName~"/position/previous/global-y");
     var prev_global_z = getprop(""~myNodeName~"/position/previous/global-z");
 
     
     var prev_distance = getprop(""~myNodeName~"/position/previous/distance");
     
     var GeoCoord = geo.Coord.new();
     GeoCoord.set_latlon(lat, lon, alt_ft * 0.3048);

     var GeoCoordprev = geo.Coord.new();
     GeoCoordprev.set_latlon(prevlat, prevlon, prevalt_ft * 0.3048);

     var directDistance = GeoCoord.distance_to(GeoCoordprev);
     
     #debprint ("Object  ", myNodeName ", distance: ", directDistance);
     
     #4X the previously traveled distance is our cutoff
     #so if our object is moving faster/further than this we assume it has
     #been reset by FG and put it back where it was before the reset.
     #Luckily, this same scheme works in the case this subroutine has moved the 
     #object--then the previous distance exactly equals the distance traveled--
     #so even though that is a much larger than usual distance (which would
     #usually trigger this subroutine to think an init had happened) since
     #the object moved that large distance on the **previous step** (due to the
     #reset) the move back is less than 4X the previous move and so it is OK.

     #A bit kludgy . . . but it works.
     if ( directDistance > 5 and directDistance > 4 * prev_distance ) {
       node.getNode("position/latitude-deg", 1).setDoubleValue(prevlat);
       node.getNode("position/longitude-deg", 1).setDoubleValue(prevlon);
       node.getNode("position/altitude-ft", 1).setDoubleValue(prevalt_ft);
       #now we want to show the previous location as this newly relocated position and distance traveled = 0;
       lat=prevlat;
       lon=prevlon;
       alt_ft=prevalt_ft;
       
 
       debprint ("Bombable: Repositioned object "~ myNodeName~ " to lat: "~ prevlat~ " long: "~ prevlon~ " altitude: "~ prevalt_ft~" ft.");
     }
  }  
  #now we save the current position 
  node.getNode("position/previous/initialized", 1).setBoolValue(1);
  node.getNode("position/previous/latitude-deg", 1).setDoubleValue(lat);
  node.getNode("position/previous/longitude-deg", 1).setDoubleValue(lon);
  node.getNode("position/previous/altitude-ft", 1).setDoubleValue(alt_ft);
  node.getNode("position/previous/global-x", 1).setDoubleValue(global_x);
  node.getNode("position/previous/global-y", 1).setDoubleValue(global_y);
  node.getNode("position/previous/global-z", 1).setDoubleValue(global_z);   
 
  node.getNode("position/previous/distance", 1).setDoubleValue(directDistance);

  # reset the timer so we will check this again in 15 seconds +/-
  # add rand() so that all objects don't do this function simultaneously
  # when 15-20 objects are all doing this simultaneously it can lead to jerkiness in FG 
  settimer(func {location_loop(id, myNodeName); }, 15 + rand() );
}



################################################
#listener function on ballistic impacts
#checks if the impact has hit our object and if so, adds the damage
# damageMult can be set high (for easy to damage things) or low (for
# hard to damage things).  Default/normal value (M1 tank) should be 1.  

var test_impact = func(changedNode, myNodeName) {

	
  var impactNodeName = changedNode.getValue();
	var impactNode = props.globals.getNode(impactNodeName);
	var node = props.globals.getNode(myNodeName);

	var oLat=node.getNode("position/latitude-deg").getValue();
	var iLat=impactNode.getNode("impact/latitude-deg").getValue();
	
	var maxLat = getprop (""~myNodeName~"/bombable/attributes/dimensions/maxLat");
	var maxLon = getprop (""~myNodeName~"/bombable/attributes/dimensions/maxLon");
	
	
  #quick-n-dirty way to tell if an impact is close to our object at all
  #without processor-intensive calculations
  #we do this first and then exit if not close, to reduce impact
  #of impacts on processing time
	deltaLat=abs(oLat-iLat);
	if (deltaLat > maxLat ) {
      #debprint ("Not close in lat.");
      return; 
  }    
	
  oLon= node.getNode("position/longitude-deg").getValue();
  iLon= impactNode.getNode("impact/longitude-deg").getValue();

  deltaLon=abs(oLon-iLon);
	if (deltaLon > maxLon )  {
      #debprint ("Not close in lon.");
      return; 
  }

  oAlt_m= node.getNode("position/altitude-ft").getValue()*0.3048;
  iAlt_m= impactNode.getNode("impact/elevation-m").getValue();
	
  #debprint ("Impactor: ", impactNodeName, ", Object: ", myNodeName);
	if (impactNodeName=="" or impactNodeName==nil or impactNode==nil) return;
	

	# figure how close the impact and terrain it's on
	var objectGeoCoord = geo.Coord.new();
	objectGeoCoord.set_latlon(oLat,oLon,oAlt_m );
	var impactGeoCoord = geo.Coord.new();
	impactGeoCoord.set_latlon(iLat, iLon, iAlt_m);
	var impactDistance_m = objectGeoCoord.direct_distance_to(impactGeoCoord);
  #var impactSurfaceDistance_m = objectGeoCoord.distance_to(impactGeoCoord);
	#var heightDifference_m=math.abs(impactNode.getNode("impact/elevation-m").getValue() - node.getNode("position/altitude-ft").getValue()*0.3048);
	
	#debprint ("Testing impact of ", impactNodeName, " on ", myNodeName, " distance ", impactDistance_m, " meters." );
	
  #debprint ("Distance= ", impactDistance_m);  
  #ignore anything more than 200 meters distant
  if (impactDistance_m > 200 ) return;
  
  #debprint (impactDistance_m);
  #return;
  
   
  var impactTerrain = impactNode.getNode("impact/type").getValue(); 

    #Potential for adding serious damage increases the closer we are to the center
  #of the object.  We'll say more than damageRadius meters away, no potential for increased damage 
  
  var damageRadius_m= getprop (""~myNodeName~"/bombable/attributes/dimensions/damageRadius_m");
  
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  vuls= b.getNode("vulnerabilities").getValues();
  
  #weight/mass of the ballistic object, in lbs    	
  #var ballisticMass_lb = impactNode.getNode("mass-slug").getValue() * 32.174049; 
  

  
  var ballisticMass_slug = getprop (""~impactNodeName~"/mass-slug");

  #ok, FG 2.4.0 leaves out the /mass-slug property, so we have to improvise.
  # We basically need to list or guess the mass of each & every type of ordinance
  # that might exist or be used.  Not good. 
  if (ballisticMass_slug != nil ) ballisticMass_lb = ballisticMass_slug *  32.174049
  else {
     ballisticMass_lb=.25;
     var impactType = getprop (""~impactNodeName~"/name");
     #debprint ("Bombable: ImpactNodeType = ", impactType);
     if (impactType==nil) impactType="bullet";

     
     
     if (find ("gun", impactType ) != -1 ) ballisticMass_lb=.025;
     if (find ("bullet", impactType) != -1 ) ballisticMass_lb=0.0249122356;
     if (find ("tracer", impactType) != -1 ) ballisticMass_lb=0.0249122356;
     if (find ("round", impactType) != -1 ) ballisticMass_lb=0.9369635;
     if (find ("cannon", impactType ) != -1 ) ballisticMass_lb=0.282191696;
     if (find ("bomb", impactType ) != -1 ) ballisticMass_lb=250;
     if (find ("heavy-bomb", impactType ) != -1 ) ballisticMass_lb=750;
     if (find ("rocket", impactType ) != -1 ) ballisticMass_lb=50;
     if (find ("missile", impactType ) != -1 ) ballisticMass_lb=185;
     if (find ("MK-82", impactType ) != -1 ) ballisticMass_lb=500;
     if (find ("aim-9", impactType ) != -1 ) ballisticMass_lb=191;
     if (find ("WP-1", impactType ) != -1 ) ballisticMass_lb=23.9;
     if (find ("GAU-8", impactType ) != -1 ) ballisticMass_lb=0.9369635;
     if (find ("smoke", impactType ) != -1 ) ballisticMass_lb=0.0;
  }
  
  var ballisticMass_kg=ballisticMass_lb/2.2;
  
  var impactVelocity_mps = getprop (""~impactNodeName~"/impact/speed-mps");
  
  #if perchance impact velocity isn't available we'll estimate it from
  # projectile size  
  # These are rough approximations/guesses based on http://en.wikipedia.org/wiki/Muzzle_velocity  
  if (impactVelocity_mps == nil or impactVelocity_mps== 0) {
    if (ballisticMass_lb < 0.1) impactVelocity_mps= 1200;
    elsif (ballisticMass_lb < 0.5) impactVelocity_mps= 900;
    elsif (ballisticMass_lb < 2) impactVelocity_mps= 500;
    elsif (ballisticMass_lb < 50) impactVelocity_mps= 250;
    elsif (ballisticMass_lb < 500) impactVelocity_mps= 150;
    elsif (ballisticMass_lb < 2000) impactVelocity_mps= 125;
    else impactVelocity_mps= 100;
  }  
  
  
  # Kinetic Energy (joules) = 1/2* mass * velocity^2  (mass in kg, velocity in mps)
  # See http://en.wikipedia.org/wiki/Kinetic_energy   
  var kineticEnergy_joules= ballisticMass_kg *  impactVelocity_mps * impactVelocity_mps /2; 
  #debprint ("mass= ", ballisticMass_lb, " vel=", impactVelocity_mps, " Ek=", kineticEnergy_joules);
	
  #If the terrain is ship, aircraft, carrier, etc. and the distance is 
  #close to our object, we assume it is our object and this an actual
  #direct hit on it that definitely will do damage even if just a projectile
  #rather than an explosive.
  #
  #Todo: Make the distances here match the actual dimensions of our object.
  #ie, a ship will be far different in size from a jeep. Even just
  #using max dimension of the object/2 rather than 6.0 would be a
  #big improvement.
  #
  #Todo: Make sure the impact is actually on **this** object, not just another
  #one that happens to be immediately adjacent to this one. 
  #
  #upping impact distance to 6.0 from 4.0, because it seems to ignore
	#legitimate hits at times.  added surface/height distance to make it more 
  #forgiving if the impact registers close to the object horizontally but above 
  #or below, which seems to happen a lot. bhugh 8/2009.
  
  #How many shots does it take to down an object?  Supposedly the Red Baron
  #at times put in as many as 500 machine-gun rounds into a target to *make
  #sure* it really went down.
  
  

  
  var easyMode=1;
  var easyModeProbability=1;
  
  #Easy Mode increases the damage radius (2X), making it easier to score hits, 
  #but doesn't increase the damage done by armament
  if (getprop(""~bomb_menu_pp~"easy-mode")) {
     #easyMode*=2;
     damageRadius_m*=2; }
     
  #Super Easy mode increases both the damage radius AND the damage done  
  #by 3X 
  if (getprop(""~bomb_menu_pp~"super-easy-mode")){
      easyMode*=3;
      easyModeProbability*=3;        
      damageRadius_m*=3; }
    
      
  
	if((impactDistance_m <= 3*damageRadius_m)  and (impactTerrain != "terrain")) {  
  #case of direct hit
      
      damagePotential=0;
      outsideIDdamagePotential=0;
      
      # Purpose of setting outsideIDdamagePotential and allowing potential 
      #damage from hits outside the strict impact damage radius, is because
      #FG seems to intercept projectiles at a certain radius out from
      #the target object.  So hits that **would have** hit directly on the airframe
      #or close to the center of the object detonate out near the object's periphery
      #just because of the quirks of FG's impact detection model.
      #So when they hit at that distance we still allow them a certain probability
      #of doing damage. 
      #Enabling "Realistic Impact Mode" turns off this whole system
      
      #likelihood of damage goes up the closer we are to the center
      impactLikelihood=(3*damageRadius_m - impactDistance_m)/(3*damageRadius_m);
      
      #It's within damageRadius--some change of doing high damage and certainty of doing some damage
      if (impactDistance_m <= damageRadius_m )  { 
           #damagePotential = (damageRadius_m - impactDistance_m)/damageRadius_m;
           damagePotential = impactLikelihood/3; #possibility of causing a high amount of damage           
           outsideIDdamagePotential=impactLikelihood; #possibility of causing a routine amount of damage
           
          #debprint ("Bombable: Very Close hit, "~ impactNodeName~ " on ", myNodeName, " Distance= ", impactDistance_m, " heightDiff= ", heightDifference_m, " terrain=", impactTerrain, " radius=", damageRadius_m, " ", damagePotential, " ", outsideIDdamagePotential);
        
        
      } elsif ( ! getprop(""~bomb_menu_pp~"realistic-impact-mode") and (impactDistance_m <= 3*damageRadius_m) ) {
           #It's close but not within damage Radius: No chance of doing serious damage but
           #some chance of doing routine damage
           
           damagePotential=0;
           outsideIDdamagePotential= impactLikelihood / 2;# /2 means we'll only get hits half as often 
           #debprint ("Bombable: Near hit, "~ impactNodeName~ " on ", myNodeName, " Distance= ", impactDistance_m, " heightDiff= ", heightDifference_m, " terrain=", impactTerrain, " radius=", damageRadius_m, " ", damagePotential, " ", outsideIDdamagePotential);
           
      }         
        
      #height_m=getprop (""~myNodeName~"/bombable/attributes/dimensions/height_m");

    
		# TODO: damage depend on mass and speed of the 
    # ballistic
    # We can't do that yet as FG doesn't seem to expose the velocity of the object
    #our ballisticMass_lb variable is basically slugs * 32 or the "mass" of 
    #the object expressed in pounds.  So ballisticMass_lb=0.8 would 
    # be a 0.8 pound bullet or cannon round.
    #
    #What we'll do instead is go on ballistic mass, but make the assumption that small projectiles
    # will be going somewhat faster and thus be a little more lethal than their mass alone
    # would indicate.
                    
    var damageCaused=0;    
		if (ballisticMass_lb < 1.2) {
    # gun/small ammo
    
    #this compensates for the lack of info on velocity--by taking the bM ^ 0.8 
    # we make the masses on the smaller end of the scale from 0-1 somewhat bigger.    
    if (ballisticMass_lb <1 ) ballisticMass_lb = math.pow( ballisticMass_lb, 0.8) ;
        
    #Guarantee of some damage, maybe big damage if it hits some vital parts
    # (the 'if' is a model for the percentage chance of it hitting some vital part)         
     	if ( rand()< damagePotential*vuls.damageVulnerability*easyModeProbability * ballisticMass_lb / 5 ) {
       damageCaused=(rand()*(1-ballisticMass_lb/2) + ballisticMass_lb/2)*vuls.damageVulnerability*ballisticMass_lb*easyMode/2;  
       debprint ("Bombable: Vital hit. ballisticMass: ", ballisticMass_lb," damPotent: ", damagePotential);
         
     	
    #Otherwise the possibility of damage 	
		 } elsif (rand() < outsideIDdamagePotential) {
		 
         damageCaused=rand () * ballisticMass_lb * vuls.damageVulnerability*easyMode * outsideIDdamagePotential;      
         debprint ("Bombable: Peripheral hit. ballisticMass: ", ballisticMass_lb," outsideIDDamPotent: ", outsideIDdamagePotential);

     }
  
			
			add_damage(damageCaused, myNodeName, "weapon" ); 
			
		} else {
		# anything larger than 1.2 lbs making a direct hit.  It's some kind of bomb or 
		# exploding ordinance, presumably		
		 
		  var damagePoss= .6 + ballisticMass_lb/250;
		  if (damagePoss>1) damagePoss=1;
			# if it hits a vital spot (which becomes more likely, the larger the bomb)
     	if ( rand()< damagePotential*vuls.damageVulnerability*easyModeProbability * ballisticMass_lb / 5  ) damageCaused=damagePoss*vuls.damageVulnerability*ballisticMass_lb*easyMode/2; 
		  else  #if it hits a regular or less vital spot
         damageCaused=rand () * ballisticMass_lb * vuls.damageVulnerability*easyMode * outsideIDdamagePotential;   

	
			add_damage( damageCaused, myNodeName, "weapon"  ); 		}
	
  # end, case of direct hit
  } else { 
  # case of a near hit, on terrain, if it's a bomb we'll add damage


   	 if (ballisticMass_lb >= 200) debprint ("Bombable: Close hit by heavy bomb, "~ impactNodeName~ " on "~ myNodeName~ " Distance= "~ impactDistance_m~ 
      #" heightDiff= "~ heightDifference_m~ 
      " terrain="~ impactTerrain~ " radius="~ damageRadius_m);

		# check submodel blast effect distance.
    if (ballisticMass_lb < 50 and ballisticMass_lb >= 10 ) {
			if(impactDistance_m <= 10 + damageRadius_m)
				add_damage(.2 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");
			elsif((impactDistance_m > 10 + damageRadius_m) and (impactDistance_m < 30 + damageRadius_m))
				add_damage(0.05 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
			elsif((impactDistance_m > 30 + damageRadius_m) and (impactDistance_m < 60 + damageRadius_m))
				add_damage(0.01 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon"); 
			else
				add_damage(0.002 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
		} elsif (ballisticMass_lb < 200 and ballisticMass_lb >= 50 ) {
			if(impactDistance_m <= 10 + damageRadius_m)
				add_damage(.5 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");
			elsif((impactDistance_m > 10 + damageRadius_m) and (impactDistance_m < 30 + damageRadius_m))
				add_damage(0.1 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
      elsif((impactDistance_m > 30 + damageRadius_m) and (impactDistance_m < 60 + damageRadius_m))
				add_damage(0.02 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon"); 
			else 	
			  add_damage(0.01 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
		 } elsif ((ballisticMass_lb >= 200) and (ballisticMass_lb < 350)) {
			# Mk-81 class
			if(impactDistance_m <= 10 + damageRadius_m)
				add_damage(2 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");
			elsif((impactDistance_m > 10 + damageRadius_m) and (impactDistance_m < 30 + damageRadius_m))
				add_damage(0.4 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
      elsif((impactDistance_m > 30 + damageRadius_m) and (impactDistance_m < 60 + damageRadius_m))
				add_damage(0.1 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon");
      else  add_damage(0.05 * vuls.damageVulnerability * ballisticMass_lb* easyMode, myNodeName, "weapon"); 
		} elsif((ballisticMass_lb >= 350) and (ballisticMass_lb < 750)) {
			# Mk-82 class
			if(impactDistance_m <= 33 + damageRadius_m )
				add_damage(2 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon"); 
			elsif((impactDistance_m > 33 + damageRadius_m) and (impactDistance_m < 50 + damageRadius_m))
				add_damage(0.5 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");
        else add_damage(0.25 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon"); 
		} elsif(ballisticMass_lb >= 750) {
			# Mk-83 class and upper
			if(impactDistance_m <= 70 + damageRadius_m )
				add_damage(2 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon"); 
			elsif((impactDistance_m > 70 + damageRadius_m ) and (impactDistance_m < 200 + damageRadius_m))
				add_damage(0.5 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");
      else add_damage(0.25 * vuls.damageVulnerability * ballisticMass_lb * easyMode, myNodeName, "weapon");   
		}
	}
	
	#any impacts somewhat close to us, we start dodging
	if (impactDistance_m < 500 ) dodge (myNodeName);
  
}


#########################################################
#rudder_roll_climb - sets the rudder position/roll degrees
#roll degrees controls aircraft & rudder position
#and for aircraft, sets an amount of climb
#controls ships, so we just change both, to be safe
var rudder_roll_climb = func (myNodeName, degrees=15, alt_ft=-20, time=10 ){

   debprint ("Bombable: rudder_roll_climb starting");
   var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
   alts= b.getNode("altitudes").getValues();
   node= props.globals.getNode(myNodeName);
   var type=node.getName();
   

   #rudder/roll
   currRudder=getprop(""~myNodeName~"/surface-positions/rudder-pos-deg");
   if (currRudder==nil) currRudder=0;
   
   currRoll=getprop(""~myNodeName~"/controls/flight/target-roll");
   if (currRoll==nil) currRoll=0;
   

   #add our amount to any existing roll/rudder & set the new roll/rudder position
   
   
   #this massively speeds up turns vs FG's built-in AI's rather
   # sedate turns; turns to the selected roll value in 5 seconds
   # rolling degrees/2 seems to give a turn of about degrees      
   if (type=="aircraft" and degrees !=0 ) aircraftRoll (myNodeName, degrees, time);
   
   #debprint ("Bombable: setprop 2218");
   setprop(""~myNodeName~"/surface-positions/rudder-pos-deg", currRudder + degrees);
   setprop(""~myNodeName~"/controls/flight/target-roll", currRoll + degrees);
   
   #altitude
   #This only works for aircraft but that's OK because it's not sensible 
   #for a ground vehicle or ship to dive or climb above or below ground/sea
   #level anyway (submarines excepted . . . but under current the FG AI system
   # it would have to be operated as an "aircraft", not a "ship", if it 
   # wants to be able to climb & dive). 
   var currAlt_ft= getprop(""~myNodeName~"/position/altitude-ft"); #where the object is, in ft
   if (currAlt_ft + alt_ft < alts.minimumAGL_ft ) alt_ft = alts.minimumAGL_ft;
   if (currAlt_ft + alt_ft > alts.maximumAGL_ft ) alt_ft = alts.maximumAGL_ft;
   
   #debprint ("Bombable: setprop 2232");          
   setprop (""~myNodeName~"/controls/flight/target-alt", alt_ft);
   
   #debprint ("2206 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
      

   #debprint (myNodeName, " dodging ", degrees, " degrees ", alt_ft, " feet");
}

################################################################
#function makes an object dodge
#
var dodge = func(myNodeName) {
   
   var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
   if ( getprop ( ""~myNodeName~"/bombable/attributes/evasions/dont-dodge" ) == 1 ) {
      #debprint ("Bombable: Dodge temporarily locked for this object. ", myNodeName );
      return;
   }   
   node= props.globals.getNode(myNodeName);
   var type=node.getName();
   debprint ("Bombable: Starting Dodge", myNodeName, " type= ", type);

   evas= b.getNode("evasions").getValues();

   
   #amount to dodge, up to dodgeMax_deg in either direction
   dodgeAmount_deg=(evas.dodgeMax_deg-evas.dodgeMin_deg)*rand()+evas.dodgeMin_deg;
   

   if (rand() > evas.dodgeROverLPreference_percent/100) dodgeAmount_deg = -dodgeAmount_deg;

   #target amount to climb or drop 
   dodgeAltAmount_ft=(evas.dodgeAltMax_ft - evas.dodgeAltMin_ft) * rand()+evas.dodgeAltMin_ft;
   
   var dodgeDelay=(evas.dodgeDelayMax_sec-evas.dodgeDelayMin_sec)*rand()+evas.dodgeDelayMin_sec;
   
   if (type=="aircraft") {
       #set rudder or roll degrees to that amount
       rudder_roll_climb (myNodeName, dodgeAmount_deg, dodgeAltAmount_ft, dodgeDelay/2);
      #hold the aircraft at this roll--otherwise the AI
      #will return it to near-level flight   

       dodgeVertSpeed_fps = 0;
       currRoll_deg=getprop (""~myNodeName~"/orientation/roll-deg");
       
       
       if ( dodgeAltAmount_ft > 0 )  dodgeVertSpeed_fps = math.abs ( evas.dodgeVertSpeedClimb_fps * dodgeAltAmount_ft/evas.dodgeAltMax_ft);
       if ( dodgeAltAmount_ft < 0 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps * dodgeAltAmount_ft/evas.dodgeAltMin_ft ); 
       
       #velocities/vertical-speed-fps seems to be fps * 1000 for some reason?  At least, approximately, 300,000 seems to be about 300 fps climb, for instance.
       # and we reduce the amount of climb/dive possible depending on the current roll angle (can't climb/dive rapidly if rolled to 90 degrees . . . )    
       #dodgeVertSpeed_fps*=1000 * math.abs(math.cos(currRoll_deg/rad2degrees));
       dodgeVertSpeed_fps*= math.abs(math.cos(currRoll_deg/rad2degrees)); 
       
       #vert-speed prob
       #just putting a large number directly into vertical-speed-fps makes the aircraft
       #jump up or down far too abruptly for realism
       #if (dodgeVertSpeed_fps!=0) setprop ("" ~ myNodeName ~ "/velocities/vertical-speed-fps", dodgeVertSpeed_fps);

      debprint ("Dodging: ", myNodeName, " ", dodgeAmount_deg, " ", dodgeAltAmount_ft, " ", dodgeVertSpeed_fps); 
      settimer ( func { aircraftRoll (myNodeName, 0, dodgeDelay/2); }, dodgeDelay/2);
      
       #Don't change rudder/roll again until the delay
     
       setprop ( ""~myNodeName~"/bombable/attributes/evasions/dont-dodge" , 1);
       # Roll/climb for dodgeDelay seconds, then wait dodgeDelay seconds (to allow
       # the aircraft's turn to develop from the roll).
       # After this delay FG's aircraft AI will automatically return
       # it to near-level flight        
       #              
       settimer ( func {
                    setprop(""~myNodeName~"/bombable/attributes/evasions/dont-dodge", 0);
                    #This resets the aircraft to 0 deg roll (via FG's
                    # AI system target roll; leaves target altitude 
                    # unchanged  )
                    rudder_roll_climb (myNodeName, -dodgeAmount_deg, dodgeAltAmount_ft, dodgeDelay/2);
       
                      } 
       , dodgeDelay );
   
   } else {  #other types besides aircraft
  
       #set rudder or roll degrees to that amount
       rudder_roll_climb (myNodeName, dodgeAmount_deg, dodgeAltAmount_ft, dodgeDelay);
      
       
      
       #Don't change rudder/roll again until the delay
     
       setprop ( ""~myNodeName~"/bombable/attributes/evasions/dont-dodge" , 1);
       # Roll/climb for dodgeDelay seconds, then wait dodgeDelay seconds (to allow
       # the aircraft's turn to develop from the roll), then
       # after this delay, reset the rudder/roll to where it was before 
       #and allow further dodging to occur. Just leave the target alt
       # where it is.   
       settimer ( func {setprop(""~myNodeName~"/bombable/attributes/evasions/dont-dodge", 0);
               rudder_roll_climb (myNodeName, -dodgeAmount_deg, 0, dodgeDelay );}, 2*dodgeDelay );
   
   }
   
}


################################################################
# function updates dFramage to the ai or main aircraft when a msg
# is received over MP 

#damageRise is the increase in damage sent by the remote MP aircraft
#damageTotal is the remote MP aircraft's current total of damage
# (This should always be <= our damage total, so it is a failsafe
# in case of some packet loss)
var mp_update_damage = func (myNodeName="", damageRise=0, damageTotal=0, smokeStart=0, fireStart=0, callsign="" ) {

  if (myNodeName=="") myNodeName="/environment";
  
  #if (myNodeName=="/environment") debprint ("Bombable: Updating main aircraft 2328");
  
  var damageValue = getprop(""~myNodeName~"/bombable/attributes/damage");
  if (damageValue == nil ) damageValue=0;
  
 	if (damageValue<damageTotal) {
   
    damageValue=damageTotal;
    msg = sprintf( "Damage for "~string.trim(callsign)~" is %d%%", damageValue*100);
  
    if (myNodeName=="/environment") selfStatusPopupTip (msg, 30); 
    else targetStatusPopupTip (msg, 30);
    debprint ("Bombable: " ~ msg );
     
  }
		
  #make sure it's in range 0-1.0
  if(damageValue > 1.0)
			damageValue = 1.0;
	elsif(damageValue < 0.0)
			damageValue = 0.0;
	
  setprop(""~myNodeName~"/bombable/attributes/damage", damageValue);
  
  if (smokeStart) startSmoke ("damagedengine", myNodeName); 
  else deleteSmoke("damagedengine", myNodeName);
  
  if (fireStart) startFire (myNodeName); 
  else deleteFire(myNodeName);
  
  
  
  if (damageValue >= 1 and damageRise > 0 ) { 
      #make the explosion
      var smokeStartsize=rand()*10 + 5;
      settimer (func {setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize); }, 2.5);#turn the big explosion off sorta quickly 
      
      
      explosiveMass_kg=getprop(""~myNodeName~"/bombable/attributes/vulnerabilities/explosiveMass_kg");
      if (explosiveMass_kg==nil or explosiveMass_kg==0) explosiveMass_kg = 10000; 
      smokeMultiplier = math.log10(explosiveMass_kg) * 10;
      setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize * smokeMultiplier + smokeMultiplier * rand());


   }
   
}

################################################################
# function sends the main aircraft's current damage, smoke, fire 
# settings over MP.  This is is to update all other MP aircraft
# with this aircraft's current damage status.  Other aircraft 
# track the damage internally, but if a 3rd aircraft damages this 
# aircraft, or if damage is added due to fire, etc., then the
# only way other MP aircraft will know about the damage is via this
# update.
#  

var mp_send_main_aircraft_damage_update = func (damageRise=0 ) {

  if (!getprop(MP_share_pp)) return "";
  if (!getprop (MP_broadcast_exists_pp)) return "";
  
  damageTotal=getprop("/environment/bombable/attributes/damage");
  if (damageTotal==nil) damageTotal=0;
  smokeStart=getprop("/environment/bombable/fire-particles/damagedengine-burning");
  if (smokeStart==nil) smokeStart=0;
  fireStart=getprop("/environment/bombable/fire-particles/fire-burning");
  if (fireStart==nil) fireStart=0;
  #mp_send_damage("", damageRise, damageTotal, smokeStart, fireStart);
  
  callsign=getprop ("/sim/multiplay/callsign");
  if (callsign==nil) callsign="";

  var msg=damage_msg (callsign, damageRise, damageTotal, smokeStart, fireStart, 3);                   
    if (msg != nil and msg != "") {
      debprint ("Bombable MADU: MP sending: "~callsign~" "~damageRise~" "~damageTotal~" "~smokeStart~" "~fireStart~" "~msg);
      mpsend(msg);
    }

}

################################################################
# function adds damage to the main aircraft when a msg
# is received over MP 
# 
# Also start smoke/fire if appropriate, and turn off engines/explode
# when damage reaches 100%.

#damageRise is the increase in damage sent by the remote MP aircraft
#damageTotal is the remote MP aircraft's current total of damage
# (This should always be <= our damage total, so it is a failsafe
# in case of some packet loss)
var mp_add_damage = func (damageRise=0, damageTotal=0, source="", message="") {

  var damageValue = getprop("/environment/bombable/attributes/damage");
  if (damageValue == nil ) damageValue=0;
  
  prevDamageValue=damageValue;
  
 	if(damageValue < 1.0) 
		damageValue += damageRise;
		
  if (damageValue<damageTotal) damageValue=damageTotal;
		
  #make sure it's in range 0-1.0
  if(damageValue > 1.0)
			damageValue = 1.0;
	elsif(damageValue < 0.0)
			damageValue = 0.0;
	
  setprop("/environment/bombable/attributes/damage", damageValue);
  
  damageIncrease=damageValue-prevDamageValue;
  
  if (damageIncrease>0)  {
      addMsg1="You've been damaged!";
      addMsg2="You are out of commission!";
      if (message!="")  {
        addMsg1=message;
        addMsg2=message;
      }         
      if (damageValue < .01) msg= sprintf( addMsg1 ~ " Damage added %1.2f%% - Total damage %d%%", damageIncrease*100, damageValue*100);
      elsif (damageValue < .1) msg= sprintf( addMsg1 ~ " Damage added %1.1f%% - Total damage %d%%", damageIncrease*100, damageValue*100); 
      elsif (damageValue < 1) msg= sprintf( addMsg1 ~ " Damage added %d%% - Total damage %d%%", damageIncrease*100, damageValue*100);
      else msg= sprintf( "======== " ~ addMsg2 ~ " Damage added %d%% - Total damage %d%% ========", damageIncrease*100, damageValue*100); 
      selfStatusPopupTip (msg, 30);
      debprint ("Bombable: " ~ msg );
      

  }
  
  
  #Update--we don't allow remote control of main aircraft's
  # fire/smoke any more.  Causes problems.  
  #if (smokeStart) startSmoke ("damagedengine", ""); 
  #else deleteSmoke("damagedengine", "");
  
  #if (fireStart) startFire (""); 
  #else deleteFire("");
  
  #start smoke/fires if appropriate
  # really we need some way to customize this for every aircraft
  # just as we do for AI/MP aircraft.  But in the meanwhile this will work:
  
  myNodeName="/environment"; #main aircraft
  
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var vuls= b.getNode("vulnerabilities").getValues();

  
        

		var fireStarted=getprop("/environment/bombable/fire-particles/fire-burning");
		if (fireStarted == nil ) fireStarted=0;
   	var damageEngineSmokeStarted=getprop("/environment/bombable/fire-particles/damagedengine-burning");
   	if (damageEngineSmokeStarted == nil ) damageEngineSmokeStarted = 0; 
   	
   	
   	
   	if (!damageEngineSmokeStarted and !fireStarted and damageIncrease > 0 and rand()*100 < vuls.engineDamageVulnerability_percent ) 
        startSmoke("damagedengine",myNodeName);
       

	  # start fire if there is enough damage AND if the damage is caused by the right thing (weapons, crash, but not.
	  # if a crash, always start a fire (sometimes we reach 100% damage, no fire,
	  # then crash later--so when we crash, always start fire regardless)     	  
		if( (  ( 
              damageValue >= 1 - vuls.fireVulnerability_percent/100 
              and damageIncrease > 0 and !fireStarted 
            ) and 
              (source=="weapons" or source=="crash" ) 
        ) or 
            (source=="crash" )
    ) {
		
		
		  debprint ("Bombable: Starting fire for main aircraft");
			
			#use small, medium, large smoke column depending on vuls.damageVulnerability
			#(high vuls.damageVulnerability means small/light/easily damaged while 
			# low vuls.damageVulnerability means a difficult, hardened target that should burn
			# more vigorously once finally on fire)   
			var fp="";                     			
			if (vuls.explosiveMass_kg < 1000 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-small.xml"; }
			elsif (vuls.explosiveMass_kg > 50000 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-large.xml"; }
			else {fp="AI/Aircraft/Fire-Particles/fire-particles.xml";} 
			
      startFire(myNodeName, fp); 
      #only one damage smoke at a time . . . 
      deleteSmoke("damagedengine",myNodeName);
      
      
      #fire can be extinguished up to MaxTime_seconds in the future,
      #if it is extinguished we set up the damagedengine smoke so 
      #the smoke doesn't entirely go away, but no more damage added
      if ( rand() * 100 < vuls.fireExtinguishSuccess_percentage ) {
      
         settimer (func { 
           deleteFire (myNodeName);
           startSmoke("damagedengine",myNodeName); 
           } ,            
           rand() * vuls.fireExtinguishMaxTime_seconds + 15 ) ;
      };
  }
  
  #turn off engines if appropriate
  if (damageValue >= 1 and (prevDamageValue < 1 or getprop("/controls/engines/engine[0]/magnetos") > 0 or getprop("/controls/engines/engine[0]/throttle") > 0 )) { 
  
      #turn off all engines
      #debprint ("Bombable: setprop 2553");
      setprop("/controls/engines/engine[0]/magnetos",0);
      setprop("/controls/engines/engine[0]/throttle",0);
      setprop("/controls/engines/engine[1]/magnetos",0);
      setprop("/controls/engines/engine[1]/throttle",0);
      setprop("/controls/engines/engine[2]/magnetos",0);
      setprop("/controls/engines/engine[2]/throttle",0);
      setprop("/controls/engines/engine[3]/magnetos",0);
      setprop("/controls/engines/engine[3]/throttle",0);
      
      #if no smoke/fire yet, now is the time to start
      startSmoke ("damagedengine", ""); 
      if (source=="weapons" or source=="crash" ) startFire (""); 
      
      var smokeStartsize=rand()*10 + 5;
      settimer (func {setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize); }, 2.5);#turn the big explosion off sorta quickly 
      
      smokeMultiplier = math.log10(vuls.explosiveMass_kg) * 10;
      setprop ("/environment/bombable/fire-particles/smoke-startsize", smokeStartsize * smokeMultiplier + smokeMultiplier * rand());

   }
   
   
   mp_send_main_aircraft_damage_update (damageRise);
   
   
}


#send the damage message via multiplayer
var mp_send_damage = func (myNodeName="", damageRise=0 ) {
      
      if (!getprop(MP_share_pp)) return "";
      if (!getprop (MP_broadcast_exists_pp)) return "";
      
      #this makes it the main aircraft if nodename=""
      if (myNodeName=="") myNodeName="/environment";
      
      #messageType 1 is letting another MP aircraft know you have damaged it
      #messageType 3 is informing all other MP aircraft know about the main
      # aircraft's current damage, smoke, fire settings
      #                   
      messageType=1;
      if (myNodeName=="/environment" or myNodeName=="environment") messageType=3;                                                                           
      var damageValue = getprop(""~myNodeName~"/bombable/attributes/damage");
      if (damageValue==nil) damageValue=0;
      
      if (myNodeName=="/environment"){
        callsign=getprop ("/sim/multiplay/callsign");
      }else {
        callsign=getprop (""~myNodeName~"/callsign");
      }
      var fireStart=getprop(""~myNodeName~"/bombable/fire-particles/fire-burning");
  		if (fireStart == nil ) fireStart=0;
     	var smokeStart=getprop(""~myNodeName~"/bombable/fire-particles/damagedengine-burning");
     	if (smokeStart == nil ) smokeStart = 0;
      # debprint ("Bombable MSD: Preparing to send MP damage update to "~callsign);       
      var msg=damage_msg (callsign, damageRise, damageValue, smokeStart, fireStart, messageType);
      
      if (msg != nil and msg != "") {
        debprint ("Bombable MSD: MP sending: "~callsign~" "~damageRise~" "~damageValue~" "~smokeStart~" "~fireStart~" "~messageType~" "~msg);
        mpsend(msg);                                                    
      }
             
}  

###############################################
#calculates angle (vertical, degrees above or below directly
# horizontal) between two geocoords, in degrees
#
var vertAngle_deg = func (geocoord1, geocoord2) {
      var dist=geocoord2.direct_distance_to(geocoord1);
      if ( dist == 0 ) return 0; 
      else return math.asin((geocoord2.alt() - geocoord1.alt())/dist ) * R2D;

}

###############################################
# Checks that the two objects are within the given distance
# and if so, checks whether myNodeName1 is aimed directly at myNodeName2
# within the heading & vertical angle given.
# If all that is true, returns number 0-1 telling how close the hit (1 being 
# perfect hit), otherwise 0 
# works with AI/MP aircraft AND with the main aircraft, which can 
# use myNodeNameX="" or "/environment" or "environment"
# Note: This whole approach won't work too well because the position & 
# orientation properties are updated only 1X per second.
# But who knows--it might be 'good enough' . . .
# 
#     # 
    # Notes on calculating the angular size of the target and being
    # able to tell if the weapon is aimed at the target with sufficient accuracy.    
    # 3 degrees @ 200 meters, 2% damage add maximum.
    # This equates to about a 6 meter damage radius
    # 
    # 2.5, 7.5 equates to an object about 10 ft high and 30 feet wide 
    #     about the dimensions of a Sopwith Camel
    #              
    # formula is tan (angle) = damage radius / distance
    # or angle = atan (damage radius/ distance)
    #  however, for 0<x<1, atan(x) ~= x (x expressed in radians).  
    #  Definitely this is close enough for our purposes.
    # 
    # Since our angles are expressed in degress the only difficulty is to convert radians        
    # to degrees. So the formula is:
    # angle (degrees) = damage radius / distance * (180/pi)
    # 
    # Below we'll put in the general height & width of the object, so the equations become:        
    #
    # # angle (degrees) = height/2 / distance * (180/pi)
    # # angle (degrees) = width/2 / distance * (180/pi)                               
    # (approximate because our damage area is a rectangle projected 
    # onto the surface of a sphere here, not just a radius )
    # Good estimate: dimension in feet divided by 4 equals angle to use here    
    #                      
    # We make the 'hit' area tall & skinny rather than wide & flat 
    # because our fighters are vertically challenged as far as movement,
    #     but quite easily able to aim horizontally
    #
    #Current notes:
    #vertDeg & horzDeg define the angular size of the target (ie, our main aircraft) at
    #the distance of maxDamageDistance_m.  3.43 degrees x 3.43 degrees at 500 meters is approximately right 
    #for (say) a sopwith camel.  Really this needs to be calculated more exactly based on the
    #actual dimensions of the main aircraft. But for now this is at least close.  There doesn't
    #seem to be a simple way to get the dimensions of the main aircraft from FG.
    #Bombable calculates the hits as on a probabalistic basis if they hit within the given
    #angular area.  The closer to the center of the target, the higher the probability of a hit
    #and the more the damage.  Since the bombable/ai weapons system is simulated (ie, based on the probably
    # of hits based on being aimed in the general area of the main aircraft, rather than
    # actually launching projectiles and seeing if they hit) this is generally a 'good enough'
    # approach.       
 

var checkAim = func (myNodeName1="", myNodeName2="",   
          targetSize_m=nil,  aiAimFudgeFactor=1, maxDistance_m=100, weaponAngle_deg=nil, weaponOffset_m=nil ) {
          
  if (myNodeName1=="/environment" or myNodeName1=="environment") myNodeName1="";
  if (myNodeName2=="/environment" or myNodeName2=="environment") myNodeName2="";

  #below is a quickndirty way to avoid doing direct_distance_to on each & every
  # call of this routine.  It assumes effective distance of any weapon is
  # no more than 200 meters.    
	var maxLat = getprop (""~myNodeName1~"/bombable/attributes/dimensions/maxLat");

	#quick-n-dirty way to tell if an impact is close to our object at all
  #without processor-intensive calculations
  #we do this first and then exit if not close, to reduce impact
  #of impacts on processing time
  # 

	var alat = getprop(""~myNodeName1~"/position/latitude-deg");
	var alon = getprop(""~myNodeName1~"/position/longitude-deg");
	var mlat = getprop(""~myNodeName2~"/position/latitude-deg");
	var mlon = getprop(""~myNodeName2~"/position/longitude-deg");
    
	deltaLat=abs(alat - mlat);
	if (deltaLat > maxLat ) {
      #debprint ("Aim: Not close in lat.");
      return 0; 
  }    

	var maxLon = getprop (""~myNodeName1~"/bombable/attributes/dimensions/maxLon");
	
  deltaLon=abs(alon - mlon );
	if (deltaLon > maxLon )  {
      #debprint ("Aim: Not close in lon.");
      return 0; 
  }

  
  if ( targetSize_m==nil or targetSize_m.horz<=0 or targetSize_m.vert <=0 or maxDistance_m <= 0) return 0;
  
  if (weaponAngle_deg==nil ){ weaponAngle_deg = {heading:0, elevation:0};}
  if (weaponOffset_m==nil ){ weaponOffset_m = {x:0,y:0,z:0}; } 
  
  var geocoord1=any_aircraft_position (myNodeName1);
  var geocoord2=any_aircraft_position (myNodeName2);
  
  #offset the location of the weapon by the weaponOffset_m amount:
  # Ok, this is slow, we're disabling it for now  
  #var geocoord1 = geocoord1.set_xyz(geocoord1.x()+ weaponOffset_m.x, geocoord1.y()+ weaponOffset_m.y, geocoord1.z()+ weaponOffset_m.z);
   
  var distance_m = geocoord1.direct_distance_to(geocoord2);
  
  if (distance_m > maxDistance_m ) return 0;
    # # angle (degrees) = height/2 / distance * (180/pi)
    # # angle (degrees) = width/2 / distance * (180/pi)                               
  
  
  if (distance_m==0) return 1;   #collision
  
  #var factor=maxDistance_m/distance_m;#as the object gets closer we can expand the degrees of a hit to be bigger; at maxDistance it is X degrees but if 1/2 maxDistance, 2X degrees, etc
  
  if (myNodeName1=="") myHeading_deg=getprop (""~myNodeName1~"/orientation/heading-deg"); 
  else myHeading_deg=getprop (""~myNodeName1~"/orientation/true-heading-deg");
  
  var headingDelta_deg=math.abs ( geocoord1.course_to(geocoord2) -  ( myHeading_deg + weaponAngle_deg.heading) );
  
  debprint( "Bombable: checkAim distance "~ distance_m ~ " heading_delta ", headingDelta_deg);

    # Formula: angle (degrees) = height/2 / distance * (180/pi)
    # # angle (degrees) = width/2 / distance * (180/pi)                               

  var horzTargetSize_deg = targetSize_m.horz/distance_m * (90/3.1417) * aiAimFudgeFactor;
  debprint( "Bombable: checkAim horzTargetSize_deg", horzTargetSize_deg);
  
  if ( headingDelta_deg > horzTargetSize_deg ) return 0;
  
  var myPitch_deg=getprop (""~myNodeName1~"/orientation/pitch-deg");
    
  var vertDelta_deg= math.abs (vertAngle_deg(geocoord1,geocoord2) - ( myPitch_deg + weaponAngle_deg.elevation ));
  
  #extra fudgefactor on vert because our fighters are a bit vertically-aiming challenged  
  var vertTargetSize_deg = targetSize_m.vert/distance_m * (90/3.1417) * aiAimFudgeFactor * 1.5;

  debprint( "Bombable: checkAim vertDelta ", vertDelta_deg, " vertTargetSize_deg ", vertTargetSize_deg );
    
  if ( vertDelta_deg  > vertTargetSize_deg ) return 0;
  
  var result = (1 - vertDelta_deg/vertTargetSize_deg) * (  1 - headingDelta_deg/horzTargetSize_deg);
  
  return result;  #ranges 0 to 1, 1 being direct hit
  
} 
  
    
var weapons_loop = func (id, myNodeName1="", myNodeName2="", targetSize_m=nil, aiAimFudgeFactor=1) {

   #we increment loopid if we want to kill this timer loop.  So check if we need to kill/exit:
   # myNodeName1 is the AI aircraft and nyNodeName2 is the main aircraft    
   var loopid = getprop(""~myNodeName1~"/bombable/weapons-loopid"); 
   id == loopid or return;

             
   #debprint ("Bombable: attacksaim: ", weaponOffset_m.x, " ", weaponOffset_m.y, " ", weaponOffset_m.z, " " );
   #the long/lat settings are only updated 1X per second so little point 
   # in updating more often than that!      
   settimer (  func { weapons_loop (id, myNodeName1, myNodeName2, targetSize_m, aiAimFudgeFactor )}, 1 + rand()/2);

   if (! getprop ( bomb_menu_pp~"ai-aircraft-weapons-enabled") ) return;
   
   var b = props.globals.getNode (""~myNodeName1~"/bombable/attributes");
   
   #if no weapons set up then just return
   if (!contains (b.getValues(), "weapons")) return;
   
   var weaps= b.getNode("weapons").getValues();
   
   #myNodeName ~ "bombable/attributes/weapons 
   if (weaps== nil or typeof(weaps) != "hash") return;
   
                          
   foreach (elem;keys (weaps) ) {

       #   elem.maxDamageDistance_m, 
       #           elem.maxDamage_percent, elem.weaponAngle_deg, 
       #           elem.weaponOffset_m
       # 
       # 
       #               
     mDD_m = weaps[elem].maxDamageDistance_m;
     if (mDD_m==nil or mDD_m==0) mDD_m=1000;       
     #debprint ("Bombable: Weapons_loop ", myNodeName1, " ", weaps[elem].maxDamageDistance_m);
     
  
     result=checkAim (myNodeName1, myNodeName2, targetSize_m, aiAimFudgeFactor,  
                 weaps[elem].maxDamageDistance_m, weaps[elem].weaponAngle_deg, 
                 weaps[elem].weaponOffset_m );
     
      
     if (result==0) continue;
     
     debprint ("Bombable: AI aircraft aimed at main aircraft, ",
        myNodeName1, " ", weaps[elem].name, " ", elem, 
        " accuracy ", int(result * 100 ),"%");
     
     var ai_callsign=getprop(""~myNodeName1~"/callsign");
     if (ai_callsign==nil) ai_callsign="";
  
     # As with our regular damage, it has a result% change of registering
     # a hit and then the damage amount is higher as result increases, too.    
     # There is a smaller chance of doing a fairly high level of damage (up to 3X the regular max), 
     # and the better/closer the hit, the greater chance of doing that significant damage.  
     var r=rand();    
     if (r < result) {
       
       var damageAdd= result * weaps[elem].maxDamage_percent/100;
       
       #Some chance of doing more damage (and a higher chance the closer the hit)
       if (r < result/5 ) damageAdd *= 3*rand();
       
       weaponName=weaps[elem].name;
       if (weaponName==nil) weaponName="Main Weapon"; 
       
       mp_add_damage ( damageAdd, 0, source="weapons", 
            message="Hit from " ~ ai_callsign ~ " - " ~ weaponName ~"!");
     
     }
  }   
} 


###############################################
#returns myNodeName position as a geo.Coord
# works for any aircraft; for main aircraft myNodeName="" or "/environment"
var any_aircraft_position = func (myNodeName) {
  if (myNodeName=="/environment" or myNodeName=="environment") myNodeName="";
  
  #if (myNodeName=="") debprint ("Bombable: Updating main aircraft 2700");
  
	var lat = getprop(""~myNodeName~"/position/latitude-deg");
	var lon = getprop(""~myNodeName~"/position/longitude-deg");
	var alt = getprop(""~myNodeName~"/position/altitude-ft") * FT2M;
	return geo.Coord.new().set_latlon(lat, lon, alt);
}

####################################################
#returns vector [direct distance (m), altitude difference (m)] 
# from main aircraft to myNodeName
var distAItoMainAircraft= func (myNodeName){
   mainAircraftPosition=geo.aircraft_position();
   aiAircraftPosition=any_aircraft_position(myNodeName);
   
   return [ aiAircraftPosition.direct_distance_to(mainAircraftPosition),
         aiAircraftPosition.alt() - mainAircraftPosition.alt() ];
}

####################################################
#returns course from myNodeName to main aircraft 
#
var courseToMainAircraft = func (myNodeName){
   mainAircraftPosition=geo.aircraft_position();
   aiAircraftPosition=any_aircraft_position(myNodeName);
   
   return aiAircraftPosition.course_to(mainAircraftPosition);
}

var attack_loop = func (id, myNodeName, looptime) {
  
  var loopid = getprop(""~myNodeName~"/bombable/attack-loopid"); 
	id == loopid or return;
	
  looptimealt = getprop(""~myNodeName~"/bombable/attack-looptime");
  if (looptimealt!=nil and looptimealt > 0) looptime = looptimealt; 
	
  settimer ( func { attack_loop (id, myNodeName, looptime) }, looptime);
	
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var atts= b.getNode("attacks").getValues();

	if (  ! getprop (bomb_menu_pp~"fighter-aircraft-attack") 
        or getprop("" ~ myNodeName~"/bombable/attributes/damage")==1 ) {
     setprop(""~myNodeName~"/bombable/attack-looptime", atts.attackCheckTime_sec);
     return;
  }          
  
  
  dist =  distAItoMainAircraft (myNodeName);
  var courseToMain_deg=courseToMainAircraft(myNodeName);
  #debprint ("Bombable: Checking attack parameters: ", dist[0], " ", atts.maxDistance_m, " ",atts.minDistance_m, " ",dist[1], " ",-atts.altitudeLowerCutoff_m, " ",dist[1] < atts.altitudeHigherCutoff_m );
  
  
  myHeading_deg=getprop (""~myNodeName~"/orientation/true-heading-deg");
  deltaHeading_deg=myHeading_deg - courseToMain_deg;                    
  deltaHeading_deg = math.mod (deltaHeading_deg + 180, 360) - 180;      
  
  #whether or not to continue the attack when within minDistance
  #If we are headed basically straight towards the main aircraft 
  # we continue (within continueAttackAngle_deg of straight on)
  # otherwise we break off the attack/evade      
  var continueAttack=0;
  if ( dist[0] < atts.minDistance_m ) { 
 
    if (math.abs ( deltaHeading_deg ) < atts.continueAttackAngle_deg) continueAttack=1;
     
  }
    
  
  #criteria for attacking
  if ( ! (dist[0] < atts.maxDistance_m and ( dist[0] > atts.minDistance_m or continueAttack ) and dist[1] > -atts.altitudeLowerCutoff_m and dist[1] < atts.altitudeHigherCutoff_m ) )  {
     
    #OK, no attack, we're too far away or too close & passed it, too low, too high
    #Instead we: 1. dodge if necessary 2. exit 
    #always dodge when close to main aircraft--unless we're aiming at it
    #ie, after passing it by, we make a dodge
    if (dist[0] < atts.minDistance_m) dodge( myNodeName);
    
    setprop(""~myNodeName~"/bombable/attack-looptime", atts.attackCheckTime_sec);
    setprop(""~myNodeName~"/bombable/attack-inprogress", "0"); 
    return;
  }
    
  
  #attack
  #   
  #debprint ("Bombable: Starting attack run of main aircraft with " ~ myNodeName );  
  
  var roll_deg= rand() * (atts.rollMax_deg-atts.rollMin_deg) + atts.rollMin_deg;
  
  var aiAircraftPosition = any_aircraft_position(myNodeName);
  var targetAlt_m=geo.aircraft_position().alt();
  attackCheckTimeEngaged_sec=atts.attackCheckTimeEngaged_sec;
  
  #Easy mode makes the attack manuevers less aggressive
  if (getprop(""~bomb_menu_pp~"easy-mode")) {
         
         roll_deg*=0.5; 
         #attackCheckTimeEngaged_sec*=2;
  }
         
  #Super Easy mode makes the attack manuevers less aggressive
  if (getprop(""~bomb_menu_pp~"super-easy-mode")){
      roll_deg*=0.33; 
      #attackCheckTimeEngaged_sec*=3;
  }
  
  courseToMain_deg += rand()*10-5; #keeps the moves from being so robotic

  elevMain_m = elev (geo.aircraft_position().lat(),geo.aircraft_position().lon() )*.3048;

  targetAGL_m=targetAlt_m-elevMain_m;
  


  var attack_inprogress=getprop(""~myNodeName~"/bombable/attack-inprogress");
  var attackloop_inprogress=getprop(""~myNodeName~"/bombable/attackloop-inprogress");
  var attackloop_targetAGL_m=getprop(""~myNodeName~"/bombable/attackloop-targetAGL_m"); 
  
  setprop(""~myNodeName~"/bombable/attack-inprogress", "1");

  # is this the start of our attack?  If so & if we're heading away from the   
  # target, we'll possibly do a loop to get turned around                    
  if ((attack_inprogress== nil or attack_inprogress==0) and (math.abs ( deltaHeading_deg ) >= 90) ) {
    
    #if we've already started an attack loop, keep doing it with the same targetAGL
    if (attackloop_inprogress and 
         ( attackloop_targetAGL_m != nil and attackloop_targetAGL_m > 0) ) {
            
            targetAGL_m = attackloop_targetAGL_m; 
            
    } else {
    #otherwise, we need to figure out our targetAGL
    
      currAlt_m = getprop(""~myNodeName~"/position/altitude-ft")*.3048;
      attackLoopAdd_m = rand() * (atts.climbPower + atts.divePower) - atts.divePower;  #for FG's AI to make a good dive/climb the difference in altitude must be at least 5000 ft
      
        # reduce size of climb/dive if in easy or super easy mode
        if (getprop(""~bomb_menu_pp~"easy-mode")) attackLoopAdd_m*=0.5; 
        if (getprop(""~bomb_menu_pp~"super-easy-mode"))  attackLoopAdd_m*=0.33;
  
      
      
      targetAGL_m = currAlt_m + attackLoopAdd_m - elevMain_m;
      
      alts= b.getNode("altitudes").getValues();
      if (targetAGL_m < alts.minimumAGL_m) targetAGL_m = alts.minimumAGL_m;
      if (targetAGL_m > alts.maximumAGL_m) targetAGL_m = alts.maximumAGL_m;
      
      #debprint ("Bombable: Starting attack turn/loop for ", myNodeName," targetAGL_m=", targetAGL_m);
      setprop(""~myNodeName~"/bombable/attackloop-inprogress", "1");  
      setprop(""~myNodeName~"/bombable/attackloop-targetAGL_m", targetAGL_m); 
   }  
      
  } else {
    setprop(""~myNodeName~"/bombable/attackloop-inprogress", "0");
    #debprint ("Bombable: Ending attack turn/loop for ", myNodeName);       
  }
  
  targetAlt_m= targetAGL_m + elevMain_m;                                       
  
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var alts = b.getNode("altitudes").getValues();
  if (targetAGL_m < alts.minimumAGL_m ) targetAGL_m = alts.minimumAGL_m;
  if (targetAGL_m > alts.maximumAGL_m ) targetAGL_m = alts.maximumAGL_m;
  
  #debprint ("Bombable: setprop 4275");
  setprop(""~myNodeName~"/bombable/attributes/altitudes/targetAGL_ft", targetAGL_m/.3048);
  setprop(""~myNodeName~"/bombable/attributes/altitudes/targetAGL_m", targetAGL_m);
  aircraftTurnToHeading ( myNodeName, courseToMain_deg, roll_deg, targetAlt_m);
  
  #update more frequently when engaged with the main aircraft
  setprop(""~myNodeName~"/bombable/attack-looptime", attackCheckTimeEngaged_sec);
  
  
}  
  
  

######################################################
# make an aircraft turn to a certain heading
# older/nonworking version
var aircraftTurnToHeadingOld = func (myNodeName, heading_deg=0, roll_deg=30, turntime=15){
   if (turntime <= 0) turntime=1;
   start_heading_deg=getprop (""~myNodeName~"/orientation/true-heading-deg");
   diff_heading_deg = heading_deg-start_heading_deg;
   while ( diff_heading_deg < 0 ) diff_heading_deg += 360;
   if (diff_heading_deg>180) diff_heading_deg += - 360;
   
   roll_deg=math.sgn (diff_heading_deg)*math.abs(roll_deg);
   
   aircraftRoll(myNodeName,roll_deg,turntime/3);
   settimer (func {
        firstturn_heading_deg=getprop (""~myNodeName~"/orientation/true-heading-deg"); 
        turnamount_deg=firstturn_heading_deg-start_heading_deg;
        while ( turnamount_deg < 0 ) turnamount_deg += 360;
        turnrate_degps=turnamount_deg/turntime*3;
        
        remaining_deg=diff_heading_deg - 2*turnamount_deg;
        if (remaining_deg >= turnrate_degps*turntime/3 ) {
           waittime=remaining_deg/turnrate_degps;
           #hold the roll amount
           aircraftRoll(myNodeName,0,waittime);
           settimer (func { aircraftRoll (myNodeName, -roll_deg, turntime/3)},    
              waittime);
           
        } else {
           roll_deg=math.sgn(remaining_deg)*roll_deg;
           remainingturntime=remaining_deg/turnrate_degps;
           aircraftRoll (myNodeName, roll_deg, remainingturntime)
        
        }
        
      
      }, turntime/3);
      


}

######################################
# internal - for making AI aircraft turn to certain heading
var aircraftTurnToHeadingControl = func (myNodeName, id, targetdegrees=0, rolldegrees=45, targetAlt_m="none" , correction=0 ) {
  
  var loopid = getprop(""~myNodeName~"/orientation/roll-loopid"); 
	id == loopid or return;
	
	roll_limit_deg=75; #if more than this FG AI goes a bit wacky
	if (math.abs(rolldegrees)>roll_limit_deg) rolldegrees=roll_limit_deg*math.sgn(rolldegrees);
  
   start_heading_deg=getprop (""~myNodeName~"/orientation/true-heading-deg");
   delta_heading_deg = targetdegrees-start_heading_deg;
   while ( delta_heading_deg < 0 ) delta_heading_deg += 360;
   if (delta_heading_deg>180) delta_heading_deg += - 360;
   
   rolldegrees=math.sgn (delta_heading_deg)*math.abs(rolldegrees);
   
   #if close to 180 degrees off we sometimes/randomly choose to turn the
   # opposite direction.  Just for variety/reduce robotic-ness.   
   if (math.abs(delta_heading_deg)> 160) if (rand()>0.5) rolldegrees=-rolldegrees;
   
  
  updateinterval_sec=.1;
  maxTurnTime=60; #max time to stay in this loop/a failsafe
  
  var rollTimeElapsed=getprop(""~myNodeName~ "/orientation/rollTimeElapsed");
  if (rollTimeElapsed==nil) rollTimeElapsed=0;
  setprop(""~myNodeName~ "/orientation/rollTimeElapsed", rollTimeElapsed+updateinterval_sec);
  
  
  rolltime=2; #seconds to reach rolldegrees
  if (rolltime<updateinterval_sec) rolltime=updateinterval_sec;
  #props.globals.getNode(""~myNodeName~ "/position");
  #ac_position=props.globals.getNode(myNodeName, 1).getParent().getPath();
  var rollTimeElapsed=getprop(""~myNodeName~ "/orientation/rollTimeElapsed");
  if (rollTimeElapsed==nil) rollTimeElapsed=0;
  delta_deg=rolldegrees*updateinterval_sec/rolltime; #31/second for rolltime seconds
  currRoll_deg=getprop(""~myNodeName~ "/orientation/roll-deg-bombable");
  if (currRoll_deg==nil) currRoll_deg=0;  
  targetRoll_deg= currRoll_deg+delta_deg;
  #Fg turns too quickly to be believable if the roll gets about 78 degrees 
  # or so.  
  #rolldegrees is supposed to put a limit on the max amount of roll allowed
  if (math.abs(targetRoll_deg) > math.abs(rolldegrees)) targetRoll_deg = rolldegrees;
  
  
	if (math.abs(targetRoll_deg)>roll_limit_deg) targetRoll_deg=roll_limit_deg*math.sgn(targetRoll_deg);
	rollMax_deg=getprop(""~myNodeName~"/bombable/attributes/attacks/rollMax_deg");
	if (rollMax_deg==nil) rollMax_deg = 50;
	if (math.abs(targetRoll_deg)>rollMax_deg) targetRoll_deg=rollMax_deg*math.sgn(targetRoll_deg);
	
	if (math.abs (currRoll_deg - targetRoll_deg)> 5) debprint ("Bombable: Changing roll: ",  -currRoll_deg + targetRoll_deg, " ", myNodeName, " 3085");


	
	#debprint ("Bombable: setprop 2937");
  #debprint ("Bombable: Setting roll-deg for ", myNodeName , " to ", targetRoll_deg, " 3086");
  setprop (""~myNodeName~ "/orientation/roll-deg", targetRoll_deg);
  setprop (""~myNodeName~ "/orientation/roll-deg-bombable", targetRoll_deg);
  
  #set the target altitude as well.  flight/target-alt is in ft  
  if (targetAlt_m != "none") {
      var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
      var evas= b.getNode("evasions").getValues();
      var alts = b.getNode("altitudes").getValues();
      
      targetAlt_ft=targetAlt_m / FT2M;
      
  
      currElev_m=elev (any_aircraft_position(myNodeName).lat(),geo.aircraft_position(myNodeName).lon() )*.3048;
  
      if (targetAlt_m - currElev_m  < alts.minimumAGL_m ) targetAlt_ft = (alts.minimumAGL_m + currElev_m)/.3048;
      if (targetAlt_m - currElev_m  > alts.maximumAGL_m ) targetAlt_ft = (alts.maximumAGL_m + currElev_m)/.3048;
  
      #debprint ("Bombable: setprop 2955");
      setprop ( "" ~ myNodeName ~ "/controls/flight/target-alt", targetAlt_ft );
      
      currAlt_ft=getprop ("" ~ myNodeName ~ "/position/altitude-ft");
      dodgeAltAmount_ft=targetAlt_ft-currAlt_ft;
      if (dodgeAltAmount_ft > evas.dodgeAltMax_ft) dodgeAltAmount_ft = evas.dodgeAltMax_ft;
      if (dodgeAltAmount_ft < evas.dodgeAltMin_ft) dodgeAltAmount_ft = evas.dodgeAltMin_ft; 
      dodgeVertSpeed_fps=0;
      
      
      
       currRoll_deg=getprop (""~myNodeName~"/orientation/roll-deg");
       
       
       if ( dodgeAltAmount_ft > 500 )  dodgeVertSpeed_fps = math.abs ( evas.dodgeVertSpeedClimb_fps );
       elsif ( dodgeAltAmount_ft > 200 )  dodgeVertSpeed_fps = math.abs ( evas.dodgeVertSpeedClimb_fps/2 );
       elsif ( dodgeAltAmount_ft > 100 )  dodgeVertSpeed_fps = math.abs ( evas.dodgeVertSpeedClimb_fps/4 ); 
       elsif ( dodgeAltAmount_ft > 0 )  dodgeVertSpeed_fps = math.abs ( evas.dodgeVertSpeedClimb_fps/6 );
       elsif  ( dodgeAltAmount_ft < -500 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps);
       elsif  ( dodgeAltAmount_ft < -200 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps/2);
       elsif  ( dodgeAltAmount_ft < -100 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps/4);
       elsif  ( dodgeAltAmount_ft < 0 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps/6);
       
       if ( dodgeAltAmount_ft < 0 )  dodgeVertSpeed_fps = - math.abs ( evas.dodgeVertSpeedDive_fps * dodgeAltAmount_ft/evas.dodgeAltMin_ft ); 
       
       #velocities/vertical-speed-fps in FG 2.2.4 ranges from -25 to +50 fps.  It only reaches those extremese if target_alt is about 5000 greater
       #or lower than the current altitude.
       #This is one reason the FG AI traffic seems sedate and not like fighter planes.  A normal Zero doing an ordinary barrel roll ranges from
       # about -250 fps to _250 fps.
       # This needs to be set on individually in the settings for each aircraft, but as a general rule -250 to 250 is a range to shoot
       # for, for decent realism.
       
       # We reduce the amount of climb/dive possible depending on the current roll angle (can't climb/dive rapidly if rolled to 90 degrees . . . )    
       dodgeVertSpeed_fps*= math.abs(math.cos(currRoll_deg/rad2degrees));
       #dodgeVertSpeed_fps*=1000;        
       
       #vert-speed prob                                                                     
       
       # If we want a change in vertical speed then we are going to change /velocities/vertical-speed-fps
       # directly.  But by a max of 10 FPS at a time, otherwise it is too abrupt. 
       if (dodgeVertSpeed_fps!=0){
          var curr_vertical_speed_fps= getprop ("" ~ myNodeName ~ "/velocities/vertical-speed-fps"); 
          vertSpeedChange_fps=dodgeVertSpeed_fps-curr_vertical_speed_fps;
          if (vertSpeedChange_fps>25) vertSpeedChange_fps=25;
          if (vertSpeedChange_fps<-25) vertSpeedChange_fps=-25;
          setprop ("" ~ myNodeName ~ "/velocities/vertical-speed-fps", curr_vertical_speed_fps + vertSpeedChange_fps);
       }
       #debprint ("Attacking: ", myNodeName, " ", dodgeAltAmount_ft, " ", dodgeVertSpeed_fps);
      
      
  }
  
  #debprint ("2672 ", getprop (""~myNodeName~"/controls/flight/target-alt")) ;
  
  #debprint("Bombable: RollControl: delta=",delta_deg, " ",targetRoll_deg," ", myNodeName);
  # Make it roll:    
  setprop(""~myNodeName~ "/orientation/rollTimeElapsed", rollTimeElapsed+updateinterval_sec);
  
  cutoff=math.abs(rolldegrees)/5;
  if (cutoff<1) cutoff=1;
  #wait a while & then roll back.  correction makes sure we don't keep
  # doing this repeatedly                              
  if ( math.abs(delta_heading_deg) > cutoff and rollTimeElapsed < maxTurnTime ) settimer (func { aircraftTurnToHeadingControl(myNodeName, loopid, targetdegrees, rolldegrees, targetAlt_m)}, updateinterval_sec );
  else { 
    setprop(""~myNodeName~ "/orientation/rollTimeElapsed", 0);
    #debprint ("Bombable: Ending aircraft turn-to-heading routine");
    aircraftRoll(myNodeName, -rolldegrees, 2);
    #debprint ("Bombable: setprop 3008");
    setprop(""~myNodeName~"/controls/flight/target-hdg", targetdegrees);
  }   
}

######################################################
# make an aircraft turn to a certain heading
#
var aircraftTurnToHeading = func (myNodeName, targetdegrees=0, rolldegrees=45, targetAlt_m="none" ) {                                                                          
  #if (crashListener != 0 ) return;
  #debprint ("Bombable: Starting aircraft turn-to-heading routine");
  #same as roll-loopid ID because we can't do this & roll @ the same time
  var loopid = getprop(""~myNodeName~"/orientation/roll-loopid");
  if (loopid==nil) loopid=0; 
  loopid +=1;
  setprop(""~myNodeName~"/orientation/roll-loopid", loopid);
  
  setprop(""~myNodeName~ "/orientation/rollTimeElapsed", 0);

  #debprint ("Bombable: Starting turn-to-heading routine, loopid=",loopid, " ", rolldegrees, " ", targetdegrees);
  props.globals.getNode(""~myNodeName~ "/orientation/rollTimeElapsed", 1).setValue( 0 );
  currRoll_deg=getprop (""~myNodeName~ "/orientation/roll-deg");
  if (currRoll_deg==nil) currRoll_deg=0;
  props.globals.getNode(""~myNodeName~ "/orientation/roll-deg-bombable", 1).setValue( currRoll_deg );
  aircraftTurnToHeadingControl (myNodeName, loopid, targetdegrees, rolldegrees, targetAlt_m);  
  #turn it off after 10 seconds
  #settimer (func {removelistener(crashListener); crashListener=0;}, 10);
  
}


######################################
# internal - for making AI aircraft roll/turn
var aircraftRollControl = func (myNodeName, id, rolldegrees=-90, rolltime=5) {
  
  var loopid = getprop(""~myNodeName~"/orientation/roll-loopid"); 
	id == loopid or return;
	
	roll_limit_deg=75;
	if (math.abs(rolldegrees)>roll_limit_deg) rolldegrees=roll_limit_deg*math.sgn(rolldegrees);
  
  updateinterval_sec=.1;
  if (rolltime<updateinterval_sec) rolltime=updateinterval_sec;
  #props.globals.getNode(""~myNodeName~ "/position");
  #ac_position=props.globals.getNode(myNodeName, 1).getParent().getPath();
  var rollTimeElapsed=getprop(""~myNodeName~ "/orientation/rollTimeElapsed");
  if (rollTimeElapsed==nil) rollTimeElapsed=0;
  delta_deg=rolldegrees*updateinterval_sec/rolltime; #31/second for rolltime seconds
  currRoll_deg=getprop(""~myNodeName~ "/orientation/roll-deg-bombable");
  if (currRoll_deg==nil) currRoll_deg=0;  
  targetRoll_deg= currRoll_deg+delta_deg;
  #Fg turns too quickly to be believable if the roll gets about 78 degrees 
  # or so.  
  #Fg seems to go totally whacky if the roll gets to, or close to, 90 degrees
  if (targetRoll_deg >  roll_limit_deg ) targetRoll_deg = roll_limit_deg;
  if (targetRoll_deg < -roll_limit_deg) targetRoll_deg =-roll_limit_deg;
  
  
  if (math.abs(targetRoll_deg)>roll_limit_deg) targetRoll_deg=roll_limit_deg*math.sgn(targetRoll_deg);
	rollMax_deg=getprop(""~myNodeName~"/bombable/attributes/attacks/rollMax_deg");
	if (rollMax_deg==nil) rollMax_deg = 50;
	if (math.abs(targetRoll_deg)>rollMax_deg) targetRoll_deg=rollMax_deg*math.sgn(targetRoll_deg);
  
 	if (math.abs (currRoll_deg - targetRoll_deg)> 5) debprint ("Bombable: Changing roll: ", currRoll_deg - targetRoll_deg, " ", myNodeName, " 3238");
  
  #debprint ("Bombable: setprop 3071");     
  #debprint ("Bombable: Setting roll-deg for ", myNodeName , " to ", targetRoll_deg, " 3240");
  setprop (""~myNodeName~ "/orientation/roll-deg", targetRoll_deg);
  
  
  #we keep the 'uncorrected' amount internally because normal behavior
  # is to go to a certain degree & then return.  If we capped at 85 deg
  # then we would end up returning too far    
  setprop (""~myNodeName~ "/orientation/roll-deg-bombable", currRoll_deg+delta_deg);
  
  #debprint("Bombable: RollControl: delta=",delta_deg, " ",targetRoll_deg," ", myNodeName);
  # Make it roll:    
  setprop(""~myNodeName~ "/orientation/rollTimeElapsed", rollTimeElapsed+updateinterval_sec);
                            
  if ( rollTimeElapsed < rolltime ) settimer (func { aircraftRollControl(myNodeName, loopid, rolldegrees, rolltime)}, updateinterval_sec );
  else { 
    setprop(""~myNodeName~ "/orientation/rollTimeElapsed", 0);
    #debprint ("Bombable: Ending aircraft roll routine");
  }   
}

var aircraftRoll = func (myNodeName, rolldegrees=-60, rolltime=5) {
                                                                          
  #if (crashListener != 0 ) return;
  #debprint ("Bombable: Starting aircraft roll routine");
  
  
  var loopid = getprop(""~myNodeName~"/orientation/roll-loopid");
  if (loopid==nil) loopid=0; 
  loopid +=1;
  setprop(""~myNodeName~"/orientation/roll-loopid", loopid);

  #debprint ("Bombable: Starting roll routine, loopid=",loopid, " ", rolldegrees, " ", rolltime);
  props.globals.getNode(""~myNodeName~ "/orientation/rollTimeElapsed", 1).setValue( 0 );
  currRoll_deg=getprop (""~myNodeName~ "/orientation/roll-deg");
  if (currRoll_deg==nil) currRoll_deg=0;
  props.globals.getNode(""~myNodeName~ "/orientation/roll-deg-bombable", 1).setValue( currRoll_deg );
  aircraftRollControl(myNodeName, loopid, rolldegrees, rolltime);
  
  #turn it off after 10 seconds
  #settimer (func {removelistener(crashListener); crashListener=0;}, 10);
  
}


var aircraftCrashControl = func (myNodeName) {
  
  var loopTime=.1;
  #props.globals.getNode(""~myNodeName~ "/position");
  #ac_position=props.globals.getNode(myNodeName, 1).getParent().getPath();
  elapsed=getprop(""~myNodeName~ "/position/crashTimeElapsed");
  if (elapsed==nil) elapsed=0;
  delta_ft=5.87*elapsed/(elapsed+5); #we're using 176 ft/sec as the terminal velocity & runninG this loop 30X per second
  # t/(t+5) is a crude approximation of tanh(t), which is the real equation
  # to use for terminal velocity under gravity.  However tanh is very expensive
  # and since we have to approximate the coefficient of drag and other variables
  # related to the damaged aircraft anyway, based on very incomplete information,
  # this approximation is about good enough and definitely much faster than tanh        
  currAlt_ft=getprop(""~myNodeName~ "/position/altitude-ft");
  
  #debprint ("Bombable: setprop 3128");  
  setprop (""~myNodeName~ "/position/altitude-ft", currAlt_ft-delta_ft);
  debprint("Bombable: CrashControl: delta=",delta_ft, " ",currAlt_ft," ", myNodeName);
  # Make it roll:  
 
  #debprint ("Bombable: Setting roll-deg for ", myNodeName , " to ", -elapsed*5*delta_ft, " 1610");    
  setprop (""~myNodeName~ "/orientation/roll-deg", -elapsed*5*delta_ft);  
  setprop(""~myNodeName~ "/position/crashTimeElapsed", elapsed+loopTime);
                            
  onGround= getprop (""~myNodeName~"/bombable/on-ground");
  if (onGround==nil) onGround=0;
  
  if (myNodeName=="") {
  
      
      var objectGeoCoord = geo.Coord.new();
	
	    objectGeoCoord.set_latlon(getprop("position/latitude-deg"), getprop("position/longitude-deg"), getprop("position/altitude-ft")*0.3048);
	    
      var velocity_kt=getprop("/velocities/groundspeed-kt");
      var heading_deg= getprop("/orientation/heading-deg");
      
	
	    objectGeoCoord.apply_course_distance(heading_deg, velocity_kt* 0.514444444 * loopTime);  
	    
	    #debprint ("Bombable: setprop 3151");
      setprop("/position/latitude-deg", objectGeoCoord.lat());
	    setprop("/position/longitude-deg", objectGeoCoord.lon());
	
	    if (getprop("/position/altitude-ft") - elev( objectGeoCoord.lat(),objectGeoCoord.lon()) <=5) onGround=1;
	    
	    #exit this immediately if we become un-crashed
	    if (!getprop("/sim/crashed")) return;
	
  }    
	    
	
      

  # elevation of -1371 ft is a failsafe (lowest elevation on earth); so is 
  #   elapsed, so that we don't get stuck in this routine forever  
  if ( onGround != 1 and currAlt_ft>-1371 and elapsed < 240 ) settimer (func { aircraftCrashControl(myNodeName)}, loopTime );
  else { 
    setprop(""~myNodeName~ "/position/crashTimeElapsed", 0);
    #we should be crashed at this point but just in case:
    if ( currAlt_ft<=-1371 ) add_damage(1, myNodeName,"crash");
    debprint ("Bombable: Ending aircraft crash routine");
  }   
}

var aircraftCrash = func (myNodeName) {
  if (myNodeName=="/environment" or myNodeName=="environment") myNodeName="";
                               
  #if (myNodeName=="") debprint ("Bombable: Updating main aircraft 3154");
                                                                                    
  #if (crashListener != 0 ) return;
  debprint ("Bombable: Starting aircraft crash routine");
  elapsed=props.globals.getNode(""~myNodeName~ "/position/crashTimeElapsed", 1).getValue( );
  if (elapsed==nil) elapsed=0;
  debprint ("Bombable: Starting crash routine, elapsed=",elapsed);
  if (elapsed!=0) return;
  props.globals.getNode(""~myNodeName~ "/position/crashTimeElapsed", 1).setValue( .1 );
  aircraftCrashControl(myNodeName);
  
  #turn it off after 10 seconds
  #settimer (func {removelistener(crashListener); crashListener=0;}, 10);
  
}


################################################################
#function adds damage to an AI aircraft 
#(called by the fire loop and ballistic impact
#listener function, typically)
# damagetype is "weapon" or "nonweapon".  nonweapon damage (fire, crash into 
# ground, etc) is not passed on via multiplayer (fire, crash, etc damage is 
# handled on their end and if all connected players add fire & crash damage via 
# multiplayer, too, then it creates a nasty cascade)
# Also slows down the vehicle whenever damage increases.
# vuls.damageVulnerability multiplies the damage, with an M1 tank = 1.  vuls.damageVulnerability=2
# means 2X the damage.
# maxSpeedReduce is a percentage, the maximum percentage to reduce speed
# in one step.  An airplane might keep moving close to the same speed
# even if the engine dies completely.  A tank might stop forward motion almost
# instantly.
var add_damage = func(damageRise, myNodeName, damagetype="weapon" ) {
  node = props.globals.getNode(myNodeName);
  
  
 
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var vuls= b.getNode("vulnerabilities").getValues();
	var spds= b.getNode("velocities").getValues();  
  var livs= b.getNode ("damageLiveries").getValues();
  var liveriesCount= b.getNode ("damageLiveries/count").getValue();
  var type=node.getName();
  var damageValue = getprop(""~myNodeName~"/bombable/attributes/damage");
  if ( damageValue==nil ) damageValue=0;
    
  var origDamageRise = damageRise;
  #make sure it's in range 0-1.0
  if(damageRise > 1.0)
			damageRise = 1.0;
	elsif(damageRise < 0.0)
			damageRise = 0.0;
  
  # update bombable/attributes/damage: 0.0 mean no damage, 1.0 mean full damage
  prevDamageValue=damageValue;
	if(damageValue < 1.0) 
		damageValue += damageRise;
		
  #make sure it's in range 0-1.0
  if(damageValue > 1.0)
			damageValue = 1.0;
	elsif(damageValue < 0.0)
			damageValue = 0.0;
	setprop(""~myNodeName~"/bombable/attributes/damage", damageValue);
  
     

	#debprint ("damageValue=",damageValue);
	callsign=node.getNode("callsign", 1).getValue();
  if (callsign==nil or callsign=="") callsign=node.getName() ~ "[" ~ node.getIndex() ~ "]";
    	
	#if (int(damageValue * 20 )!=int(prevDamageValue * 20 )) {
	# 
	# 
	    	
  if ( damageValue>prevDamageValue ) {
  
  damageRiseDisplay= int( damageRise*100 + 0.5);
	if (damageRise <.01) damageRiseDisplay = sprintf ("%1.2f",damageRise*100);
  elsif (damageRise <.1) damageRiseDisplay = sprintf ("%1.1f",damageRise*100);
  
  
     var msg= "Damage added: " ~ damageRiseDisplay ~ "% - Total damage: " ~ int( damageValue * 100 +0.5) ~ "% for " ~  string.trim(callsign);
     debprint ("Bombable: " ~ msg ~ " (" ~ myNodeName ~ ", " ~ origDamageRise ~")" );
     
     #Always display the message if a weapon hit or large damageRise. Otherwise
     #only display about 1 in 20 of the messages.
     #If we don't do this the small 0.4 damageRises from fires overwhelm the message area
     #and we don't know what's going on.
     if (damagetype == "weapon" or damageRise >4 or rand()<.05) targetStatusPopupTip (msg, 20);

      
  }   	

  if ( damageValue==1 and damageValue>prevDamageValue and type == "aircraft") {
    aircraftCrash (myNodeName);
  }

  onGround= getprop (""~myNodeName~"/bombable/on-ground");
  if (onGround==nil) onGround=0;
  
  if (onGround) {
  
      #all to a complete stop
      #debprint ("Bombable: setprop 3263");
      setprop(""~myNodeName~"/controls/tgt-speed-kts", 0);

      setprop(""~myNodeName~"/controls/flight/target-spd", 0); 
         
      setprop(""~myNodeName~"/velocities/true-airspeed-kt", 0); 
      
      #we hit the ground, now we are 100% dead
      setprop(""~myNodeName~"/bombable/attributes/damage", 1);
      
      if (liveriesCount > 0 and liveriesCount != nil ) {
      
      livery = livs.damageLivery [ int ( damageValue * ( liveriesCount - 1 ) ) ];
  	   setprop(""~myNodeName~"/bombable/texture-corps-path", 
         livery );
		  }
    	   	    	
       
      return;
  } 
  
  
  #debprint (damageRise, " ", myNodeName);
  #max speed reduction due to damage, in %
   
  minSpeed = spds.minSpeed_kt;
   
  #if the object is "on the ground" or "completely sunk" due to damage
  # then we make it come to a stop much more dramatically  
  #if ( onGround){
  #   if (spds.maxSpeedReduce_percent<20)  spds.maxSpeedReduce_percent=20;
  #   minSpeed=0;
  #   onGround=1;
  #} else onGround=0;
  
  
  #for moving objects (ships & aircraft), reduce velocity each time damage added
	#eventually  stopping when damage = 1.  
	#But don't reduce speed below minSpeed.
	#we put it here outside the "if" statement so that burning
	#objects continue to slow/stop even if their damage is already at 1
	# (this happens when file/reset is chosen in FG)
	var tgt_spd_kts=getprop (""~myNodeName~"/controls/tgt-speed-kts"); 
  if (tgt_spd_kts == nil ) tgt_spd_kts=0;

	var flight_tgt_spd=getprop (""~myNodeName~"/controls/flight/target-spd"); 
  if (flight_tgt_spd == nil ) flight_tgt_spd=0;
    
  var true_spd=getprop (""~myNodeName~"/velocities/true-airspeed-kt");
  if (true_spd == nil ) true_spd=0; 
  
  maxSpeedReduceProp=1-spds.maxSpeedReduce_percent/100;  #spds.maxSpeedReduce_percent is a percentage
  speedReduce= 1-damageValue;
  if (speedReduce < maxSpeedReduceProp) speedReduce=maxSpeedReduceProp;
  
  pitch=getprop (""~myNodeName~"/orientation/pitch-deg");
  
  var node = props.globals.getNode(myNodeName);
  

  #debprint ("type=", type);
  
  if (type=="aircraft")  {
      
      #if (pitch > - 90)
      #      setprop (""~myNodeName~"/orientation/pitch-deg", pitch-1);
            


      if ( damageValue >= 0.75 and !onGround) {    
          #debprint ("Bombable: setprop 3333");
          if (flight_tgt_spd > minSpeed) 
              setprop(""~myNodeName~"/controls/flight/target-spd", 
              flight_tgt_spd * speedReduce);
          else setprop(""~myNodeName~"/controls/flight/target-spd", 
             minSpeed);  
      }
  
  #ships etc we control all these ways, making sure the speed decreases but
  #not below the minimum allowed
  }  else {     
    #debprint ("Bombable: setprop 3344");
    if (tgt_spd_kts > minSpeed)  
      setprop(""~myNodeName~"/controls/tgt-speed-kts", 
         tgt_spd_kts * speedReduce);    

    if (flight_tgt_spd > minSpeed) 
      setprop(""~myNodeName~"/controls/flight/target-spd", 
         flight_tgt_spd * speedReduce);
         
    if (true_spd > minSpeed) 
      setprop(""~myNodeName~"/velocities/true-airspeed-kt", 
         true_spd * speedReduce);				
  }  
    

		var fireStarted=getprop(""~myNodeName~"/bombable/fire-particles/fire-burning");
		if (fireStarted == nil ) fireStarted=0;
   	var damageEngineSmokeStarted=getprop(""~myNodeName~"/bombable/fire-particles/damagedengine-burning");
   	if (damageEngineSmokeStarted == nil ) damageEngineSmokeStarted = 0; 
   	
   	#don't print this for every fire damage rise, but otherwise . . .
   	#if (!fireStarted or damageRise > vuls.fireDamageRate_percentpersecond * 2.5 ) debprint ("Damage added: ", damageRise, ", Total damage: ", damageValue);
    #Start damaged engine smoke but only sometimes; greater chance when hitting an aircraft
   	
   	
   	if (!damageEngineSmokeStarted and !fireStarted and rand() < damageRise * vuls.engineDamageVulnerability_percent / 2 ) 
        startSmoke("damagedengine",myNodeName);
       

	  # start fire if there is enough damages.
		#if(damageValue >= 1 - vuls.fireVulnerability_percent/100 and !fireStarted ) {
				
    #a percentage change of starting a fire with each hit
		if( rand() < .035 * damageRise * (vuls.fireVulnerability_percent) and !fireStarted ) {
		
		  debprint ("Bombable: Starting fire");
			
			#use small, medium, large smoke column depending on vuls.damageVulnerability
			#(high vuls.damageVulnerability means small/light/easily damaged while 
			# low vuls.damageVulnerability means a difficult, hardened target that should burn
			# more vigorously once finally on fire)   
			#var fp="";      
			#if (vuls.explosiveMass_kg < 1000 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-small.xml"; }
			#elsif (vuls.explosiveMass_kg > 50000 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-large.xml"; }
			#else {fp="AI/Aircraft/Fire-Particles/fire-particles.xml";} 

      #small, med, large fire depending on size of hit that caused it			
			var fp="";      
			if (damageRise < 0.08 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-small.xml"; }
			elsif (damageRise > 0.25 ) { fp="AI/Aircraft/Fire-Particles/fire-particles-large.xml"; }
			else {fp="AI/Aircraft/Fire-Particles/fire-particles.xml";}
      
      startFire(myNodeName, fp); 
      #only one damage smoke at a time . . . 
      deleteSmoke("damagedengine",myNodeName);
      
      
      #fire can be extinguished up to MaxTime_seconds in the future,
      #if it is extinguished we set up the damagedengine smoke so 
      #the smoke doesn't entirely go away, but no more damage added
      if ( rand() * 100 < vuls.fireExtinguishSuccess_percentage ) {
      
         settimer (func { 
           deleteFire (myNodeName);
           startSmoke("damagedengine",myNodeName); 
           } ,            
           rand() * vuls.fireExtinguishMaxTime_seconds + 15 ) ;
      }
           
;
#      debprint ("started fire");
    
    #Set livery to the one corresponding to this amount of damage
    if (liveriesCount > 0 and liveriesCount != nil ) {
      livery = livs.damageLivery [ int ( damageValue * ( liveriesCount - 1 ) ) ];
  	   setprop(""~myNodeName~"/bombable/texture-corps-path", 
         livery );
		
		}
	}
	#only send damage via multiplayer if it is weapon damage from our weapons
	if (type=="multiplayer" and damagetype == "weapon") {
     mp_send_damage(myNodeName, damageRise);
  };
}


####################################################
#functions to increment loopids
#these are called on init and destruct (which should be called
#when the object loads/unloads)
#When the loopid increments it will kill any timer functions
#using that loopid for that object.  (Otherwise they will just
#continue to run indefinitely even though the object itself is unloaded)
var inc_loopid = func (nodeName="", loopName="") {
  if (nodeName=="") nodeName="/environment";
	var loopid = getprop(""~nodeName~"/bombable/" ~ loopName ~ "-loopid"); 
	if ( loopid == nil ) loopid=0;
	loopid += 1;
	setprop(""~nodeName~"/bombable/" ~ loopName ~ "-loopid", loopid);
	return loopid;
}



#####################################################
# Set livery color (including normal through 
# slightly and then completely damaged)
#
# Example:
#
# liveries = [ 
#          "Models/livery_nodamage.png", 
#          "Models/livery_slightdamage.png", 
#          "Models/livery_highdamage.png"
#  ];
#  bombable.set_livery (cmdarg().getPath(), liveries);

var set_livery = func (nodeName, liveries) {
  var node = props.globals.getNode(nodeName);
  var livs = node.getNode ("/bombable/attributes/damageLiveries",1).getValues();
  
  #set new liveries, also set the count to the number
  #of liveries installed
  if (liveries == nil or size ( liveries) == 0 ) {
     livs.removeChildren();
     node.getNode("bombable/attributes/damageLiveries/count", 1).setValue( 0 );
    
  } else {
    node.getNode("bombable/attributes/damageLiveries/count", 1).setValue( size (liveries) );
    node.getNode ("/bombable/attributes/damageLiveries",1).setValues( { damageLivery : liveries } );
    
  	#current color (we'll set it to the undamaged color;
    #if the object is on fire/damage damaged this will soon be updated)
    #by the timer function
    #To actually work, the aircraft's xml file must be set up with an 
    #animation to change the texture, keyed to the bombable/texture-corps-path
    #property
  	node.getNode("bombable/texture-corps-path", 1).setValue(liveries[0]);
  }
}

var checkRange = func (v=nil, low=nil, high=nil, default=1) {

 if ( v == nil ) v=default;
 if ( low != nil and v < low  ) v = low;
 if ( high != nil and v > high ) v = high;
 
 return v;
}

var checkRangeHash = func (b=nil, v=nil, low=nil, high=nil, default=1) {
 if (contains (b, v)) return checkRange (b[v],low, high, default)
 else return default;
 
}


#####################################################
# initialize: Do sanity checking, then
# slurp the pertinent properties into
# the object's node tree under sub-node "bombable"
# so that they can be access by all the different
# subroutines
# 
# Now if you just need a certain property or two you can simply read it 
# with getprops.
#
# But for those routines that use many/all we can just grab them all with
#  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
#  bomb= b.getValues();  #all under the "bombable/attributes" branch
# Then use values like bomb.dimensions.width_m etc.
# Normally don't do this as it slurps in MANY values
#
# But (better if you only need one sub-branch)
#  dims= b.getNode("dimensions").getValues(); 
# Gets values from subbranch 'dimensions'.
# Then your values are dims.width_m etc.
#
#
var initialize = func ( b ){

 #only allow initialization for ai & multiplayer objects
 # in FG 2.4.0 we're having trouble with strange(!?) init requests from
 # joysticks & the like  
 var init_allowed=0;
 if (find ("/ai/models/", b.objectNodeName ) != -1 ) init_allowed=1;
 if (find ("/multiplayer/", b.objectNodeName ) != -1 ) init_allowed=1;

 if (init_allowed!=1) {
   debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", b.objectNodeName);
   return;
 } 
 


 #do sanity checking on input
 #also calculate a few values that will be useful later on &
 #add them to the object

 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
 var inited= getprop(""~b.objectNodeName~"/bombable/attributes-initialized");
 
 if (inited==1) {
   debprint ("Bombable: Attempt to re-initialize attributes when it has not been de-initialized; aborting re-initialization. ", b.objectNodeName);
   return;
 } 
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 setprop(""~b.objectNodeName~"/bombable/attributes-initialized", 1);
 


  #initialize damage level of this object
  b.damage = 0;
  b.updateTime_s = checkRange ( b.updateTime_s, 0, 10, 1);
    
  ##altitudes sanity checking
  b.altitudes.wheelsOnGroundAGL_m = checkRange ( b.altitudes.wheelsOnGroundAGL_m, -1000000, 1000000, 0 );
  b.altitudes.minimumAGL_m = checkRange ( b.altitudes.minimumAGL_m, -1000000, 1000000, 0 );
  b.altitudes.maximumAGL_m = checkRange ( b.altitudes.maximumAGL_m, -1000000, 1000000, 0 );
  #keep this one negative or zero:
  b.altitudes.crashedAGL_m = checkRange ( b.altitudes.crashedAGL_m, -1000000, 0, -0.001 );
  if (b.altitudes.crashedAGL_m == 0 )b.altitudes.crashedAGL_m = -0.001;
  
  b.altitudes.initialized=0; #this is how ground_loop knows to initialize the alititude on its first call
  b.altitudes.wheelsOnGroundAGL_ft=b.altitudes.wheelsOnGroundAGL_m/.3048;
  b.altitudes.minimumAGL_ft=b.altitudes.minimumAGL_m/.3048;
  b.altitudes.maximumAGL_ft=b.altitudes.maximumAGL_m/.3048;
  b.altitudes.crashedAGL_ft=b.altitudes.crashedAGL_m/.3048;
    
  #crashedAGL must be at least a bit lower than minimumAGL
  if (b.altitudes.crashedAGL_m > b.altitudes.minimumAGL_m )
       b.altitudes.crashedAGL_m = b.altitudes.minimumAGL_m - 0.001;
       
  #evasions sanity checking
  b.evasions.dodgeDelayMax_sec = checkRange ( b.evasions.dodgeDelayMax_sec, 0, 600, 30 );
  b.evasions.dodgeDelayMin_sec = checkRange ( b.evasions.dodgeDelayMin_sec, 0, 600, 5 );
  if (b.evasions.dodgeDelayMax_sec< b.evasions.dodgeDelayMin_sec) 
    b.evasions.dodgeDelayMax_sec=b.evasions.dodgeDelayMin_sec;
  
  b.evasions.dodgeMax_deg = checkRange ( b.evasions.dodgeMax_deg, 0, 180, 90 );
  b.evasions.dodgeMin_deg = checkRange ( b.evasions.dodgeMin_deg, 0, 180, 30 );
  if (b.evasions.dodgeMax_deg< b.evasions.dodgeMin_deg) 
    b.evasions.dodgeMax_deg=b.evasions.dodgeMax_deg;
  
  
  b.evasions.dodgeROverLPreference_percent = checkRange ( b.evasions.dodgeROverLPreference_percent, 0, 100, 50 );
  
  b.evasions.dodgeAltMax_m = checkRangeHash ( b.evasions, "dodgeAltMax_m", -100000, 100000, 20 ); 
  b.evasions.dodgeAltMin_m = checkRangeHash ( b.evasions, "dodgeAltMin_m", -100000, 100000, -20 );         
  if (b.evasions.dodgeAltMax_m < b.evasions.dodgeAltMin_m) 
    b.evasions.dodgeAltMax_m = b.evasions.dodgeAltMin_m;
  b.evasions.dodgeAltMin_ft = b.evasions.dodgeAltMin_m/.3048;
  b.evasions.dodgeAltMax_ft = b.evasions.dodgeAltMax_m/.3048;
  
  
  b.evasions.dodgeVertSpeedClimb_mps = checkRangeHash (b.evasions, "dodgeVertSpeedClimb_mps", 0, 3000, 50 );
  b.evasions.dodgeVertSpeedDive_mps = checkRangeHash ( b.evasions, "dodgeVertSpeedDive_mps", 0, 5000, 100 );          
  b.evasions.dodgeVertSpeedClimb_fps = b.evasions.dodgeVertSpeedClimb_mps/.3048;
  b.evasions.dodgeVertSpeedDive_fps = b.evasions.dodgeVertSpeedDive_mps/.3048;
    
  
  
  
  ##dimensions sanity checking  
  b.dimensions.width_m = checkRange ( b.dimensions.width_m, 0, nil , 30 );
  b.dimensions.length_m = checkRange ( b.dimensions.length_m, 0, nil, 30 );
  b.dimensions.height_m = checkRange ( b.dimensions.height_m, 0, nil, 30 );
  b.dimensions.damageRadius_m =checkRange ( b.dimensions.damageRadius_m, 0, nil, 6 );
  #add some helpful new:
  #   
  b.dimensions.width_ft = b.dimensions.width_m/.3048;
  b.dimensions.length_ft = b.dimensions.length_m/.3048;
  b.dimensions.height_ft = b.dimensions.height_m/.3048;   
  b.dimensions.damageRadius_ft = b.dimensions.damageRadius_m/.3048;
  
 
  ## velocities sanity checking
  b.velocities.maxSpeedReduce_percent = checkRange ( b.velocities.maxSpeedReduce_percent, 0, 100, 1 );
  b.velocities.minSpeed_kt = checkRange (b.velocities.minSpeed_kt, 0, nil, 0 );
  b.velocities.damagedAltitudeChangeMaxRate_meterspersecond = checkRange (b.velocities.damagedAltitudeChangeMaxRate_meterspersecond, 0, nil, 0.5 );

  ##damage sanity checking
  if (b.vulnerabilities.damageVulnerability<=0) b.vulnerabilities.damageVulnerability = 1;
  b.vulnerabilities.engineDamageVulnerability_percent = checkRange (b.vulnerabilities.engineDamageVulnerability_percent, 0, 100, 1 );
  b.vulnerabilities.fireVulnerability_percent = checkRange (b.vulnerabilities.fireVulnerability_percent, -1, 100, 20 );             
  b.vulnerabilities.fireDamageRate_percentpersecond = checkRange (b.vulnerabilities.fireDamageRate_percentpersecond, 0, 100, 1 );
  b.vulnerabilities.fireExtinguishMaxTime_seconds = checkRange (b.vulnerabilities.fireExtinguishMaxTime_seconds, 0, nil, 3600 );
  b.vulnerabilities.fireExtinguishSuccess_percentage = checkRange ( b.vulnerabilities.fireExtinguishSuccess_percentage, 0, 100, 10 );
  b.vulnerabilities.explosiveMass_kg = checkRange ( b.vulnerabilities.explosiveMass_kg, 0, 10000000, 1000 );

  ##attacks sanity checking
  if (b.attacks.minDistance_m < 0) b.attacks.maxDistance_m=100;
  if (b.attacks.maxDistance_m < b.attacks.minDistance_m ) b.attacks.maxDistance_m=2*b.attacks.minDistance_m;
  if (b.attacks.rollMin_deg == nil ) b.attacks.rollMin_deg=30;  
  if (b.attacks.rollMin_deg < 0) b.attacks.rollMin_deg=100;  
  if (b.attacks.rollMax_deg == nil ) b.attacks.rollMax_deg=80;
  if (b.attacks.rollMax_deg < b.attacks.rollMax_deg) b.attacks.rollMax_deg=b.attacks.rollMin_deg + 30;
  if (b.attacks.climbPower == nil ) b.attacks.climbPower=2000;
  if (b.attacks.climbPower < 0) b.attacks.climbPower=2000;    
  if (b.attacks.divePower == nil ) b.attacks.divePower=4000;
  if (b.attacks.divePower < 0) b.attacks.divePower=4000;       
  if (b.attacks.attackCheckTime_sec == nil ) b.attacks.attackCheckTime_sec=15;
  if (b.attacks.attackCheckTime_sec < 0.1) b.attacks.attackCheckTime_sec=0.1;
  if (b.attacks.attackCheckTimeEngaged_sec == nil ) b.attacks.attackCheckTimeEngaged_sec=1.25;
  if (b.attacks.attackCheckTimeEngaged_sec < 0.1) b.attacks.attackCheckTimeEngaged_sec=0.1;

  ##weapons sanity checking
  if (contains(b, "weapons")) {
    var n=0;
    if (typeof (b.weapons) == "hash") { 
      
      foreach (elem ; keys(b.weapons)) {
        n+=1;
        if (b.weapons[elem].name == nil ) b.weapons[elem].name="Weapon " ~ n;
        if (b.weapons[elem].maxDamage_percent == nil ) b.weapons[elem].maxDamage_percent=5;
        if (b.weapons[elem].maxDamage_percent < 0) b.weapons[elem].maxDamage_percent=0;
        if (b.weapons[elem].maxDamage_percent > 100) b.weapons[elem].maxDamage_percent=100;                         
        if (b.weapons[elem].maxDamageDistance_m == nil ) b.weapons[elem].maxDamageDistance_m=500;
        if (b.weapons[elem].maxDamage_percent <= 0) b.weapons[elem].maxDamageDistance_m=1;
        if (b.weapons[elem].weaponAngle_deg.heading == nil ) b.weapons[elem].weaponAngle_deg.heading=0;
        if (b.weapons[elem].weaponAngle_deg.elevation == nil ) b.weapons[elem].weaponAngle_deg.elevation=0;
        if (b.weapons[elem].weaponOffset_m.x == nil ) b.weapons[elem].weaponOffset_m.x=0;
        if (b.weapons[elem].weaponOffset_m.y == nil ) b.weapons[elem].weaponOffset_m.y=0;  
        if (b.weapons[elem].weaponOffset_m.z == nil ) b.weapons[elem].weaponOffset_m.z=0;  
      }
    }  
  }  
                              
  b.damageLiveries.count = size (b.damageLiveries.damageLivery) ;
 
  
  #the object has stored the node telling where to store itself on the 
  # property tree . . .   
  b.objectNode.getNode("bombable/attributes",1).setValues( b );
}


#####################################################
#call to make an object bombable
#
# features/parameters are set by a bombableObject and
# a previous call to initialize (above)
var bombable_init = func(myNodeName) {

 #only allow initialization for ai & multiplayer objects
 # in FG 2.4.0 we're having trouble with strange(!?) init requests from
 # joysticks & the like  
 var init_allowed=0;
 if (find ("/ai/models/", myNodeName ) != -1 ) init_allowed=1;
 if (find ("/multiplayer/", myNodeName ) != -1 ) init_allowed=1;

 if (init_allowed!=1) {
   debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", myNodeName);
   return;
 } 

  
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
 var inited= getprop(""~myNodeName~"/bombable/bombable-initialized");
 if (inited==1) {
   debprint ("Bombable: Attempt to re-initialize bombable_init when it has not been de-initialized; abort re-initialization. ", myNodeName);
   return;
 } 
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 setprop(""~myNodeName~"/bombable/bombable-initialized", 1);



	
  debprint ("Bombable: Starting to initialize for "~myNodeName);
  if (myNodeName=="" or myNodeName==nil) {
    myNodeName=cmdarg().getPath();
    debprint ("Bombable: myNodeName blank, re-reading: "~myNodeName);
  }
    
  var node = props.globals.getNode (""~myNodeName);
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  dims= b.getNode("dimensions").getValues();
  atts= b.getNode("attacks").getValues();	
	type=node.getName();
	
  #we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	loopid=inc_loopid(myNodeName, "fire");
 	
	var objectGeoCoord = geo.Coord.new();
	var objectGeoCoordMove = geo.Coord.new();
	objectGeoCoord.set_latlon(node.getNode("position/latitude-deg").getValue(), node.getNode("position/longitude-deg").getValue(), (node.getNode("position/altitude-ft").getValue()*0.3048));
	objectGeoCoordMove.set_latlon(node.getNode("position/latitude-deg").getValue(), node.getNode("position/longitude-deg").getValue(), (node.getNode("position/altitude-ft").getValue()*0.3048));
	objectGeoCoordMove.apply_course_distance(0, dims.damageRadius_m + 200);  #we're assuming 200 meters is the max effective distance of our largest bombs
	objectGeoCoordMove.apply_course_distance(90, dims.damageRadius_m+200);
	maxLat=math.abs( objectGeoCoordMove.lat() - objectGeoCoord.lat()) ;
	maxLon=math.abs( objectGeoCoordMove.lon() - objectGeoCoord.lon()) ;
	
	#this is a kludge/optimization and obviously relationship between distance & 
	# lat/long changes depending on your location on earth.  Particularly it will 
	# malfunction badly near the north/south pole.  In those locations we'll 
	# set maxLon to a safe value. 
  # todo:make these values update every 30 seconds or so, to solve that issue
  # but still allow the optimizations. (Not an issue except near north/south 
  # poles)   	
	if (abs(node.getNode("position/latitude-deg").getValue()) > 80 ) maxLon=180; 
	
	debprint ("Bombable: maxLat = ", maxLat, " maxLon = ", maxLon);
	
  #put these in nodes also so they can be easily updated by an external
  #routine or timer
	props.globals.getNode(""~myNodeName~"/bombable/attributes/dimensions/maxLat",1).setDoubleValue( maxLat);
	props.globals.getNode(""~myNodeName~"/bombable/attributes/dimensions/maxLon",1).setDoubleValue( maxLon);
	

	
	
  #Set the listener to check for impact damage. 
  #FG aircraft use a wide variety of nodes to report impact of armament
  #So we try to check them all.  There is no real overhead to this as
  #only the one(s) active with a particular aircraft will ever get any activity.
  #This should make all aircraft in the CVS version of FG (as of Aug 2009),
  #which have armament that reports an impact, work with bombable.nas AI
  #objects.
  #
  impactReporters= [  
    "ai/models/model-impact",  #this is the FG default reporter
    "sim/armament/weapons/impact",
    "sim/ai/aircraft/impact/bullet",
    "sim/ai/aircraft/impact/gun",
    "sim/ai/aircraft/impact/cannon",
    "sim/model/bo105/weapons/impact/MG",
    "sim/model/bo105/weapons/impact/HOT",
    "sim/ai/aircraft/impact/droptank",
    "sim/ai/aircraft/impact/bomb" 
  ];
  

  var listenerids=[];
  	
  foreach (var i; impactReporters) {   
    #debprint ("i: " , i); 		
	  listenerid=setlistener(i, func ( changedImpactReporterNode ) { 
      test_impact( changedImpactReporterNode, myNodeName ); });
    append(listenerids, listenerid);
  
  }
	

	#start the loop to check for fire damage
	settimer(func{fire_loop(loopid,myNodeName);},5.0 + rand());
	
	debprint ("Bombable: Effect *bombable* loaded for "~myNodeName~" loopid="~ loopid);


    #what to do when re-set is selected
  setlistener("/sim/signals/reinit", func {
    deleteFire(myNodeName);
    deleteSmoke("damagedengine", myNodeName);
  	node.getNode("bombable/attributes/damage", 1).setDoubleValue(0);
  	setprop (""~myNodeName~"/bombable/on-ground",  0);
	  
    if (myNodeName!="") type=props.globals.getNode(myNodeName).getName();
    else type="";
    if (type=="multiplayer") mp_send_damage(myNodeName, 0);
  
    debprint ("Bombable: Damage level and smoke reset for "~ myNodeName);
  });
  
  
  if (type=="multiplayer") {
    
    #set up the mpreceive listener.  The final 0, 0) makes it
    # trigger only when the location has *changed*.  This is necessary
    # because the location is written to each frame, but only changed
    # occasionally.                 
    listenerid=setlistener(myNodeName~MP_message_pp,mpreceive, 0, 0);
    append(listenerids, listenerid);
    
    #settimer (func {mpreceive(myNodeName,loopid)}, mpTimeDelayReceive);
    debprint ("Bombable: Setup mpreceive for ", myNodeName);
             
  }
  
  
  
  props.globals.getNode(""~myNodeName~"/bombable/listenerids",1).setValues({listenerids: listenerids});
  
	
	return;
	
	
}

#####################################################
# Call to make your object stay on the ground, or at a constant
# distance above ground level--like a jeep or tank that drives along 
# the ground, or an aircraft that moves along at, say, 500 ft AGL.  
# The altitide will be continually readjusted
# as the object (set up as, say, and AI ship or aircraft moves.
# In addition, for "ships" the pitch will change to (roughly) match 
# when going up or downhill.
#
var ground_init = func( myNodeName ) {

 #only allow initialization for ai & multiplayer objects
 # in FG 2.4.0 we're having trouble with strange(!?) init requests from
 # joysticks & the like  
 var init_allowed=0;
 if (find ("/ai/models/", myNodeName ) != -1 ) init_allowed=1;
 if (find ("/multiplayer/", myNodeName ) != -1 ) init_allowed=1;

 if (init_allowed!=1) {
   debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", myNodeName);
   return;
 } 


 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
 var inited= getprop(""~myNodeName~"/bombable/ground-initialized");
 if (inited==1) {
   debprint ("Bombable: Attempt to re-initialize ground_init when it has not been de-initialized; aborting re-initialization. ", myNodeName);
   return;
 } 
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 setprop(""~myNodeName~"/bombable/ground-initialized", 1);



  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  alts= b.getNode("altitudes").getValues();	
				
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "ground");
	
  var node = props.globals.getNode(myNodeName);
  type=node.getName();
  
  #don't even try to do this to multiplayer aircraft
  if (type == "multiplayer") return;
  
	# Add some useful nodes
	
	
	
  #get the object's initial altitude
	var lat = getprop(""~myNodeName~"/position/latitude-deg");
  var lon = getprop(""~myNodeName~"/position/longitude-deg");
  var alt=elev (lat, lon);
  
  #Do some checking for the ground_loop function so we don't always have
  #to check this in that function
  #damageAltAdd is the (maximum) amount the object will descend 
  #when it is damaged.
  
	settimer(func { ground_loop(loopid, myNodeName); }, 4.0 + rand());
	
	debprint ("Bombable: Effect *maintain altitude above ground level* loaded for "~ myNodeName);
  # altitude adjustment=", alts.wheelsOnGroundAGL_ft, " max drop/fall when damaged=",
  # damageAltAdd, " loopid=", loopid);

}

#####################################################
# Call to make your object keep its location even after a re-init
# (file/reset).  For instance a fleet of tanks, cars, or ships
# will keep its position after the reset rather than returning
# to their initial position.'
#
# Put this nasal code in your object's load:
#      bombable.location_init (cmdarg().getPath())

var location_init = func(myNodeName) {

 #only allow initialization for ai & multiplayer objects
 # in FG 2.4.0 we're having trouble with strange(!?) init requests from
 # joysticks & the like  
 var init_allowed=0;
 if (find ("/ai/models/", myNodeName ) != -1 ) init_allowed=1;
 if (find ("/multiplayer/", myNodeName ) != -1 ) init_allowed=1;

 if (init_allowed!=1) {
   debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", myNodeName);
   return;
 } 
	
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
 var inited= getprop(""~myNodeName~"/bombable/location-initialized");
 if (inited==1) {
   debprint ("Bombable: Attempt to re-initialize location_init when it has not been de-initialized; aborting re-initialization. ", myNodeName);
   return;
 } 
 # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
 setprop(""~myNodeName~"/bombable/location-initialized", 1);

	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "location");
	
  var node = props.globals.getNode(myNodeName);

  type=node.getName();
  
  #don't even try to do this to multiplayer aircraft
  if (type == "multiplayer") return;
	
	settimer(func { location_loop(loopid, myNodeName); }, 15.0 + rand());

  debprint ("Bombable: Effect *relocate after reset* loaded for "~ myNodeName~ " loopid="~ loopid);

}

##########################################################
# Call to make your object turn & attack the main aircraft
#
# Put this nasal code in your object's load:
#      bombable.attack_init (cmdarg().getPath())

var attack_init = func(myNodeName) {

   #only allow initialization for ai & multiplayer objects
   # in FG 2.4.0 we're having trouble with strange(!?) init requests from
   # joysticks & the like  
   var init_allowed=0;
   if (find ("/ai/models/", myNodeName ) != -1 ) init_allowed=1;
   if (find ("/multiplayer/", myNodeName ) != -1 ) init_allowed=1;
  
   if (init_allowed!=1) {
     debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", myNodeName);
     return;
   } 


   # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
   # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
   var inited= getprop(""~myNodeName~"/bombable/attack-initialized");
   if (inited==1) {
     debprint ("Bombable: Attempt to re-initialize attack_init when it has not been de-initialized; aborting re-initialization. ", myNodeName);
     return;
   } 
   # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
   setprop(""~myNodeName~"/bombable/attack-initialized", 1);

	
	var node = props.globals.getNode(myNodeName);
  type=node.getName();
  #don't even try to do this to multiplayer aircraft
  if (type == "multiplayer") return;
	
	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid (myNodeName, "attack");
  
  var b = props.globals.getNode (""~myNodeName~"/bombable/attributes");
  var atts= b.getNode("attacks").getValues();

	attackCheckTime=atts.attackCheckTime_sec;
	if (attackCheckTime==nil or attackCheckTime<0.5)attackCheckTime=0.5;
  
  
	settimer(func { attack_loop(loopid, myNodeName,attackCheckTime); }, attackCheckTime + rand());

  debprint ("Bombable: Effect *attack* loaded for "~ myNodeName~ " loopid="~ loopid);

}

############################################################
# Call to make your object fire weapons at the main aircraft
# If the main aircraft gets in the 'fire zone' directly ahead
# of the weapons you set up, the main aircraft will be damaged
#
# Put this nasal code in your object's load:
#      bombable.weapons_init (cmdarg().getPath())

var weapons_init = func(myNodeName) {

   #only allow initialization for ai & multiplayer objects
   # in FG 2.4.0 we're having trouble with strange(!?) init requests from
   # joysticks & the like  
   var init_allowed=0;
   if (find ("/ai/models/", myNodeName ) != -1 ) init_allowed=1;
   if (find ("/multiplayer/", myNodeName ) != -1 ) init_allowed=1;
  
   if (init_allowed!=1) {
     debprint ("Bombable: Attempt to initialize a Bombable subroutine on an object that is not AI or Multiplayer; aborting initialization. ", myNodeName);
     return;
   } 

   # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
   # if it 1 and we're trying to initialize, something has gone wrong and we abort with a message.  
   var inited= getprop(""~myNodeName~"/bombable/weapons-initialized");
   if (inited==1) {
     debprint ("Bombable: Attempt to re-initialize weapons_init when it has not been de-initialized; aborting re-initialization. ", myNodeName);
     return;
   } 
   
   # set to 1 if initialized and 0 when de-inited. Nil if never before inited.
   setprop(""~myNodeName~"/bombable/weapons-initialized", 1);
  

    #overall height & width of main aircraft in meters   
    var mainAircraftSize_m = { vert : 4, horz : 8 };
    
    var aiAimFudgeFactor=22; #Our AI aircraft are somewhat terrible aimers and due to FG internal limitations they only get a chance to fire 1X per second, and on FG's schedule, not theirs.  So we give them a little fudge factor to help them have a better chance of hitting something 
                                 
    

                
    #settimer (  func { weapons_loop (myNodeName, "", vertAngle_deg, horzAngle_deg, atts.maxDamageDistance_m, atts.maxDamage_percent)}, 5);  


  	#we increment this each time we are inited or de-inited
  	#when the loopid is changed it kills the timer loops that have that id
    var loopid=inc_loopid (myNodeName, "weapons");    
    
    settimer (  func { weapons_loop (loopid, myNodeName, "", mainAircraftSize_m, aiAimFudgeFactor)}, 5 + rand());
                       
    debprint ("Bombable: Effect *weapons* loaded for ", myNodeName);
  
  
}


#####################################################
#unload function (delete/destructor) for initialize
#
#typical usage:
#<PropertyList>
#...
# <nasal>
#...
#  <unload>
#      bombable.initialize_del (cmdarg().getPath(), id);
#  </unload
# </nasal>  
#</PropertyList>
# Note: As of Bombable 3.0m, id is not used for anything
# (listenerids are stored as nodes, which works much better)

var initialize_del = func(myNodeName, id="") {
	   
  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/attributes-initialized", 0);	
	debprint ("Bombable: Effect initialize unloaded for "~ myNodeName );
	
}



#####################################################
#unload function (delete/destructor) for bombable_init
#
#typical usage:
#<PropertyList>
#...
# <nasal>
#...
#  <unload>
#      bombable.bombable_del (cmdarg().getPath(), id);
#  </unload
# </nasal>  
#</PropertyList>
# Note: As of Bombable 3.0m, id is not used for anything
# (listenerids are stored as nodes, which works much better)

var bombable_del = func(myNodeName, id="") {
	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "bomb");
	
	
	 listids = props.globals.getNode(""~myNodeName~"/bombable/listenerids",1).getValues();
	
  #remove the listener to check for impact damage				
	if (contains (listids, "listenerids")) { 
      foreach (k;listids.listenerids) { removelistener(k); }
  }
  props.globals.getNode(""~myNodeName~"/bombable/listenerids",1).removeChildren(); 
	
	#this loop will be killed when we increment loopid as well
	#settimer(func { fire_loop(loopid, myNodeName); }, 5.0+rand());
   
  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/bombable-initialized", 0);	
	debprint ("Bombable: Effect *bombable* unloaded for "~ myNodeName~ " loopid=", loopid);
	

}

#####################################################
# del/destructor function for ground_init
# Put this nasal code in your object's unload:
#      bombable.bombable_del (cmdarg().getPath());
var ground_del = func(myNodeName) {
				
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "ground");

  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/ground-initialized", 0);	
	
	debprint ("Bombable: Effect *drive on ground* unloaded for "~ myNodeName~ " loopid="~ loopid);

	
}

#####################################################
# del/destructor for location_init
# Put this nasal code in your object's unload:
#      bombable.location_del (cmdarg().getPath());

var location_del = func(myNodeName) {
	
	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "location");
	
  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/location-initialized", 0);	

  debprint ("Bombable: Effect *relocate after reset* unloaded for "~ myNodeName~ " loopid="~ loopid);

}

#####################################################
# del/destructor for attack_init
# Put this nasal code in your object's unload:
#      bombable.location_del (cmdarg().getPath());

var attack_del = func(myNodeName) {
	
	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "attack");
	
  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/attack-initialized", 0);	

  debprint ("Bombable: Effect *attack* unloaded for "~ myNodeName~ " loopid="~ loopid);

}

#####################################################
# del/destructor for weapons_init
# Put this nasal code in your object's unload:
#      bombable.location_del (cmdarg().getPath());

var weapons_del = func(myNodeName) {
	
	
	#we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid(myNodeName, "weapons");
	
  #set this to 0/false when de-inited 
  setprop(""~myNodeName~"/bombable/weapons-initialized", 0);	

  debprint ("Bombable: Effect *weapons* unloaded for "~ myNodeName~ " loopid="~ loopid);

}


var countmsg=0;


var setupBombableMenu = func {
      
      
      setprop (bomb_menu_save_lock, 1); #prevents writing these changes to the bombable menu to the save file
      #multiplayer mode enabled by default
      if (getprop (MP_share_pp) == nil )   
        props.globals.getNode(MP_share_pp, 1).setBoolValue(1);
        
        
     #fighter attack turned on by default
     if (getprop (""~bomb_menu_pp~"fighter-aircraft-attack") == nil )  
        props.globals.getNode(""~bomb_menu_pp~"fighter-aircraft-attack", 1).setIntValue(1);
        
      #make none of the easy mode, super-easy mode, realistic-impact-mode enabled by default
      if (getprop (""~bomb_menu_pp~"easy-mode") == nil )  
        props.globals.getNode(""~bomb_menu_pp~"easy-mode", 1).setIntValue(0);
      if (getprop (""~bomb_menu_pp~"super-easy-mode") == nil )
        props.globals.getNode(""~bomb_menu_pp~"super-easy-mode", 1).setIntValue(0);
      if (getprop (""~bomb_menu_pp~"realistic-impact-mode") == nil )
        props.globals.getNode(""~bomb_menu_pp~"realistic-impact-mode", 1).setIntValue(0);
      if (getprop (bomb_menu_pp~"ai-aircraft-weapons-enabled") == nil )
        props.globals.getNode(bomb_menu_pp~"ai-aircraft-weapons-enabled", 1).setIntValue(1);
      
      setprop (bomb_menu_save_lock, 0); #now any changes to bombable menu optiosn will be saved to the save file  
    #the "==nil" trick doesn't work because this io.read routine
    # leaves unchecked values as 'nil'
    # so we set our defaults first & then load the file
    #              
    var target = props.globals.getNode(""~bomb_menu_pp);
      io.read_properties(bombable_settings_file, target);
    
    
}



###########################################################
#initializers
#

#Turn fire/smoke on globally for the fire-particles system.  
#As soon as a fire-particle model is placed it will
#start burning.  To stop it from burning, simply remove the model.
#You can turn off all smoke/fires globally by setting the trigger to false 


var broadcast = nil;
var Binary = nil;
var seq = 0;
var rad2degrees=180/math.pi;
var bomb_menu_pp="/environment/bombable/menusettings/";
var bombable_settings_file=getprop("/sim/fg-home") ~ "/state/bombable-startup-settings.xml";  
var bomb_menu_save_lock="/environment/bombable/menulock/";

var trigger1_pp= ""~bomb_menu_pp~"fire-particles/";
var trigger2_pp= "-trigger";
var burning_pp= "-burning";
var life1_pp= "/environment/bombable/fire-particles/";
var life2_pp= "-life-sec";                                                       
var burntime1_pp= "/environment/bombable/fire-particles/";
var burntime2_pp= "-burn-time";
var vulnerabilities_pp = "/environment/bombable/attributes/vulnerabilities/";
var GF_damage_pp = vulnerabilities_pp ~ "gforce_damage/";
var GF_damage_menu_pp = bomb_menu_pp ~ "gforce_damage/";
     
var MP_share_pp = bomb_menu_pp~"/MP-share-events/";
var MP_broadcast_exists_pp = "/environment/bombable/mp_broadcast_exists/";
var screenHProp = nil;

var tipArgTarget=nil;  
var tipArgSelf=nil;
var currTimerTarget = 0;
var currTimerSelf = 0;

var lockNum=0;
var lockWaitTime=1;
var masterLockWaitTime=.3;
crashListener = 0;

####################################
#Set up initial variables for the mpsend/receive/queue system
#     
#The location we use for exchanging the messages
# send at MP_message_pp and receive at myNodeName~MP_message_pp
# ie, "/ai/models/multiplayer[3]"~MP_message_pp
var MP_message_pp="/sim/multiplay/generic/string[9]";
var msgTable={};
#we'll make delaySend 2X delayReceive--should make message receipt more reliable
var mpTimeDelayReceive=1/8; #delay between checking for mp messages in seconds
var mpTimeDelaySend=1/4; #delay between sending messages.
var mpsendqueue=[];
settimer (func {mpprocesssendqueue()}, 5); #wait 5 seconds before initial send

#Add damage when aircraft is accelerated beyond reasonable bounds
var damageCheckTime=1;  
settimer (func { damageCheck () }, 60); #wait 30 sec before first damage check because sometimes there is a very high transient g-force on initial startup
  
var bombableInit = func {


  debprint("Bombable: Initializing variables.");
  
  screenHProp = props.globals.getNode("/sim/startup/ysize");
  tipArgTarget = props.Node.new({ "dialog-name" : "PopTipTarget" });  
  tipArgSelf = props.Node.new({ "dialog-name" : "PopTipSelf" });
  
  #read any existing bombable-startup-settings.xml  file if it exists
  # getprop("/sim/fg-home") = fg-home directory   
  
  #for some reason this isn't work; trying a 5 sec delay to 
  # see if that fixes it.  Something is maybe coming along
  # afterward an overwriting the values?      
  settimer (setupBombableMenu, 5);
  
	#these are for the "mothership" not the AI or MP objects
  #setprop("/environment/bombable/fire-particles/fire-trigger", 0);
	#setprop("/environment/bombable/attributes/damage", 0);
	
  # Add some useful nodes
  
  setprop ("/environment/bombable/fire-particles/smoke-startsize", 11.0);
  setprop ("/environment/bombable/fire-particles/smoke-endsize", 50.0);
  setprop ("/environment/bombable/fire-particles/smoke-startsize-small", 6.5);
  setprop ("/environment/bombable/fire-particles/smoke-endsize-small", 40);
  setprop ("/environment/bombable/fire-particles/smoke-startsize-large", 26.0);
  setprop ("/environment/bombable/fire-particles/smoke-endsize-large", 150.0);
  
  props.globals.getNode(bomb_menu_pp ~ "fire-particles/fire-trigger", 1).setBoolValue(1);
	props.globals.getNode("/environment/bombable/attributes/damage", 1).setDoubleValue(0.0);
	
	if (getprop (bomb_menu_pp~"debug") == nil )
  	props.globals.getNode(bomb_menu_pp~"debug", 1).setBoolValue(0);
	
  #turn on debug flag (for testing)
	#props.globals.getNode(bomb_menu_pp~"debug", 1).setBoolValue(1);

	#set vuls for main aircraft
	vulnerabilitiesSet=getprop (""~vulnerabilities_pp~"/vulnerabilities_set");	
  if (vulnerabilitiesSet==nil or ! vulnerabilitiesSet ) setVulnerabilities ();
  
  setprop (bomb_menu_save_lock, 1);
  foreach (var smokeType; [
            ["fire",88, 3600], 
            ["jetcontrail", 77, -1],
            ["smoketrail", 55, -1],         
            ["pistonexhaust", 15, -1],
            ["damagedengine",  55, -1],
            ["flare",66,3600]
            ] ) {
  
  #trigger is the overall flag for that type of smoke/fire
  # for Bombable as a whole
  # burning is the flag as to whether that effect is turned on
  # for the main aircraft      
  # 
  # lock means we don't write these changes to the menusettings save file  
      
         props.globals.getNode(""~life1_pp~smokeType[0]~burning_pp, 1).setBoolValue(0);
         if (getprop (""~trigger1_pp~smokeType[0]~trigger2_pp) == nil )
             props.globals.getNode(""~trigger1_pp~smokeType[0]~trigger2_pp, 1).setBoolValue(1);
  	     props.globals.getNode(""~life1_pp~smokeType[0]~life2_pp, 1).setDoubleValue(smokeType[1]);
         props.globals.getNode(""~burntime1_pp~smokeType[0]~burntime2_pp, 1).setDoubleValue(smokeType[2]);
  
           
  } 

  setprop (bomb_menu_save_lock, 0);           
  init_fire_particles_dialog();
  
  #we increment this each time we are inited or de-inited
	#when the loopid is changed it kills the timer loops that have that id
	var loopid=inc_loopid("", "fire");
  settimer(func{fire_loop(loopid,"");},5.0 + rand());

  #reset damage & fires for main object
  var reset_damage_fires = func  {
  
    deleteFire("");
    deleteSmoke("damagedengine", "");
  	setprop("/environment/bombable/attributes/damage", 0);
  	setprop ("/environment/bombable/on-ground",  0 );
  	#blow away the locks for MP communication--shouldn't really
    # be needed--but just a little belt & suspendors things here
    # to make sure that no old damage (prior to the reset) is sent
    # to other aircraft again after the reset, and that none of the 
    # locks are stuck.                            
    props.globals.getNode("/environment/bombable").removeChild("locks",0);
  	
  	var msg_add="";
    var msg=reset_msg();
  	if (msg != "" and getprop(MP_share_pp) and getprop (MP_broadcast_exists_pp) ) {
      debprint ("Bombable RESET: MP sending: "~msg);
      mpsend(msg);
      msg_add=" and broadcast via multi-player";
    } 
      
      
    debprint ("Bombable: Damage level & smoke reset for main object"~msg_add);

    var msg= "Your damage reset to 0%";  
    selfStatusPopupTip (msg, 30);
  
  };
  
  #what to do when re-set is selected
  setlistener("/sim/signals/reinit", func {

    reset_damage_fires ();
    #for some reason this isn't work; trying a 5 sec delay to 
    # see if that fixes it    
    settimer (setupBombableMenu, 5);
     
  });
  
  #what to do when any bombable setting is changed
  setlistener(""~bomb_menu_pp, func {
  
    #the lock prevents the file from being written if we are setting/
    # changing menu values internally or setting menu defaults
    # We only want to save the menu properties when the **user**
    # makes changes.            
    if (!getprop(bomb_menu_save_lock))
        io.write_properties(bombable_settings_file, ""~bomb_menu_pp);
    
  },0,2);#0,0 means (0) don't do on initial startup and (2) call listener func
  # on change of any child value
  
  # action to take when main aircraft crashes (or un-crashes)
  setlistener("/sim/crashed", func {
    if (getprop("/sim/crashed")) {
    
      mp_add_damage(1,1, "crash", "You crashed!");   #adds the damage to the main aircraft
        
      debprint ("Bombable: You crashed - on fire and damage set to 100%");
      
      #experimental/doesn't quite work right yet
      #aircraftCrash(""); #Experimental!
  
    } else {
    
       debprint ("Bombable: Un-crashed--resetting damage & fires."); 
       reset_damage_fires ();
    } 
     
  });
  
  #whenever the main aircraft's damage level, fire or smoke levels are updated,
  # broadcast the updated damage level via MP, but with a delay
  # (delay is because the mp_broadcast system seems to get overwhelmed)
  # when a lot of firing is going on)
  #         
  setlistener("/environment/bombable/attributes/damage", func {
    
    settimer (func {mp_send_main_aircraft_damage_update (0)}, 4);
     
  });
  setlistener("/environment/bombable/fire-particles/fire-burning", func {
    
    settimer (func {mp_send_main_aircraft_damage_update (0)}, 3.5);
     
  });
  setlistener("/environment/bombable/fire-particles/damagedengine-burning", func {
    
    settimer (func {mp_send_main_aircraft_damage_update (0)}, 4.5);
     
  });
  
  
  print ("Bombable (ver. "~ bombableVersion ~") - bombable & fire particle effects loaded");

  #we save this for last because mp_broadcast doesn't exist for some people,
  # so runtime error & exit at this point for them.  
  
  props.globals.getNode(MP_broadcast_exists_pp, 1).setBoolValue(0);
    
  # is multiplayer enabled (overall for FG)?
  if ( getprop("/sim/multiplay/txhost") ) {
    Binary = mp_broadcast.Binary;
    print("Bombable: Bombable successfully set up for multiplayer dogfighting (enabled/disable multiplayer in the bombable menu)");
    props.globals.getNode(MP_broadcast_exists_pp, 1).setBoolValue(1);
  }
  
  #broadcast = mp_broadcast.BroadcastChannel.new(msg_channel_mpp, parse_msg, 0);
  #if (broadcast==nil) print ("Bombable: Error, mp_broadcast was not set up correctly");
  #else { 
     
  #};
  #test_msg();

} 



#we do the setlistener to wait until various things in FG are initialized
# which the functions etc in bombableInit depend on.  Then we wait an additional 15 seconds

_setlistener("/sim/signals/nasal-dir-initialized", func {

    #settimer (func {bombableInit()} , 15);
    bombableInit();

});
     
#How to copy a selected value(s) over to an MP situations:
#setlistener("/controls/engines/engine/mixture", func(n) {
#              setprop("/sim/multiplay/generic/float[0]", n.getValue());
#      });